@@ -15,21 +15,16 @@
 /*    Xavier URBAIN                                                       */
 /*                                                                        */
 /*  This software is free software; you can redistribute it and/or        */
-/*  modify it under the terms of the GNU General Public                   */
-/*  License version 2, as published by the Free Software Foundation.      */
+/*  modify it under the terms of the GNU Library General Public           */
+/*  License version 2, with the special exception on linking              */
+/*  described in file LICENSE.                                            */
 /*                                                                        */
 /*  This software is distributed in the hope that it will be useful,      */
 /*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
 /*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  */
 /*                                                                        */
-/*  See the GNU General Public License version 2 for more details         */
-/*  (enclosed in the file GPL).                                           */
-/*                                                                        */
 /**************************************************************************/
 
-// the i-th bit of x as 0 or 2^i
-//@ logic int bit(int x, int i)
-
 // the number of bits 1 in x
 //@ logic int nbits(int x)
 
@@ -37,28 +32,13 @@
 
 //@ axiom nbits_zero : nbits(0) == 0
 
-// the lowest bit 1 in x
-//@ logic int lowest_bit(int x)
-
-/*@ axiom lowest_bit_def : 
-  @   \forall int x; x != 0 => 
-  @     \exists int i; (i >= 0 && 
-  @                     lowest_bit(x) == bit(x,i) &&
-  @                     bit(x,i) != 0 &&
-  @                     \forall int j; 0 <= j < i => bit(x,j) == 0)
-  @*/
-
 /*@ axiom lowest_bit_zero :
-  @   \forall int x; lowest_bit(x) == 0 <=> x == 0
-  @*/
-
-/*@ axiom lowest_bit_trick :
-  @   \forall int x; x & -x == lowest_bit(x)
+  @   \forall int x; (x&-x) == 0 <=> x == 0
   @*/
 
 /*@ axiom remove_one_bit :
-  @    \forall int x, int i; 
-  @       bit(x,i) != 0 => nbits(x - bit(x,i)) == nbits(x) - 1
+  @    \forall int x; 
+  @       x != 0 => nbits(x - (x&-x)) == nbits(x) - 1
   @*/
 
 /*@ ensures \result == nbits(x) */
@@ -70,3 +50,9 @@ int count_bits(int x) {
   for (c = 0; d = x&-x; x -= d) c++;
   return c;
 }
+
+/*
+Local Variables: 
+compile-command: "make count_bits.gui"
+End: 
+*/
