@@ -15,16 +15,14 @@
 /*    Xavier URBAIN                                                       */
 /*                                                                        */
 /*  This software is free software; you can redistribute it and/or        */
-/*  modify it under the terms of the GNU General Public                   */
-/*  License version 2, as published by the Free Software Foundation.      */
+/*  modify it under the terms of the GNU Library General Public           */
+/*  License version 2, with the special exception on linking              */
+/*  described in file LICENSE.                                            */
 /*                                                                        */
 /*  This software is distributed in the hope that it will be useful,      */
 /*  but WITHOUT ANY WARRANTY; without even the implied warranty of        */
 /*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  */
 /*                                                                        */
-/*  See the GNU General Public License version 2 for more details         */
-/*  (enclosed in the file GPL).                                           */
-/*                                                                        */
 /**************************************************************************/
 
 //@ logic int num_of_pos(int i,int j,int t[]) reads t[i]
@@ -83,3 +81,8 @@ void m(int t[], int length) {
   }
 }
 
+/*
+Local Variables: 
+compile-command: "make muller.nosep"
+End: 
+*/
