@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cltyping.ml,v 1.125 2008/02/05 12:10:47 marche Exp $ i*)
+(*i $Id: cltyping.ml,v 1.130 2008-11-05 14:03:13 filliatr Exp $ i*)
 
 open Coptions
 open Format
@@ -183,9 +181,17 @@ let rec type_term env t =
 and type_term_node loc env = function
   | PLconstant (IntConstant _ as c) -> 
       Tconstant c, c_exact_int
-  | PLconstant (RealConstant _ as c) ->
+  | PLconstant (RealConstant s) ->
       use_floats := true;
-      Tconstant c, c_real
+      let s,fk = float_constant_type ~in_logic:true s in
+      begin match fk with
+	| Ctypes.Real -> Tconstant (RealConstant s), c_real
+	| _ ->
+	    Tunop (Ufloat_conversion,
+		     { term_node = Tconstant (RealConstant s);
+		       term_loc = loc; term_type = c_real }), 
+	    c_float fk	      
+      end
   | PLvar x ->
       let info = 
 	try Env.find x.var_name env with Not_found -> 
@@ -408,8 +414,7 @@ and type_term_node loc env = function
 	 let t = Env.find "result" env in 
 	 begin match t with
 	   | Var_info v -> Clogic.Tvar v, v.var_type
-	   | Fun_info f -> 
-               error loc ("result is a function")
+	   | Fun_info _f -> error loc ("result is a function")
 	 end
        with Not_found -> error loc "\\result meaningless")
   | PLnull ->
@@ -417,18 +422,17 @@ and type_term_node loc env = function
       Cenv.set_var_type (Var_info info) (c_void_star Not_valid) false;
       Clogic.Tvar info, (c_void_star Not_valid)
   | PLcast (ty, t) ->
+      let tty = type_logic_type loc env ty in
       let t = type_term env t in
       let tt = t.term_type in
-      begin match ty, tt.ctype_node with
-	| LTvoid, Tvoid -> 
+      begin match tty.ctype_node, tt.ctype_node with
+	| Tvoid, Tvoid -> 
 	    t.term_node, tt
-	| (LTchar _ | LTshort _ | LTint _ | LTlong _ | LTlonglong _ | LTinteger
-	  | LTfloat | LTdouble | LTlongdouble | LTreal), 
+	| (Tint _ | Tfloat _ | Tenum _), 
 	  (Tenum _ | Tint _ | Tfloat _) -> 
-	    let t = coerce (type_logic_type loc env ty) t in
-	    t.term_node, t.term_type
+	    let t = coerce tty t in t.term_node, t.term_type
 	| _ -> 
-	    warning loc "ignored cast in annotation"; t.term_node, tt
+	    Tcast(tty, t) , tty
       end
   | PLvalid _ | PLvalid_index _ | PLvalid_range _ | PLfresh _ | PLseparated _ 
   | PLexists _ | PLforall _ | PLimplies _ | PLiff _ | PLfalse
