@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: cprint.ml,v 1.50 2008/02/05 12:10:47 marche Exp $ i*)
+(*i $Id: cprint.ml,v 1.53 2008-11-05 14:03:13 filliatr Exp $ i*)
 
 (* Pretty-printer for normalized AST *)
 
@@ -39,7 +37,7 @@ open Info
 open Pp
 open Cutil
 
-let declare_struct fmt s (_,fields) =
+let declare_struct fmt s (_,_fields) =
   fprintf fmt "@[<hov 2>struct %s {@\n" s;
   begin match Cenv.tag_type_definition s with
     | Cenv.TTStructUnion(_,fields) ->
@@ -84,6 +82,7 @@ let term_binop = function
 let rec nterm fmt t = match t.nterm_node with
   | NTconstant (IntConstant s | RealConstant s) ->
       fprintf fmt "%s" s
+  | NTstring_literal s -> fprintf fmt "\"%s\"" s
   | NTvar x ->
       fprintf fmt "%s" x.var_unique_name
   | NTapp {napp_pred = li; napp_args = tl;} ->
@@ -160,7 +159,7 @@ let rec npredicate fmt p = match p.npred_node with
 	 inside a boolean formula, with the relational operator morally
 	 binding tighter than any boolean operator. *)
       fprintf fmt "@[%a %s %a@]" nterm t1 (relation rel) nterm t2
-  | NPand (p1, p2) ->
+  | NPand (_p1, _p2) ->
       (* improved printing for range inequalities, e.g. 0 <= i < 100 *)
       nconjunct fmt p
   | NPor (p1, p2) ->
@@ -353,7 +352,7 @@ and nconjunct fmt p =
   let list_conjuncts,name_map =
     List.fold_left 
       (fun (cl,m) p -> match p.npred_node with
-         | NPrel (t1,rel,t2) ->
+         | NPrel (t1,_rel,t2) ->
 	     begin match search_and_combine p t1 (* left = *)true m with
 	       | (Some cp),new_m ->
 		   (* [p] combined, do not search with [t2] *)
@@ -568,7 +567,7 @@ let rec nstatement fmt s = match s.nst_node with
       fprintf fmt "%s: %a" l.label_info_name nstatement s
   | NSgoto (status, l) ->
       fprintf fmt "goto(%s) %s" (ngoto_status status) l.label_info_name 
-  | NSswitch (e, m, l) ->
+  | NSswitch (e, _m, l) ->
       fprintf fmt "@[switch (%a) {@\n    @[%a@]@\n}@]"
 	nexpr e (print_list newline ncase) l
   | NSassert p ->
@@ -651,7 +650,7 @@ let nfile fmt p =
 
 let nfunctions fmt =
   Hashtbl.iter
-    (fun f (s, ty, fi, st,_) -> 
+    (fun _f (s, ty, fi, st,_) -> 
        match st with
 	 | Some st ->
 	     fprintf fmt "%a%a %s(@[%a@])@\n%a@\n" spec s ctype ty fi.fun_name
