@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: csymbol.ml,v 1.11 2008/02/05 12:10:47 marche Exp $ *)
+(* $Id: csymbol.ml,v 1.14 2008-11-05 14:03:14 filliatr Exp $ *)
 
 (* TO DO:
 
@@ -280,7 +278,7 @@ struct
 	  | IPnot_null_char_pointed (t1,t2) -> IPnull_char_pointed (t1,t2)
 	end
 
-  let rewrite_pred_wrt_var p v noccur = p	
+  let rewrite_pred_wrt_var p _v _noccur = p	
 
   let rec collect_predicate_vars p = match p with
     | IPfalse | IPtrue | IPany -> 
@@ -427,7 +425,7 @@ struct
     let plist = S.fold (fun e l -> e::l) pset [] in
     match plist with 
       | [] -> failwith "[make_conjunct] expecting non-empty list"
-      | p :: prest as plist -> make_sub {p with npred_node=NPtrue} plist 
+      | p :: _prest as plist -> make_sub {p with npred_node=NPtrue} plist 
 
   let subtract p1 p2 =
     let p1list = get_conjuncts p1 in
