@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ctyping.ml,v 1.155 2008/02/05 12:10:47 marche Exp $ i*)
+(*i $Id: ctyping.ml,v 1.161 2008-11-05 14:03:14 filliatr Exp $ i*)
 
 open Format
 open Coptions
@@ -84,10 +82,10 @@ let rec sizeof loc =
     | Tfloat Double -> of_int 8
     | Tfloat LongDouble -> of_int 12
     | Tfloat Real -> assert false
-    | Tvar x -> assert false (* should be expansed *)
+    | Tvar _x -> assert false (* should be expansed *)
     | Tarray (_,ty, Some e) -> 
 	mul e (sizeof ty)
-    | Tarray (_,ty, None) -> if field then of_int 0 else of_int 1
+    | Tarray (_,_ty, None) -> if field then of_int 0 else of_int 1
     | Tpointer _ -> of_int 4
     | Tstruct n ->
 	(match tag_type_definition n with
@@ -126,11 +124,11 @@ and eval_const_expr_noerror (e : texpr) = match e.texpr_node with
       Int64.mul (eval_const_expr_noerror t1)  (eval_const_expr_noerror t2)
   | TEbinary (t1, Cast.Bdiv_int _, t2) -> 
       Int64.div (eval_const_expr_noerror t1)  (eval_const_expr_noerror t2)  
-  | TEbinary (t1, Cast.Bdiv_float _, t2) -> 
+  | TEbinary (_t1, Cast.Bdiv_float _, _t2) -> 
       invalid_arg "not a constant expression 2"
   | TEcast (_, e) -> eval_const_expr_noerror e
   | TEunary (Uint_conversion, e) -> eval_const_expr_noerror e
-  | TEsizeof (t,n) -> n
+  | TEsizeof (_t,n) -> n
   | TEvar (Var_info v) ->
       if e.texpr_type.Ctypes.ctype_const 
       then v.enum_constant_value
@@ -301,14 +299,6 @@ let conversion e1 e2 =
 	coerce ty e1, coerce ty e2, ty
     | _ -> assert false
 
-let float_constant_type s =
-  let n = String.length s in
-  assert (n >= 1);
-  match s.[n-1] with
-    | 'f' | 'F' -> String.sub s 0 (n-1), Float
-    | 'l' | 'L' -> String.sub s 0 (n-1), LongDouble
-    | _ -> s, Double
-
 let has_suffix ~suf s =
   let nsuf = String.length suf in
   let n = String.length s  in
@@ -356,7 +346,7 @@ let warn_for_read_only loc e =
       Loc.report_position Coptions.log loc;
       fprintf Coptions.log "Variable %s is assigned@." x.var_name;
       set_assigned x
-  | TEvar (Fun_info f) ->
+  | TEvar (Fun_info _f) ->
       warning loc ("function assignment (ignored)")
   | _ when e.texpr_type.ctype_const ->
       warning loc "assigment of read-only location"
@@ -472,14 +462,14 @@ and type_type_node ?(ghost=false) ?(parameters=false) loc env ty =
       let fl = enum_fields Int64.zero fl in
       Env.set_enum_type loc env (Tenum x) fl 
   | CTfun (pl, tyn) ->
-      let pl = List.map (fun (ty,x) -> type_type loc env ty) pl in
+      let pl = List.map (fun (ty,_x) -> type_type loc env ty) pl in
       let pl = match pl with
 	| [{ctype_node = Tvoid}] -> []
 	| _ -> pl
       in
       Tfun (pl, type_type loc env tyn)
 
-and type_integer loc env = function
+and type_integer _loc env = function
   | Cast.Char -> Char
   | Cast.Short -> Short
   | Cast.Int -> Int 
@@ -503,7 +493,7 @@ and type_expr_node loc env = function
       TEconstant c, noattr (Tint ik)
   | CEconstant (RealConstant s) ->
       use_floats := true;
-      let s,fk = float_constant_type s in
+      let s,fk = float_constant_type ~in_logic:false s in
       TEunary (Ufloat_conversion,
 	       { texpr_node = TEconstant (RealConstant s);
 		 texpr_loc = loc; texpr_type = c_real }), 
@@ -615,7 +605,7 @@ and type_expr_node loc env = function
       end
   | CEunary (Uamp, e) ->
       (* TODO: cas o e est une fonction *)
-      let e = type_lvalue env e in
+      let e = type_lvalue ~modifiable:false env e in
       let ty = e.texpr_type in
       if is_bitfield ty then error loc "cannot take address of bitfield";
       if ty.ctype_storage = Register then 
@@ -814,7 +804,7 @@ and type_expr_node loc env = function
 		  when compatible_type e.texpr_type t 
 		    || (pointer_type t && is_null e) ->
 		  check_args (i+1) (coerce t e :: el') (el, tl)
-	      | e :: _, _ :: _ ->
+	      | _e :: _, _ :: _ ->
 		  error loc "incompatible type for argument %d" i
 	      | [], _ :: _ ->
 		  error loc "too few arguments"
@@ -840,29 +830,30 @@ and type_expr_node loc env = function
       let n = sizeof loc ty in
       TEsizeof(ty,n), c_int
 
-and type_lvalue env e = 
+and type_lvalue ?(modifiable=true) env e = 
   let loc = e.loc in
   let e = type_expr env e in
-  check_lvalue loc e;
+  check_lvalue ~modifiable loc e;
   e
 
-and check_lvalue loc e = match e.texpr_node with
-  | TEvar v -> 
-      begin 
-	match (var_type v).ctype_node with
-	  | Tarray (_,_,Some _ ) -> error loc "not a lvalue"
-	  | _ -> ()
-      end
-  | TEunary (Ustar, _)
-  | TEarrget _ 
-  | TEarrow _ 
-  | TEdot _ ->
-      (* TODO: check that e is not of enum type *)
-      ()
-  | TEcast (_, e) ->
-      check_lvalue loc e
-  | _ -> 
-      error loc "invalid lvalue"
+and check_lvalue ?(modifiable=true) loc e = 
+  match e.texpr_node with
+    | TEvar v -> 
+	begin 
+	  match (var_type v).ctype_node with
+	    | Tarray (_,_,Some _ ) when modifiable -> error loc "not a lvalue"
+	    | _ -> ()
+	end
+    | TEunary (Ustar, _)
+    | TEarrget _ 
+    | TEarrow _ 
+    | TEdot _ ->
+	(* TODO: check that e is not of enum type *)
+	()
+    | TEcast (_, e) ->
+	check_lvalue loc e
+    | _ -> 
+	error loc "invalid lvalue"
 
 and type_expr_option env eo = Option_misc.map (type_expr env) eo
 
@@ -939,7 +930,7 @@ let rec type_initializer loc env ty = function
 	     (match tag_type_definition n with
 		| TTIncomplete -> 
 		    error loc "initializer but incomplete type"
-		| TTStructUnion (Tstruct n, fl) -> 
+		| TTStructUnion (Tstruct _n, fl) -> 
 		    Ilist (type_struct_initializer loc env fl el)
 		| _ -> 
 		    assert false)
@@ -974,7 +965,7 @@ let array_size_from_initializer loc ty i = match ty.ctype_node, i with
       let s = of_int (List.length l) in
       { ty with ctype_node = Tarray (Valid(Int64.zero,s),ety, Some s) }
 
-  | Tarray (_,ety, None), None -> error loc "array size missing"  
+  | Tarray (_,_ety, None), None -> error loc "array size missing"  
 
   | Tarray (_,ety, None), Some (Iexpr e) ->
       (* since string literals in initializers are no longer transformed
@@ -1045,7 +1036,7 @@ let rec build_label_tree st acc : label_tree list =
     | CSgoto _
     | CSreturn _
     | CSannot _ -> acc
-    | CSif (e, s1, s2) ->
+    | CSif (_e, s1, s2) ->
 	let l1 = build_label_tree s1 [] in
 	let l2 = build_label_tree s2 [] in
 	(LabelBlock l1) :: (LabelBlock l2) :: acc
@@ -1315,7 +1306,7 @@ let type_spec_decl loc = function
       Taxiom (id, type_predicate (Env.empty ()) p)
   | LDinvariant (id, p) -> 
       Tinvariant (id, type_predicate (Env.empty ()) p)
-  | LDlogic (id, ty, labels, pl, ll) ->
+  | LDlogic (id, ty, _labels, pl, ll) ->
       let ty = type_logic_type loc (Env.empty ()) ty in
       let pl,env' = type_logic_parameters loc (Env.empty ()) pl in
       id.logic_args <- List.map fst pl;
@@ -1323,7 +1314,7 @@ let type_spec_decl loc = function
       let ll = List.map (type_location env') ll in
       Cenv.add_logic id.logic_name (List.map snd pl, ty, id);
       Tlogic (id, Function (pl, ty, ll))
-  | LDlogic_def (id, ty, labels, pl, t) ->
+  | LDlogic_def (id, ty, _labels, pl, t) ->
       let ty = type_logic_type loc (Env.empty ()) ty in
       let pl,env' = type_logic_parameters loc (Env.empty ()) pl in
       id.logic_args <- List.map fst pl;
@@ -1460,7 +1451,7 @@ let type_decl d = match d.node with
 		    try
 		      set_const_value info (eval_const_expr_noerror e)
 		    with
-			Invalid_argument msg -> ()	
+			Invalid_argument _msg -> ()	
 		  end
 	      | _ ->
 		  ()
