@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: info.ml,v 1.48 2008/05/28 14:53:34 marche Exp $ i*)
+(*i $Id: info.ml,v 1.51 2008-11-05 14:03:14 filliatr Exp $ i*)
 
 open Ctypes
 open Creport
@@ -81,7 +79,7 @@ let rec same_why_type wt1 wt2 =
 
 let rec same_why_type_no_zone wt1 wt2 =
   match wt1, wt2 with
-    | Pointer z1, Pointer z2 -> true
+    | Pointer _z1, Pointer _z2 -> true
     | Memory (a1,_), Memory (a2,_) ->
 	same_why_type_no_zone a1 a2
     | Int, Int -> true
