@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: invariant.ml,v 1.49 2008/02/05 12:10:48 marche Exp $ i*)
+(*i $Id: invariant.ml,v 1.52 2008-11-05 14:03:14 filliatr Exp $ i*)
 
 open Coptions
 open Creport
@@ -71,14 +69,14 @@ let find_pred x = snd (find_pred x)
   
 let rec predicate_for name t =
   match t.nterm_type.ctype_node with
-    | Tstruct (n) ->
+    | Tstruct _n ->
 	(*	NPand *)
 	npvalid t
 	  (*,
  	    NPapp (find_pred ("valid_" ^ n), [t]))*)
-    | Tarray (_,ty, None) ->
+    | Tarray (_,_ty, None) ->
 	error Loc.dummy_position "array size missing in `%s'" name
-    | Tarray (_,ty, Some s) ->
+    | Tarray (_,_ty, Some s) ->
 	  let i = default_var_info "counter" in
 	  set_var_type (Var_info i) c_int false; 
 	  let vari = noattr_term c_int (NTvar i) in
@@ -127,17 +125,17 @@ let make_forall_range loc t b f =
 			nprel (vari, Lt, int_nconstant (Int64.to_string b))) in
       make_forall [c_int, i] (make_implies ineq pred)
      
-let rec tab_struct loc v1 v2 s ty n n1 n2=
+let rec tab_struct loc v1 v2 s ty _n n1 n2=
   (make_forall_range loc v2 s 
-     (fun t i -> 
+     (fun t _i -> 
 	local_separation loc n1 v1 (n2^"[i]") (indirection loc ty t)))
 
 and local_separation loc n1 v1 n2 v2 =
   match (v1.nterm_type.Ctypes.ctype_node,v2.nterm_type.Ctypes.ctype_node) 
   with
-    | Tarray (_,ty, None), _ ->
+    | Tarray (_,_ty, None), _ ->
 	error loc "array size missing in `%s'" n1
-    | _, Tarray (_,ty, None) ->
+    | _, Tarray (_,_ty, None) ->
 	error loc "array size missing in `%s'" n2
     | Tstruct n , Tarray (_,ty,Some s) -> tab_struct loc v1 v2 s ty n n1 n2
     | Tarray (_,ty,Some s) , Tstruct n -> tab_struct loc v2 v1 s ty n n1 n2
@@ -150,10 +148,10 @@ and local_separation loc n1 v1 n2 v2 =
 	     nptrue)
 	  (make_and 
 	     (make_forall_range loc v1 s1 
-		(fun t i -> local_separation loc (n1^"[i]") 
+		(fun t _i -> local_separation loc (n1^"[i]") 
 		     (indirection loc ty1 t) n2 v2))
 	     (make_forall_range loc v2 s2  
-		(fun t i -> local_separation loc n1 v1 (n2^"[j]") 
+		(fun t _i -> local_separation loc n1 v1 (n2^"[j]") 
 		     (indirection loc ty2 t))))
     | _, _ -> nptrue
 
@@ -189,7 +187,7 @@ let rec separation_intern2  n1 v1 =
 				 make_implies (nprel (i1, Neq, i2)) 
 				   (not_alias Loc.dummy_position t1 t2))))
 		    (make_forall_range Loc.dummy_position v1 s 
-		       (fun t i -> 
+		       (fun t _i -> 
 			  separation_intern2 n1 
 			    (noattr_term ty t.nterm_node)))    
 	      | _ -> nptrue
@@ -324,7 +322,7 @@ let separation_first mark diag v1 v2 =
 		       (NTconstant (IntConstant 
 				      ((Int64.to_string s))))])))]
 	else []
-    | Tarray (_,ty1, Some s1),  Tarray(_,ty2, Some s2) ->
+    | Tarray (_,ty1, Some _s1),  Tarray(_,ty2, Some _s2) ->
 	let make_sub_term p ty v = 
 	  let zone = find_zone_for_term p in
 	  let () = type_why_new_zone zone v in
@@ -388,40 +386,20 @@ let separation_first mark diag v1 v2 =
 	  
 
 let rec separation_intern n =
-  let l =
-    begin
-      match  tag_type_definition n with
-	| TTStructUnion ((Tstruct _),fl) ->
-	    fl
-	| TTStructUnion (t,fl) -> 
-	    begin match t with
-	      | Tstruct _ -> assert false
-	      | Tfun (_, _) -> assert false
-	      | Tenum _ -> assert false
-	      | Tunion _ -> assert false
-	      | Tpointer (_, _) -> assert false
-	      | Tarray (_, _, _) -> assert false
-	      | Ctypes.Tvar _ -> assert false
-	      | Tfloat _ -> assert false
-	      | Tint _ -> assert false
-	      | Tvoid -> assert false
-	    end
-	| TTEnum (_, _) -> assert false
-	| TTIncomplete -> assert false
-    end  
-  in
-  let array_intern_separation v1  =
-    let n1 = v1.var_unique_name in
-    match v1.var_type.Ctypes.ctype_node with
-      | Tarray (_,_,None) -> 
-	  error Loc.dummy_position "array size missing in `%s[i]'" n1
-      | Tarray (_,ty,Some s) ->
-	  begin
-	    match ty.Ctypes.ctype_node with
-	      | Tstruct _ -> 
-		  let pre = "%separation1_range_" ^ n1  in 
-		  let info = Info.default_logic_info (pre) in
-		  info.logic_heap_args <- single v1; 
+  match  tag_type_definition n with
+    | TTStructUnion ((Tstruct _),l) ->
+	let array_intern_separation v1  =
+	  let n1 = v1.var_unique_name in
+	  match v1.var_type.Ctypes.ctype_node with
+	    | Tarray (_,_,None) -> 
+		error Loc.dummy_position "array size missing in `%s[i]'" n1
+	    | Tarray (_,ty,Some s) ->
+		begin
+		  match ty.Ctypes.ctype_node with
+		    | Tstruct _ -> 
+			let pre = "%separation1_range_" ^ n1  in 
+			let info = Info.default_logic_info (pre) in
+			info.logic_heap_args <- single v1; 
 		  Cenv.add_pred (pre)  ([], info);
 		  [noattr_located (Cast.Ninvariant_strong (
 				    "internal_separation_" ^ n1 ^ "_array1" , 
@@ -460,13 +438,28 @@ let rec separation_intern n =
 	      | _ -> []
 	  end
       | _ -> []
-  in
-  (List.fold_left (fun acc t ->
-     array_intern_separation t@acc) [] l) @ 
-    (fold_2 (separation_first true false ) l)  
+	in
+	(List.fold_left (fun acc t ->
+			   array_intern_separation t@acc) [] l) @ 
+	  (fold_2 (separation_first true false ) l)  
+    | TTStructUnion (t,_fl) -> 
+	begin match t with
+	  | Tstruct _ -> assert false
+	  | Tfun (_, _) -> assert false
+	  | Tenum _ -> assert false
+	  | Tunion _ -> [] (* TODO ? *)
+	  | Tpointer (_, _) -> assert false
+	  | Tarray (_, _, _) -> assert false
+	  | Ctypes.Tvar _ -> assert false
+	  | Tfloat _ -> assert false
+	  | Tint _ -> assert false
+	  | Tvoid -> assert false
+	end
+    | TTEnum (_, _) -> assert false
+    | TTIncomplete -> assert false
 
 
-let separation_2_struct s1 l1 s2 l2 acc=
+let separation_2_struct _s1 l1 _s2 l2 acc=
   let l1 = snd l1 in
   let l2 = snd l2 in
   List.fold_left (fun acc1 t1 ->
@@ -476,7 +469,7 @@ let separation_2_struct s1 l1 s2 l2 acc=
     acc l1
 
 let add_predicates l =
-  let f s (ty,fl) l2 = 
+  let f s (_ty,fl) l2 = 
     let l2 = List.fold_right 
       (fun f acc -> 
 	 begin
@@ -511,10 +504,10 @@ let add_predicates l =
 				   (IntConstant (Int64.to_string s)))])))::
 		   begin
 		     match ty.ctype_node with
-		       | Tarray (_,ty, None)->
+		       | Tarray (_,_ty, None)->
 			   error Loc.dummy_position 
 			     "array size missing in `%s'" f.var_name
-		       | Tarray (_,ty, Some s1) ->
+		       | Tarray (_,ty, Some _s1) ->
 			 [noattr_located (
 			    Cast.Ninvariant_strong 
 			      ("valid_matrice" ^ n1,
