@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 open Logic
@@ -62,15 +60,11 @@ let print_term fmt t =
 	fprintf fmt "%b" b
     | Tconst ConstUnit -> 
 	fprintf fmt "void" 
-    | Tconst (ConstFloat (i,f,e)) -> 
-	let f = if f = "0" then "" else f in
-	let e = (if e = "" then 0 else int_of_string e) - String.length f in
-	if e = 0 then
-	  fprintf fmt "%s%s" i f
-	else if e > 0 then
-	  fprintf fmt "(%s%s * 1%s)" i f (String.make e '0')
-	else
-	  fprintf fmt "(%s%s / 1%s)" i f (String.make (-e) '0')
+    | Tconst (ConstFloat (i,f,e)) ->
+	if e = "" then
+	  fprintf fmt "%s.%s" i f
+	else 
+	  fprintf fmt "(%s.%se%s)" i f e  
     | Tvar id when id == implicit ->
 	fprintf fmt "<?>" (* should not happen *)
     | Tvar id when id == t_zwf_zero ->
