@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 open Logic
@@ -40,16 +38,16 @@ let print_term fmt t =
   let rec print0 fmt t =
     print1 fmt t
   and print1 fmt = function
-    | Tapp (id, [a;b], _) when id == t_add_int ->
+    | Tapp (id, [a;b], _) when id == t_add_int || id == t_add_real ->
 	fprintf fmt "%a +@ %a" print1 a print2 b
-    | Tapp (id, [a;b], _) when id == t_sub_int ->
+    | Tapp (id, [a;b], _) when id == t_sub_int || id == t_sub_real ->
 	fprintf fmt "%a -@ %a" print1 a print2 b
     | t ->
 	print2 fmt t
   and print2 fmt = function
-    | Tapp (id, [a;b], _) when id == t_mul_int ->
+    | Tapp (id, [a;b], _) when id == t_mul_int || id == t_mul_real ->
 	fprintf fmt "%a *@ %a" print2 a print3 b
-    | Tapp (id, [a;b], _) when id == t_div_int ->
+    | Tapp (id, [a;b], _) when id == t_div_int || id == t_div_real ->
 	fprintf fmt "@[%a /@ %a@]" print2 a print3 b
     | Tapp (id, [a;b], _) when id == t_mod_int ->
 	fprintf fmt "@[%a@ %%@ %a@]" print2 a print3 b
@@ -63,14 +61,10 @@ let print_term fmt t =
     | Tconst ConstUnit -> 
 	fprintf fmt "tt" 
     | Tconst (ConstFloat (i,f,e)) -> 
-	let f = if f = "0" then "" else f in
-	let e = (if e = "" then 0 else int_of_string e) - String.length f in
-	if e = 0 then
-	  fprintf fmt "%s%s" i f
-	else if e > 0 then
-	  fprintf fmt "(%s%s * 1%s)" i f (String.make e '0')
-	else
-	  fprintf fmt "(%s%s / 1%s)" i f (String.make (-e) '0')
+	if e = "" then
+	  fprintf fmt "%s.%s" i f
+	else 
+	  fprintf fmt "(%s.%se%s)" i f e  
     | Tvar id when id == implicit ->
 	fprintf fmt "<?>"
     | Tvar id when id == t_zwf_zero ->
