@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 open Options
@@ -47,7 +45,7 @@ let pwd = Sys.getcwd ()
 let source = ref ""
 let tv_source = ref (GText.view ())
 
-let active = ref true
+let active = ref false
 let desactivate () = active := false
 let activate () = active := true
 let swap_active () = active := not !active
@@ -169,7 +167,7 @@ let move_to_source = function
 	  b#set_text (banner());
 	  begin
 	    try
-	      let why_logo_image = Options.lib_file "why-logo-1.png" in
+	      let why_logo_image = Filename.concat Version.libdir "why-logo-1.png" in
 	      let why_logo = GdkPixbuf.from_file why_logo_image in
 	      b#insert_pixbuf ~iter:b#start_iter ~pixbuf:why_logo 
 	    with _ -> 
@@ -304,12 +302,12 @@ let print_all (tbuf:GText.buffer) s p =
   let (_,concl) = p in
   let mytag = 
     tbuf#create_tag
-      [`UNDERLINE `DOUBLE;
+      [`UNDERLINE `SINGLE;
        `FOREGROUND (get_fc "separator"); 
        `BACKGROUND (get_bc "separator")] 
   in
   tbuf#insert ~tags:[mytag] ~iter:tbuf#end_iter 
-    "_                                                        _\n\n";
+    "              \n\n";
   let conclusion = 
     let print_predicate = 
       if is_active () 
