@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: stat.ml,v 1.78 2008/04/15 09:00:07 filliatr Exp $ i*)
+(*i $Id: stat.ml,v 1.91 2008-11-05 14:03:14 filliatr Exp $ i*)
 
 open Printf
 open Options
@@ -44,6 +42,7 @@ let is_caduceus =
   List.exists (fun f -> Filename.basename f = "caduceus.why") Options.files 
 *)
 
+(*
 let get_files fl = 
   let files = 
     List.map (fun f -> Filename.chop_extension (Filename.basename f)) fl 
@@ -53,10 +52,13 @@ let get_files fl =
        let file = (Filename.concat (Sys.getcwd ()) f) ^ ".c" in
        if Sys.file_exists file then file :: l else l)
     fl files
+*)
 
 let () =
   try
-    Main.main ()
+    Format.eprintf "Computation of VCs...@.";
+    Main.main ();
+    Format.eprintf "Computation of VCs done.@."
   with e ->
     Report.explain_exception Format.err_formatter e;
     Format.pp_print_newline Format.err_formatter ();
@@ -109,10 +111,12 @@ let set_loaded_config () =
   Colors.set_all_colors (Config.get_colors ())
 
 let () =
+  Format.eprintf "Reading GWhy configuration...@.";
   ignore (GtkMain.Main.init ());
   Config.create_default_config ();
   Config.load ();
-  set_loaded_config ()
+  set_loaded_config ();
+  Format.eprintf "GWhy configuration loaded...@."
 
 let modifiable_font_views = ref []
 
@@ -129,8 +133,8 @@ let enlarge_font () =
 let reduce_font () =
   decr Colors.font_size; change_font ()
 
-let display_info = ref (function s -> failwith "not ready")
-let flash_info = ref (function s -> failwith "not ready")
+let display_info = ref (function _s -> failwith "not ready")
+let flash_info = ref (function _s -> failwith "not ready")
 
 let cache_check = ref (GButton.toggle_button ())
 let hard_proof_check = ref (GButton.toggle_button ())
@@ -159,25 +163,43 @@ module View = struct
   open GtkTree
   open Model
 
+  let () = 
+    Format.eprintf "Creating GWhy Tree view...@."
+
   let renderer = GTree.cell_renderer_text [`XALIGN 0.] 
   let first_col = GTree.view_column ~title:"Proof obligations " 
     ~renderer:(renderer, ["text", Model.name]) () 
- 
+  let last_col = GTree.view_column ~title:"Statistics" 
+    ~renderer:(renderer, ["text", Model.stat]) ()
 
-  let add_columns ~(view : GTree.view) ~model =
+  let () = 
+    first_col#set_resizable true;
+    first_col#set_max_width 400;
+    last_col#set_resizable true
+ 
+  let add_columns ~(view : GTree.view) ~_model =
     (* let renderer = GTree.cell_renderer_text [`XALIGN 0.] in *)
     let icon_renderer = GTree.cell_renderer_pixbuf [ `STOCK_SIZE `BUTTON ] in
     let _n : int = view#append_column first_col in
-    List.map
-      (fun p ->
-	 let vc = 
-	   GTree.view_column ~title:p.pr_name 
-	     ~renderer:(icon_renderer, ["stock_id", p.pr_icon]) ()
-	 in
-	 vc#set_clickable true;
-	 let _n : int = view#append_column vc in
-	 p, vc)
-      (Model.get_provers ())
+    let l = 
+      List.map
+	(fun p ->
+	   let vc = 
+	     GTree.view_column ~title:(prover_name_with_version_and_enc p) 
+	       ~renderer:(icon_renderer, ["stock_id", p.pr_icon]) ()
+	   in
+	   vc#set_resizable true;
+	   p.pr_viewcol <- Some vc;
+	   if p.pr_info.DpConfig.version <> "" then vc#set_clickable true;
+	   let _n : int = view#append_column vc in
+	   p, vc)
+	(Model.get_provers ())
+    in
+    let _n : int = view#append_column last_col 
+    in l
+
+  let () = 
+    Format.eprintf "GWhy Tree view created...@.";
 
 end
 
@@ -188,7 +210,7 @@ let update_buffer tv =
 
 open Logic_decl
 
-let show_xpl (loc,xpl) (tv:GText.view) =
+let show_xpl (loc,xpl) (_tv:GText.view) =
   match loc with
     | (s,l,b,e) ->
 	let loc = 
@@ -217,7 +239,7 @@ let select_obligs (model:GTree.tree_store) (tv:GText.view)
 	 Pprinter.text_of_obligation tv o;
 	 let mark = `MARK (tv#buffer#create_mark tv#buffer#end_iter) in
 	 tv#scroll_to_mark ~use_align:true mark;
-	 show_xpl (loc,xpl) tv_s  
+	 show_xpl (loc,xpl) tv_s
        with Not_found -> 
 	 try
 	   let (id,beh,(f,l,b,e)) = Hashtbl.find Util.program_locs s in
@@ -262,16 +284,19 @@ let update_statistics (model:GTree.tree_store) row f children =
   let statistics = get_statistics model row in
 *)
   let total = string_of_int (get_all_results f model) in
+(*
   let name =
     if f="" then "User goals" else
     try
-      let (id,beh,(f,l,b,e)) = Hashtbl.find Util.program_locs f in
-      beh ^ " of " ^ id
+      let (id,beh,(_f,_l,_b,_e)) = Hashtbl.find Util.program_locs f in
+      beh ^ "\nof " ^ id
     with Not_found -> "unknown function " ^ f
   in
   model#set 
-    ~row ~column:Model.name (name^" "^" ("^total^"/"^(string_of_int children)^")")
-
+    ~row ~column:Model.name name
+*)
+  model#set 
+    ~row ~column:Model.stat (total^"/"^(string_of_int children))
 
 let build_statistics (model:GTree.tree_store) f = 
   try
@@ -332,7 +357,7 @@ let try_proof oblig =
 (* 
  * run a prover on an obligation and update the model 
  *)
-let run_prover_child p (view:GTree.view) (model:GTree.tree_store) 
+let run_prover_child p (_view:GTree.view) (model:GTree.tree_store) 
                      o bench alone = 
   let column_p = p.Model.pr_icon in
   let (_, _, oblig, seq) = o in
@@ -355,7 +380,7 @@ let run_prover_child p (view:GTree.view) (model:GTree.tree_store)
       in
       let get_result = function
 	| Calldp.Valid _ -> 
-	    Cache.add seq p.Model.pr_name;
+	    Cache.add seq (Model.prover_id p);
 	    model#set ~row ~column:column_p `YES ; 1
 	| Calldp.Timeout _ -> 
 	    model#set ~row ~column:column_p `CUT; 0
@@ -479,6 +504,7 @@ let run_benchmark (view:GTree.view) (model:GTree.tree_store) () =
   Queue.iter (fun f -> run_benchmark_fct view model f ()) Model.fq
 
 let main () = 
+  Format.eprintf "Creating GWhy views...@.";
   let w = GWindow.window 
 	    ~allow_grow:true ~allow_shrink:true
 	    ~width:!Colors.window_width ~height:!Colors.window_height 
@@ -509,11 +535,13 @@ let main () =
   let accel_group = factory#accel_group in
   let file_menu = factory#add_submenu "_File" in
   let file_factory = new GMenu.factory file_menu ~accel_group in
+  (*
   let _ = 
     file_factory#add_image_item (*~stock:`REFRESH*) ~label:"_Refresh"
       ~key:GdkKeysyms._R () 
   in
   let _ = file_factory#add_separator () in
+  *)
   let _ = 
     file_factory#add_image_item ~key:GdkKeysyms._Q ~label:"_Quit" 
       ~callback:(fun () -> exit 0) () 
@@ -543,15 +571,20 @@ let main () =
 
   (* horizontal paned *)
   let hp = GPack.paned `HORIZONTAL  ~border_width:3 ~packing:vbox#add () in
+(*
   let vtable = 
     GPack.table ~row_spacings:5 ~homogeneous:false ~packing:hp#add () 
   in
+*)
+(*
   let table = GPack.table ~col_spacings:15 ~homogeneous:false () in
   let _ = vtable#attach ~left:0 ~top:0 table#coerce in
+*)
 
   let model = Model.create_model () in
 
   (* function list *)
+(*
   let files = List.map 
     (fun f -> 
        if Filename.is_relative f 
@@ -567,20 +600,25 @@ let main () =
   in
   table#attach ~left:1 ~top:0 ~expand:`BOTH files_combo#coerce;
   let _ = files_combo#entry#set_editable false in
+*)
   
   (* left tree of proof obligations *)
   let scrollview = 
     GBin.scrolled_window ~hpolicy:`AUTOMATIC ~vpolicy:`AUTOMATIC 
-    ~width:(!Colors.window_width / 2) () 
+    ~width:(!Colors.window_width / 2) ~packing:hp#add () 
   in
+  let _ = scrollview#set_shadow_type `ETCHED_OUT in
+(*
   let _ = vtable#attach ~left:0 ~top:1 ~expand:`BOTH scrollview#coerce in
-  let view = GTree.view ~model ~packing:scrollview#add_with_viewport () in
+*)
+
+  let view = GTree.view ~model ~packing:scrollview#add () in
   (* has effect but not nice 
      let () = view#misc#modify_font !general_font in
   *)
   let _ = view#selection#set_mode `SINGLE in
   let _ = view#set_rules_hint true in
-  let vc_provers = View.add_columns ~view ~model in
+  let vc_provers = View.add_columns ~view ~_model:model in
   
   (* cache and view menu *)
   let _ = configuration_factory#add_separator () in
@@ -632,7 +670,7 @@ let main () =
     List.iter
       (fun p -> 
 	 let m = configuration_factory#add_check_item 
-	   ~active:(List.mem p (Model.get_provers ())) p.Model.pr_name
+	   ~active:(List.mem p (Model.get_provers ())) (Model.prover_id p)
 	 in 
 	 ignore(m#connect#toggled  
 		  ~callback:(fun () -> 
@@ -732,22 +770,23 @@ let main () =
   (* menus for povers *)
   let _ = proof_factory#add_separator ()  in
   let select_prover p = 
-    (try !flash_info (p.Model.pr_name ^" selected for default mode !")
+    (try !flash_info ((Model.prover_id p) ^" selected for default mode !")
     with _ -> ());
     Model.set_prover p
   in
   let provers_m = 
-    let name = (Model.get_default_prover ()).Model.pr_name
+    let name = Model.prover_id (Model.get_default_prover ())
     and pp = List.hd (Model.get_provers ()) in
-    let fm = proof_factory#add_radio_item ~active:(name = pp.Model.pr_name) 
-      pp.Model.pr_name in
+    let n = Model.prover_id pp in
+    let fm = proof_factory#add_radio_item ~active:(name = n) n in
     ignore(fm#connect#toggled  
 	     ~callback:(fun () -> select_prover pp));
     let group = fm#group in
     (pp, fm) :: List.map
       (fun p -> 
+	 let n = Model.prover_id p in
 	 let m = proof_factory#add_radio_item 
-	   ~active:(name = p.Model.pr_name) ~group p.Model.pr_name 
+	   ~active:(name = n) ~group n 
 	 in 
 	 ignore(m#connect#toggled  
 		  ~callback:(fun () -> select_prover p));
@@ -800,7 +839,7 @@ let main () =
   let _ = view#misc#connect#realize ~callback:view#expand_all in
 
   (* vertical paned *)
-  let hb = GPack.paned `VERTICAL  ~border_width:3 ~packing:hp#add2 () in
+  let hb = GPack.paned `VERTICAL  ~border_width:0 ~packing:hp#add2 () in
   let _ = hb#misc#connect#realize
 	  ~callback:(fun _ ->hb#set_position (!Colors.window_height*6/10 ) ) in
   let _ = hb#set_position (!Colors.window_height*9/10 ) in
@@ -842,7 +881,7 @@ let main () =
   let _ = GMisc.label ~text:" Timeout" ~xalign:0. ~packing:hbox#pack () in
   let timeout_b = GEdit.spin_button ~digits:0 ~packing:hbox#pack () in
   timeout_spin := timeout_b;
-  timeout_b#adjustment#set_bounds ~lower:1. ~upper:999. ~step_incr:1. ();
+  timeout_b#adjustment#set_bounds ~lower:1. ~upper:max_float ~step_incr:1. ();
   timeout_b#adjustment#set_value (float_of_int (Tools.get_timeout ()));
   let _ = 
     timeout_b#connect#value_changed ~callback:
@@ -912,14 +951,16 @@ let main () =
   (*
    * function selection 
    *)
+(*
   let _ =
     files_combo#entry#event#connect#after#focus_in ~callback:
-      begin fun ev -> 
+      begin fun _ev -> 
 	Pprinter.move_to_source None;
 	Pprinter.reset_last_file (); (* ?? pourquoi ?? *)
 	false
       end
   in
+*)
 
   (*
    * Running obligation 
@@ -928,11 +969,22 @@ let main () =
     (view#connect#row_activated ~callback:
        (fun p col -> 
 	  let row = model#get_iter p in
-          let name = col#title in
           let prover = 
+	    match List.filter 
+	      (fun p -> 
+		 match p.Model.pr_viewcol with
+		   | None -> assert false
+		   | Some c -> c#title = col#title
+	       (* WHY c == col is not enough ???? *)) 
+	      (Model.get_provers ()) with
+		| [p] -> p
+		| [] -> assert false
+		| _ -> assert false
+(*
             try Model.get_prover name 
             with Model.No_such_prover -> Model.get_default_prover ()
-          in
+*)
+         in
 	  let s = model#get ~row ~column:Model.fullname in
 	  (if model#iter_has_child row then
 	     prove (run_prover_fct prover 
@@ -968,7 +1020,7 @@ let main () =
 		     Hashtbl.iter
 		       (fun p m -> 
 			  Buffer.add_string buffer 
-			    (p.Model.pr_name ^ ": \n" ^ m ^" \n\n"))
+			    (Model.prover_id p ^ ": \n" ^ m ^" \n\n"))
 		       failed_with;
 		     let tbuf = GText.buffer () in
 		     tv2#set_buffer tbuf;
@@ -996,7 +1048,7 @@ let main () =
     load_cache "gwhy.cache";
     if not (Cache.is_empty ()) then 
       Hashtbl.iter 
-	(fun s (_,_,o,seq) -> 
+	(fun _s (_,_,o,seq) -> 
 	   let cleaned = Cache.clean seq in
 	   if in_cache cleaned then begin 
 	     let row = Hashtbl.find Model.orows o in
@@ -1033,11 +1085,20 @@ let main () =
       Model.frows
   in
   w#add_accel_group accel_group;
-  w#show ()
+  w#show ();
+  Format.eprintf "GWhy views created.@."
+
 
 
 (* Main *)
 let _ = 
+  begin
+    try
+      DpConfig.load_rc_file ()
+    with Not_found ->
+      print_endline "Why config file not found, please run why-config first.";
+      exit 1
+  end;
 (*
   if not is_caduceus then Pprinter.desactivate ();
 *)
