@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 (*
@@ -75,12 +73,17 @@ let rec name_type t =
     | JTYlogic _ -> assert false
 
 let rec intro_array_struct t =
+  let n = name_type t in 
   try
-    let _ = Hashtbl.find array_struct_table t in ()
+    let _ = Hashtbl.find array_struct_table n in ()
   with Not_found ->
-    let n = name_type t in 
-    Java_options.lprintf "Adding array struct for type %s@." n;
-    Hashtbl.add array_struct_table t (n ^ "M", n ^ "P")
+    Java_options.lprintf "Adding array struct for type %a under name %s@." 
+      Java_typing.print_type t n;
+      (* array structs indexed by name rather than by type,
+	 to void generation of two structs for nullable and non_null types
+	 of the same name.
+	 - Nicolas R. *)
+      Hashtbl.add array_struct_table n (t, n ^ "M", n ^ "P")
 
 let term t = () (* TODO *)
 
@@ -152,31 +155,41 @@ let switch_label = function
   | Java_ast.Default -> ()
   | Java_ast.Case e -> expr e
   
+let behavior (id,assumes,throws,assigns,ensures) =
+  Option_misc.iter assertion assumes;
+  Option_misc.iter (fun (_,l) -> List.iter term l) assigns;
+  assertion ensures
+
+let loop_annot annot =
+  assertion annot.loop_inv;
+  List.iter (fun (_,a) -> assertion a) annot.behs_loop_inv;
+  Option_misc.iter term annot.loop_var
+
 let rec statement s =
   match s.java_statement_node with
     | JSskip 
     | JSreturn_void
-    | JSbreak _ -> ()
+    | JSbreak _ | JScontinue _ -> ()
     | JSblock l -> List.iter statement l	  
     | JSvar_decl (vi, init, s) ->
 	Option_misc.iter do_initializer init;
 	statement s
     | JSif (e, s1, s2) -> expr e; statement s1; statement s2
-    | JSwhile(e,inv,dec,s) ->
-	  expr e; assertion inv; term dec; statement s
-    | JSfor (el1, e, inv, dec, el2, body) ->
+    | JSdo (s, annot, e) ->
+	statement s; loop_annot annot; expr e
+    | JSwhile(e,annot,s) ->
+	  expr e; loop_annot annot; statement s
+    | JSfor (el1, e, annot, el2, body) ->
 	List.iter expr el1;
 	expr e;
-	assertion inv;
-	term dec;
+	loop_annot annot;
 	List.iter expr el2;
 	statement body
-    | JSfor_decl(decls,e,inv,dec,sl,body) ->
+    | JSfor_decl(decls,e,annot,sl,body) ->
 	List.iter 
 	  (fun (_,init) -> Option_misc.iter do_initializer init) decls;
 	expr e;
-	assertion inv;
-	term dec;
+	loop_annot annot;
 	List.iter expr sl;
 	statement body
     | JSthrow e
@@ -193,6 +206,11 @@ let rec statement s =
 	List.iter statement s;
 	List.iter (fun (_,s) -> List.iter statement s) catches;
 	Option_misc.iter (List.iter statement) finally
+    | JSstatement_spec(req,dec,behs,s) ->
+	Option_misc.iter assertion req;
+	Option_misc.iter term dec;
+	List.iter behavior behs;
+	statement s
 
 let param vi =
   match vi.java_var_info_type with
