@@ -15,23 +15,21 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 (***************************************************************************
 
 Abstract syntax trees for Java source files
 
-$Id: java_ast.mli,v 1.36 2008-05-23 13:51:39 marche Exp $
+$Id: java_ast.mli,v 1.43 2008-12-09 09:14:18 marche Exp $
 
 ***************************************************************************)
 
@@ -107,7 +105,7 @@ and pexpr_node =
   | JPEif of pexpr * pexpr * pexpr
   | JPEthis
   | JPEfield_access of java_field_access
-  | JPEcall_name of qualified_ident * pexpr list
+  | JPEcall_name of qualified_ident * identifier list * pexpr list
   | JPEcall_expr of pexpr * identifier * pexpr list
   | JPEsuper_call of identifier * pexpr list
   | JPEnew of qualified_ident * pexpr list
@@ -161,6 +159,13 @@ type variable_declaration =
 
 (*s statements *)
 
+type pbehavior =
+    { java_pbehavior_assumes : pexpr option;
+      java_pbehavior_assigns : (Loc.position * pexpr list) option;
+      java_pbehavior_throws : (qualified_ident * identifier option) option;
+      java_pbehavior_ensures : pexpr 
+    }
+
 type 'a switch_label =
   | Case of 'a 
   | Default
@@ -195,9 +200,12 @@ and pstatement_node =
   | JPSghost_local_decls of variable_declaration
   | JPSghost_statement of pexpr
   | JPSannot of Lexing.position * string
-  | JPSloop_annot of pexpr * pexpr option
+  | JPSloop_annot of pexpr * (identifier * pexpr) list * pexpr option
+  | JPSstatement_spec of 
+      pexpr option * pexpr option * (identifier * pbehavior) list
 
 and block = pstatement list
+
 ;;
 
 (*s method declarations *)
@@ -235,13 +243,6 @@ type constructor_declaration =
       
 (*s class declarations *)
 
-type pbehavior =
-    { java_pbehavior_assumes : pexpr option;
-      java_pbehavior_assigns : (Loc.position * pexpr list) option;
-      java_pbehavior_throws : (qualified_ident * identifier option) option;
-      java_pbehavior_ensures : pexpr 
-    }
-
 type field_declaration =
   | JPFmethod of method_declaration * block option
   | JPFconstructor of constructor_declaration *
@@ -281,10 +282,12 @@ type type_declaration =
   | JPTclass of class_declaration
   | JPTinterface of interface_declaration
   | JPTannot of Lexing.position * string
-  | JPTlemma of identifier * bool * logic_label list * pexpr
+  | JPTlemma of identifier * bool * identifier list * pexpr
   | JPTlogic_type_decl of identifier 
-  | JPTlogic_reads of identifier * type_expr option * logic_label list * parameter list * pexpr list 
-  | JPTlogic_def of identifier * type_expr option * logic_label list * parameter list * pexpr 
+  | JPTlogic_reads of identifier * type_expr option * identifier list * parameter list * pexpr list 
+  | JPTlogic_def of identifier * type_expr option * identifier list * parameter list * pexpr 
+  | JPTinductive of identifier * identifier list * parameter list * (identifier * identifier list * pexpr) list
+  | JPTaxiomatic of identifier * type_declaration list
 
 type import_statement =
   | Import_package of qualified_ident
