@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 open Java_env
@@ -34,13 +32,11 @@ let rec term acc t =
   match t.java_term_node with
     | JTlit _ | JTvar _ 
     | JTstatic_field_access _ -> acc
-    | JTapp (f,lt) -> f::(List.fold_left term acc lt)
+    | JTapp (f,labs,lt) -> f::(List.fold_left term acc lt)
     | JTat(t,_) -> term acc t
     | JTbin (t1,_,_,t2) -> term (term acc t1) t2
     | JTun (_,_,t1) -> term acc t1
-(*
     | JTif(t1,t2,t3) -> term (term (term acc t1) t2) t3
-*)
 (*
     | JTinstanceof(t,_)
     | JTunary (_,t) 
@@ -61,43 +57,15 @@ let rec assertion acc p =
   | JAnot a -> assertion acc a
   | JAbin_obj(t1,_,t2)
   | JAbin(t1,_,_,t2) -> term (term acc t1) t2
-  | JAapp(f,lt) -> f::(List.fold_left term acc lt)
+  | JAapp(f,labs,lt) -> f::(List.fold_left term acc lt)
   | JAand(p1,p2) | JAor(p1,p2) 
   | JAimpl (p1,p2) | JAiff(p1,p2) -> 
       assertion (assertion acc p1) p2
-(*
   | JAif(t1,p2,p3) -> 
       assertion (assertion (term acc t1) p2) p3
-  | JAnot p 
-  | JAold p 
-*)
   | JAquantifier (_,_,p) -> assertion acc p
   | JAbool_expr t | JAinstanceof (t, _, _) -> term acc t
 
-(*
-let spec s = 
-  begin
-  match s.requires with
-    | None -> []
-    | Some p -> 
-	predicate p
-  end @
-  begin
-  match s.assigns with
-    | None -> []
-    | Some l -> List.fold_left (fun acc t -> (term t) @acc)  [] l
-  end @
-  begin
-    match s.ensures with
-      | None -> []
-      | Some p -> predicate p
-  end @
-  begin
-    match s.decreases with
-      | None -> []
-      | Some (t,_) -> term t
-  end
-*)
 
 let rec expr acc e : 'a list = 
   match e.java_expr_node with
@@ -145,6 +113,23 @@ let loop_annot acc la =
   term (assertion acc la.java_loop_invariant) la.java_loop_variant
 *)
 
+let behavior acc (id,assumes,throws,assigns,ensures) =
+  let acc = Option_misc.fold_left assertion acc assumes in
+  let acc = 
+    match assigns with
+      | None -> acc
+      | Some(_,l) -> List.fold_left term acc l
+  in
+  assertion acc ensures
+
+let loop_annot acc annot =    
+  let acc = assertion acc annot.loop_inv in
+  let acc = 
+    List.fold_left (fun acc (_,a) -> assertion acc a) acc annot.behs_loop_inv 
+  in
+  match annot.loop_var with 
+    | None -> acc 
+    | Some dec -> term acc dec 
 
 let rec statement acc s : ('a list * 'b list) = 
   match s.java_statement_node with  
@@ -166,6 +151,7 @@ let rec statement acc s : ('a list * 'b list) =
     | JSassert (_,t) -> let (a,b) = acc in (assertion a t,b)
     | JSreturn_void
     | JSbreak _ -> acc 
+    | JScontinue _ -> acc 
     | JSswitch (e, l)-> 
 	let (a,b) = acc in
 	let b = expr b e in
@@ -175,50 +161,57 @@ let rec statement acc s : ('a list * 'b list) =
     | JSthrow e 
     | JSreturn e 
     | JSexpr e -> let (a, b) = acc in (a, expr b e)
-    | JSfor (inits, cond, inv, dec, updates, body)-> 
+    | JSfor (inits, cond, annot, updates, body)-> 
 	let (a, b) = acc in
 	let b = List.fold_left expr
 	  (List.fold_left expr (expr b cond) updates)
 	  inits
 	in
-	let a = match dec with 
-	  | None -> a 
-	  | Some dec -> term (assertion a inv) dec 
-	in
-	  statement (a, b) body
-    | JSfor_decl (inits, cond, inv, dec, updates, body)-> 
+	let a = loop_annot a annot in
+	statement (a, b) body
+    | JSfor_decl (inits, cond, annot, updates, body)-> 
 	let (a,b) = acc in
 	let b = List.fold_left 
 	  (fun acc (vi,i) -> Option_misc.fold_left initialiser acc i)
 	  (List.fold_left expr (expr b cond) updates)
 	  inits
 	in
-	let a = match dec with 
-	  | None -> a 
-	  | Some dec -> term (assertion a inv) dec 
-	in
-	  statement (a,b) body
-    | JSwhile (cond, inv, dec, body)-> 
+	let a = loop_annot a annot in
+	statement (a,b) body
+    | JSdo (body, annot, cond)-> 
+	let a, b = acc in
+	let a, b = statement (a, b) body in
+	let a = loop_annot a annot in
+	let b = expr b cond in
+	a, b
+    | JSwhile (cond, annot, body)-> 
 	let (a,b) = acc in
 	let b = expr b cond in
-	let a = match dec with 
-	  | None -> a 
-	  | Some dec -> term (assertion a inv) dec 
-	in
+	let a = loop_annot a annot in
 	statement (a,b) body
     | JSvar_decl (vi, init, s)-> 
 	let (a,b)=acc in
 	statement (a,Option_misc.fold_left initialiser b init) s
     | JSskip -> acc
+    | JSstatement_spec(req,dec,behs,s) ->
+	let (a,b) = acc in
+	let a = Option_misc.fold_left assertion a req in
+	let a = Option_misc.fold_left term a dec in
+	let a = List.fold_left behavior a behs in
+	statement (a,b) s
 
 and statements acc l = List.fold_left statement acc l
 
-let compute_logic_calls f t = 
+let compute_logic_calls f (t : [< Java_typing.logic_decl_body]) = 
   let calls =
     match t with
-      | Java_typing.JTerm t -> term [] t 
-      | Java_typing.JAssertion a -> assertion [] a 
-      | Java_typing.JReads r -> List.fold_left term [] r 
+      | `Term t -> term [] t 
+      | `Assertion a -> assertion [] a 
+      | `Reads r -> List.fold_left term [] r 
+      | `Inductive l ->
+	  List.fold_left
+	    (fun acc (_,_,a) -> assertion acc a) [] l
+      | `Builtin -> []
   in
   f.java_logic_info_calls <- calls
 
@@ -231,7 +224,7 @@ let compute_constr_calls f req body =
     f.constr_info_calls <- b
       
 module LogicCallGraph = struct 
-  type t = (int, (java_logic_info * Java_typing.logic_body)) Hashtbl.t 
+  type t = (int, (java_logic_info * Java_typing.logic_def_body)) Hashtbl.t 
   module V = struct
     type t = java_logic_info
     let compare f1 f2 = Pervasives.compare f1.java_logic_info_tag f2.java_logic_info_tag
