@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: java_env.mli,v 1.38 2008-05-23 13:51:39 marche Exp $ *)
+(* $Id: java_env.mli,v 1.42 2008-11-05 14:03:14 filliatr Exp $ *)
 
 (*s types and environments *)
 
@@ -145,7 +143,7 @@ and package_info =
     {
       package_info_tag : int;
       package_info_name : string;
-      package_info_directory : string;
+      mutable package_info_directories : string list;
     }
     
 and java_class_info =
@@ -154,10 +152,12 @@ and java_class_info =
       class_info_name : string;
       class_info_package_env : package_info list;
       mutable class_info_incomplete : bool;
+      mutable class_info_is_final : bool;
       mutable class_info_extends : java_class_info option;
       mutable class_info_is_exception : bool;
       mutable class_info_implements : interface_info list;
       mutable class_info_fields : java_field_info list;
+      mutable class_info_final_fields : java_field_info list;
       mutable class_info_methods : method_info list;
       mutable class_info_constructors : constructor_info list;
     }
@@ -170,6 +170,7 @@ and interface_info =
       mutable interface_info_incomplete : bool;
       mutable interface_info_extends : interface_info list;
       mutable interface_info_fields : java_field_info list;
+      mutable interface_info_final_fields : java_field_info list;
       mutable interface_info_methods : method_info list;
     }
 
@@ -179,9 +180,11 @@ and java_type_info =
 
 (*s literals, shared between ASTs and typed ASTs *)
 
+type float_suffix = [`Single |`Double |`Real]
+
 type literal =
     | Integer of string
-    | Float of string
+    | Float of string * float_suffix
     | Bool of bool
     | String of string
     | Char of string
