@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: java_tast.mli,v 1.36 2008/04/11 12:38:34 marche Exp $ *)
+(* $Id: java_tast.mli,v 1.43 2008-12-09 09:14:18 marche Exp $ *)
 
 open Java_env
 
@@ -35,22 +33,22 @@ type bin_op = Java_ast.bin_op
 type un_op = Java_ast.un_op
 type incr_decr_op = Java_ast.incr_decr_op
 
+type label_assoc = (logic_label * logic_label) list
+
 type term_node =
     | JTlit of literal
     | JTvar of java_var_info
-(*
-    | JTold of term
-*)
     | JTat of term * logic_label
     | JTbin of term * base_type * bin_op * term   
     | JTun of base_type * un_op * term
-    | JTapp of java_logic_info * term list
+    | JTapp of java_logic_info * label_assoc * term list
     | JTfield_access of term * java_field_info
     | JTstatic_field_access of java_type_info * java_field_info
     | JTarray_length of term 
     | JTarray_access of term * term
     | JTarray_range of term * term option * term option
     | JTcast of java_type * term
+    | JTif of term * term * term
 
 and term =
     { java_term_node : term_node;
@@ -74,8 +72,9 @@ type assertion_node =
   | JAbool_expr of term
   | JAbin of term * base_type * bin_op * term   
   | JAbin_obj of term * bin_op * term   
-  | JAapp of java_logic_info * term list
+  | JAapp of java_logic_info * label_assoc * term list
   | JAinstanceof of term * logic_label * java_type
+  | JAif of term * assertion * assertion
 
 and assertion =
     { java_assertion_node : assertion_node;
@@ -137,6 +136,20 @@ type initialiser =
   | JIlist of initialiser list
 
 type 'a switch_label = 'a Java_ast.switch_label
+
+type behavior =
+    Java_ast.identifier * (* behavior name *)
+      assertion option * (* assumes *)
+      java_class_info option * (* throws *)
+      (Loc.position * term list) option * (* assigns *)
+      assertion (* ensures *)
+
+type loop_annot =
+    { loop_inv : assertion;
+      behs_loop_inv : (Java_ast.identifier * assertion) list;
+      loop_var : term option;
+    }
+
 type statement =
   { java_statement_loc : Loc.position ;
     java_statement_node : statement_node }
@@ -148,29 +161,28 @@ and statement_node =
   | JSreturn of expr 
   | JSvar_decl of java_var_info * initialiser option * statement
   | JSblock of block
-  | JSwhile of expr * assertion * term option * statement  
-      (*r condition, invariant, variant, loop body *)
-  | JSfor of expr list * expr * assertion * term option * expr list * statement  
-      (*r init, condition, invariant, variant, steps, loop body *)
+  | JSdo of statement * loop_annot * expr  
+      (*r loop body, annot, condition *)
+  | JSwhile of expr * loop_annot * statement  
+      (*r condition, annot, loop body *)
+  | JSfor of expr list * expr * loop_annot * expr list * statement  
+      (*r init, condition, annot, steps, loop body *)
   | JSfor_decl of (java_var_info * initialiser option) list * 
-      expr * assertion * term option * expr list * statement  
-      (*r decls, condition, invariant, variant, steps, loop body *)
+      expr * loop_annot * expr list * statement  
+      (*r decls, condition, annot, steps, loop body *)
   | JSexpr of expr
   | JSassert of string option * assertion
   | JSswitch of expr * (expr switch_label list * block) list
   | JSbreak of string option
+  | JScontinue of string option
   | JSthrow of expr
   | JStry of block * (java_var_info * block) list * block option
+  | JSstatement_spec of 
+      assertion option * term option * behavior list * statement
+	(*r requires, decreases, behaviors, statement *)
 (*
-  | JSvar_decl of variable_declaration
-  | JPScontinue of identifier option
   | JPSlabel of identifier * pstatement
-  | JPSdo of pstatement * pexpr
-  | JPSfor of pstatement list * pexpr * pstatement list * pstatement  
-  | JPSfor_decl of variable_declaration * pexpr * pstatement list * pstatement
   | JPSsynchronized of pexpr * block
-  | JPSannot of Lexing.position * string
-  | JPSloop_annot of pexpr * pexpr
 *)
 (*
   | Kml_annot_statement of jml_annotation_statement
