@@ -15,29 +15,27 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_constructors.ml,v 1.3 2008/04/15 13:09:53 moy Exp $ *)
+(* $Id: jc_constructors.ml,v 1.27 2008-12-19 14:23:00 marche Exp $ *)
 
 open Jc_env
-open Jc_fenv
 open Jc_region
 open Jc_ast 
+open Jc_fenv
 
-class located ~loc =
+class positioned ~pos =
 object
-  method loc: Loc.position = 
-    match loc with None -> Loc.dummy_position | Some loc -> loc 
+  method pos: Loc.position = 
+    match pos with None -> Loc.dummy_position | Some pos -> pos 
 end
 
 class typed ~typ =
@@ -45,16 +43,16 @@ object
   method typ: jc_type = typ
 end
 
-class logic_labeled ~logic_label =
+class labeled ~label =
 object
-  val mutable llab: logic_label option = logic_label
-  method logic_label = llab
-  method set_logic_label lab = llab <- lab
+  val mutable llab: label option = label
+  method label = llab
+  method set_label lab = llab <- lab
 end
 
-class name_labeled ~name_label =
+class marked ~mark =
 object
-  method name_label: string = name_label
+  method mark: string = mark
 end
 
 class regioned ~region =
@@ -64,83 +62,117 @@ object
   method set_region x = r <- x
 end
 
-class identifier ?loc name = 
+class identifier ?pos name = 
 object
-  inherit located loc
+  inherit positioned pos
   method name: string = name
 end
 
-class ['a] node_located ?loc node = 
+class ['a] node_positioned ?pos node = 
 object
-  inherit located loc
+  inherit positioned pos
   method node: 'a = node
 end
 
-class ptype ?(loc = Loc.dummy_position) node = 
+class ptype ?(pos = Loc.dummy_position) node = 
 object
-  inherit [ptype_node] node_located ~loc node
+  inherit [ptype_node] node_positioned ~pos node
 end
 
-class pexpr ?(loc = Loc.dummy_position) node =
+class pexpr ?(pos = Loc.dummy_position) node =
 object
-  inherit [pexpr_node] node_located ~loc node
+  inherit [pexpr_node] node_positioned ~pos node
 end
 
-class pexpr_with ?loc ?node e =
-  let loc = match loc with None -> e#loc | Some loc -> loc in
+class pexpr_with ?pos ?node e =
+  let pos = match pos with None -> e#pos | Some pos -> pos in
   let node = match node with None -> e#node | Some node -> node in
-  pexpr ~loc node
+  pexpr ~pos node
 
-class nexpr ?(loc = Loc.dummy_position) ?logic_label node =
+class nexpr ?(pos = Loc.dummy_position) ?label node =
 object
-  inherit logic_labeled logic_label
-  inherit [nexpr_node] node_located ~loc node
+  inherit labeled label
+  inherit [nexpr_node] node_positioned ~pos node
 end
 
-class nexpr_with ?loc ?node e =
-  let loc = match loc with None -> e#loc | Some loc -> loc in
+class nexpr_with ?pos ?node e =
+  let pos = match pos with None -> e#pos | Some pos -> pos in
   let node = match node with None -> e#node | Some node -> node in
-  let llab = e#logic_label in
-  nexpr ~loc ~logic_label:llab node
+  let llab = e#label in
+  nexpr ~pos ~label:llab node
 
-class pattern ?(loc = Loc.dummy_position) ~typ node =
+class pattern ?(pos = Loc.dummy_position) ~typ node =
 object
   inherit typed typ
-  inherit [pattern_node] node_located ~loc node
+  inherit [pattern_node] node_positioned ~pos node
 end
 
-class pattern_with ?loc ?node ?typ p =
-  let loc = match loc with None -> p#loc | Some loc -> loc in
+class pattern_with ?pos ?node ?typ p =
+  let pos = match pos with None -> p#pos | Some pos -> pos in
   let node = match node with None -> p#node | Some node -> node in
   let typ = match typ with None -> p#typ | Some typ -> typ in
-  pattern ~loc ~typ node
+  pattern ~pos ~typ node
 
-class term ?(loc = Loc.dummy_position) ~typ ?(name_label="") ?region node =
+class term ?(pos = Loc.dummy_position) ~typ ?(mark="") ?label ?region node =
   let region = 
     match region with None -> dummy_region | Some region -> region 
   in
 object
   inherit typed typ
   inherit regioned region
-  inherit name_labeled name_label
-  inherit [term_node] node_located ~loc node
+  inherit marked mark
+  inherit labeled label
+  inherit [term_node] node_positioned ~pos node
 end
 
-class term_with ?loc ?typ ?name_label ?region ?node t =
-  let loc = match loc with None -> t#loc | Some loc -> loc in
+class term_with ?pos ?typ ?mark ?region ?node t =
+  let pos = match pos with None -> t#pos | Some pos -> pos in
   let typ = match typ with None -> t#typ | Some typ -> typ in
-  let name_label = 
-    match name_label with None -> t#name_label | Some name_label -> name_label 
+  let mark = 
+    match mark with None -> t#mark | Some mark -> mark 
   in
+  let llab = t#label in
   let region = match region with None -> t#region | Some region -> region in
   let node = match node with None -> t#node | Some node -> node in
-  term ~loc ~typ ~name_label ~region node
+  term ~pos ~typ ~mark ?label:llab ~region node
 
-class term_var ?(loc = Loc.dummy_position) ?(name_label="") v =
-  term ~loc ~typ:v.jc_var_info_type ~name_label ~region:v.jc_var_info_region
+class term_var ?(pos = Loc.dummy_position) ?(mark="") v =
+  term ~pos ~typ:v.jc_var_info_type ~mark ~region:v.jc_var_info_region
     (JCTvar v)
 
-class expr ?(loc = Loc.dummy_position) ~typ ?(name_label="") ?region
+class location ?(pos = Loc.dummy_position) ?label ?region node =
+  let region = 
+    match region with None -> dummy_region | Some region -> region 
+  in
+object
+  inherit regioned region
+  inherit labeled label
+  inherit [location_node] node_positioned ~pos node
+end
+
+(* ignore argument's label *)
+class location_with ?pos ?label ?region ~node t =
+  let pos = match pos with None -> t#pos | Some pos -> pos in
+  let region = match region with None -> t#region | Some region -> region in
+  location ~pos ?label ~region node
+
+class location_set ?(pos = Loc.dummy_position) ?label ?region node =
+  let region = 
+    match region with None -> dummy_region | Some region -> region 
+  in
+object
+  inherit regioned region
+  inherit labeled label
+  inherit [location_set_node] node_positioned ~pos node
+end
+
+(* ignore argument's label *)
+class location_set_with ?pos ?label ?region ~node t =
+  let pos = match pos with None -> t#pos | Some pos -> pos in
+  let region = match region with None -> t#region | Some region -> region in
+  location_set ~pos ?label ~region node
+
+class expr ?(pos = Loc.dummy_position) ~typ ?(mark="") ?region
   ?original_type node =
   let region = 
     match region with None -> dummy_region | Some region -> region 
@@ -148,17 +180,17 @@ class expr ?(loc = Loc.dummy_position) ~typ ?(name_label="") ?region
 object
   inherit typed typ
   inherit regioned region
-  inherit name_labeled name_label
-  inherit [expr_node] node_located ~loc node
+  inherit marked mark
+  inherit [expr_node] node_positioned ~pos node
   method original_type = 
     match original_type with None -> typ | Some original_type -> original_type
 end
 
-class expr_with ?loc ?typ ?name_label ?region ?node ?original_type e =
-  let loc = match loc with None -> e#loc | Some loc -> loc in
+class expr_with ?pos ?typ ?mark ?region ?node ?original_type e =
+  let pos = match pos with None -> e#pos | Some pos -> pos in
   let typ = match typ with None -> e#typ | Some typ -> typ in
-  let name_label = 
-    match name_label with None -> e#name_label | Some name_label -> name_label 
+  let mark = 
+    match mark with None -> e#mark | Some mark -> mark 
   in
   let region = match region with None -> e#region | Some region -> region in
   let node = match node with None -> e#node | Some node -> node in
@@ -166,51 +198,53 @@ class expr_with ?loc ?typ ?name_label ?region ?node ?original_type e =
     | None -> e#original_type
     | Some original_type -> original_type
   in
-  expr ~loc ~typ ~name_label ~region ~original_type node
+  expr ~pos ~typ ~mark ~region ~original_type node
 
-class assertion ?(name_label="") ?(loc = Loc.dummy_position) node =
+class assertion ?(mark="") ?label ?(pos = Loc.dummy_position) node =
 object
-  inherit name_labeled name_label
-  inherit [assertion_node] node_located ~loc node
+  inherit marked mark
+  inherit labeled label
+  inherit [assertion_node] node_positioned ~pos node
 end
 
-class assertion_with ?loc ?name_label ?node a =
-  let loc = match loc with None -> a#loc | Some loc -> loc in
-  let name_label = 
-    match name_label with None -> a#name_label | Some name_label -> name_label 
+class assertion_with ?pos ?mark ?node a =
+  let pos = match pos with None -> a#pos | Some pos -> pos in
+  let mark = 
+    match mark with None -> a#mark | Some mark -> mark 
   in
+  let llab = a#label in
   let node = match node with None -> a#node | Some node -> node in
-  assertion ~loc ~name_label node
+  assertion ~pos ~mark ?label:llab node
 
-class ['expr] ptag ?(loc = Loc.dummy_position) node =
+class ['expr] ptag ?(pos = Loc.dummy_position) node =
 object
-  inherit ['expr ptag_node] node_located ~loc node
+  inherit ['expr ptag_node] node_positioned ~pos node
 end
 
-class ['expr] ptag_with ?loc ?node t =
-  let loc = match loc with None -> t#loc | Some loc -> loc in
+class ['expr] ptag_with ?pos ?node t =
+  let pos = match pos with None -> t#pos | Some pos -> pos in
   let node = match node with None -> t#node | Some node -> node in
-  ['expr] ptag ~loc node
+  ['expr] ptag ~pos node
 
-class tag ?(loc = Loc.dummy_position) node =
+class tag ?(pos = Loc.dummy_position) node =
 object
-  inherit [tag_node] node_located ~loc node
+  inherit [tag_node] node_positioned ~pos node
 end
 
-class tag_with ?loc ?node t =
-  let loc = match loc with None -> t#loc | Some loc -> loc in
+class tag_with ?pos ?node t =
+  let pos = match pos with None -> t#pos | Some pos -> pos in
   let node = match node with None -> t#node | Some node -> node in
-  tag ~loc node
+  tag ~pos node
 
-class ['expr] decl ?(loc = Loc.dummy_position) node =
+class ['expr] decl ?(pos = Loc.dummy_position) node =
 object
-  inherit ['expr decl_node] node_located ~loc node
+  inherit ['expr decl_node] node_positioned ~pos node
 end
 
-class ['expr] decl_with ?loc ?node t =
-  let loc = match loc with None -> t#loc | Some loc -> loc in
+class ['expr] decl_with ?pos ?node t =
+  let pos = match pos with None -> t#pos | Some pos -> pos in
   let node = match node with None -> t#node | Some node -> node in
-  ['expr] decl ~loc node
+  ['expr] decl ~pos node
 
 (*******************************************************************************)
 (*                             constant constructors                           *)
@@ -230,7 +264,7 @@ module Const = struct
   let mkreal ?value ?valuestr () =
     match value, valuestr with
       | Some value, None -> JCCreal (string_of_float value)
-      | None, Some valuestr -> JCCreal valuestr
+      | None, Some valuestr -> JCCreal(valuestr)
       | _ -> failwith "Jc_constructors.Const.mkint: use with ~value OR \
 ~valuestr only"
 end
@@ -244,7 +278,7 @@ let oo a b = match a with
   | Some a -> a
 
 module PExpr = struct
-  let mk ?loc ~node () = new pexpr ?loc node
+  let mk ?pos ~node () = new pexpr ?pos node
 
   let mkconst ~const = mk ~node:(JCPEconst const)
   let mkvoid = mkconst ~const:Const.mkvoid
@@ -254,18 +288,18 @@ module PExpr = struct
   let mkreal ?value ?valuestr = mkconst ~const:(Const.mkreal ?value ?valuestr ())
 
   let mkbinary ~expr1 ~op ~expr2 = mk ~node:(JCPEbinary(expr1, op, expr2))
-  let mkbinary_list ~default ~op ?expr1 ?expr2 ?list ?loc
+  let mkbinary_list ~default ~op ?expr1 ?expr2 ?list ?pos
       () =
     match expr1, expr2, list with
       | None, None, Some el ->
           List.fold_left
-            (fun expr2 expr1 -> mkbinary ?loc ~expr1 ~expr2 ~op ())
+            (fun expr2 expr1 -> mkbinary ?pos ~expr1 ~expr2 ~op ())
             default el
-      | Some expr1, Some expr2, None -> mkbinary ~expr1 ~op ~expr2 ?loc ()
+      | Some expr1, Some expr2, None -> mkbinary ~expr1 ~op ~expr2 ?pos ()
       | _ -> failwith "Jc_constructors.PExpr.mkbinary_list should be used \
 either with (~expr1 AND ~expr2) OR ~list only."
   let mkand = mkbinary_list ~default:(mkboolean ~value:true ()) ~op:`Bland
-  let mkor = mkbinary_list ~default:(mkboolean ~value:true ()) ~op:`Blor
+  let mkor = mkbinary_list ~default:(mkboolean ~value:false ()) ~op:`Blor
   let mkadd = mkbinary_list ~default:(mkint ~value:0 ()) ~op:`Badd
 
   let mklabel ~label ~expr = mk ~node:(JCPElabel(label, expr))
@@ -297,33 +331,33 @@ either with (~expr1 AND ~expr2) OR ~list only."
     mk ~node:(JCPEif(condition, expr_then, expr_else))
   let mkblock ~exprs = mk ~node:(JCPEblock exprs)
   let mkdecl ~typ ~var ?init = mk ~node:(JCPEdecl(typ, var, init))
-  let mklet ?typ ~var ?init ~body ?loc () =
+  let mklet ?typ ~var ?init ~body ?pos () =
     match typ with
-      | None -> mk ~node:(JCPElet(typ, var, init, body)) ?loc ()
+      | None -> mk ~node:(JCPElet(typ, var, init, body)) ?pos ()
       | Some typ ->
 	  mkblock ~exprs:[
-	    mkdecl ~typ ~var ?init ?loc ();
+	    mkdecl ~typ ~var ?init ?pos ();
 	    body
-	  ] ?loc ()
+	  ] ?pos ()
   let mklet_nodecl ?typ ~var ?init ~body =
     mk ~node:(JCPElet(typ, var, init, body))
-  let mkrange ?left ?right ?locations ?loc () =
-    let r = mk ~node:(JCPErange(left, right)) ?loc () in
+  let mkrange ?left ?right ?locations ?pos () =
+    let r = mk ~node:(JCPErange(left, right)) ?pos () in
     match locations with
       | None -> r
-      | Some l -> mkadd ~expr1:l ~expr2:r ?loc ()
+      | Some l -> mkadd ~expr1:l ~expr2:r ?pos ()
   let mkalloc ?(count = mkint ~value:1 ()) ~typ =
     mk ~node:(JCPEalloc(count, typ))
   let mkfree ~expr = mk ~node:(JCPEfree expr)
   let mkmutable ~expr ~tag = mk ~node:(JCPEmutable(expr, tag))
-  let mktag_equality ~tag1 ~tag2 = mk ~node:(JCPEtagequality(tag1, tag2))
+  let mktag_equality ~tag1 ~tag2 = mk ~node:(JCPEeqtype(tag1, tag2))
   let mkmatch ~expr ~cases = mk ~node:(JCPEmatch(expr, cases))
-  let mkassert ~expr = mk ~node:(JCPEassert expr)
+  let mkassert ~expr = mk ~node:(JCPEassert ([],Aassert,expr))
   let mkwhile ?(condition = mkboolean ~value:true ())
-      ?(invariant = mkboolean ~value:true ()) ?variant ~body =
+      ?(invariant = []) ?variant ~body =
     mk ~node:(JCPEwhile(condition, invariant, variant, body))
   let mkfor ?(inits = []) ?(condition = mkboolean ~value:true ())
-      ?(updates = []) ?(invariant = mkboolean ~value:true ()) ?variant ~body =
+      ?(updates = []) ?(invariant = []) ?variant ~body =
     mk ~node:(JCPEfor(inits, condition, updates, invariant, variant, body))
   let mkreturn ?(expr = mkvoid ()) = mk ~node:(JCPEreturn expr)
   let mkbreak ?(label = "") = mk ~node:(JCPEbreak label)
@@ -336,8 +370,8 @@ either with (~expr1 AND ~expr2) OR ~list only."
   let mkunpack ~expr ?tag = mk ~node:(JCPEunpack(expr, tag))
   let mkswitch ~expr ?(cases = []) = mk ~node:(JCPEswitch(expr, cases))
 
-  let mkcatch ~exn ?(name = "") ?body ?loc () =
-    exn, name, (match body with None -> mkvoid ?loc () | Some body -> body)
+  let mkcatch ~exn ?(name = "") ?body ?pos () =
+    exn, name, (match body with None -> mkvoid ?pos () | Some body -> body)
 
   let mkshift ~expr ~offset = mkadd ~expr1:expr ~expr2:offset
   let mknot = mkunary ~op:`Unot
@@ -346,13 +380,16 @@ either with (~expr1 AND ~expr2) OR ~list only."
   let mkiff = mkbinary ~op:`Biff
   let mkincr_heap ~expr ~field ?(op = `Upostfix_inc) =
     mkunary ~op ~expr:(mkderef ~expr ~field ())
+
+  let mkcontract ~requires ~decreases ~behaviors ~expr =
+    mk ~node:(JCPEcontract(requires, decreases, behaviors, expr))
 end
 
 module PDecl = struct
   open PExpr
 
-  let mk ?loc ~node () =
-    new node_located ?loc node
+  let mk ?pos ~node () =
+    new node_positioned ?pos node
   let mkfun_def ?(result_type = new ptype (JCPTnative Tunit)) ~name
       ?(params = []) ?(clauses = []) ?body =
     mk ~node:(JCDfun(result_type, name, params, clauses, body))
@@ -360,22 +397,28 @@ module PDecl = struct
     mk ~node:(JCDlemma(name, axiom, labels, body))
   let mklogic_var_def ~typ ~name ?body =
     mk ~node:(JCDlogic_var(typ, name, body))
-  let mklogic_def ?typ ~name ?(labels = []) ?(params = []) ?reads ?body =
-    let roe = match reads, body with
-      | Some r, None -> JCreads r
-      | None, Some b -> JCexpr b
-      | None, None -> JCreads []
-      | Some _, Some _ ->
+  let mklogic_def ?typ ~name ?(labels = []) ?(params = []) ?reads ?body ?inductive = 
+  let roe = match reads, body, inductive with
+      | None, None, None -> JCreads []
+      | Some r, None, None -> JCreads r
+      | None, Some b, None -> JCexpr b
+      | None, None, Some l -> JCinductive l 
+      | _ ->
           raise
-            (Invalid_argument "mklogic_def: cannot use both ~reads and ~body")
+            (Invalid_argument "mklogic_def: cannot use both ~reads and ~body and ~inductive")
     in
     mk ~node:(JCDlogic(typ, name, labels, params, roe))
+      
+  let mkaxiomatic ~name ~decls =
+    mk ~node:(JCDaxiomatic(name,decls))
+  let mklogic_type ~name =
+    mk ~node:(JCDlogic_type(name))
   let mkvar_def ~typ ~name ?init =
     mk ~node:(JCDvar(typ, name, init))
   let mkglobal_inv_def ~name ~body =
     mk ~node:(JCDglobal_inv(name, body))
-  let mktag_def ~name ?super ?(fields = []) ?(invariants = []) =
-    mk ~node:(JCDtag(name, super, fields, invariants))
+  let mktag_def ~name ?(params = []) ?super ?(fields = []) ?(invariants = []) =
+    mk ~node:(JCDtag(name, params, super, fields, invariants))
   let mkenum_type_def ~name ~left ~right =
     mk ~node:(JCDenum_type(name, left, right))
   let mkexception_def ~name ?arg_type =
@@ -389,16 +432,24 @@ module PDecl = struct
   let mkabstract_domain_def ~value = mk ~node:(JCDabstract_domain value)
   let mkint_model_def ~value = mk ~node:(JCDint_model value)
 
-  let mkrequires expr = JCCrequires expr
-  let mkbehavior ?(loc = Loc.dummy_position) ~name ?throws ?assumes ?requires
+  let mkbehavior ?(pos = Loc.dummy_position) ~name ?throws ?assumes ?requires
       ?assigns ?(ensures = mkboolean ~value:true ()) () =
-    JCCbehavior(loc, name, throws, assumes, requires, assigns, ensures)
-  let mkbehavior_with ?loc ?name ?throws ?assumes ?requires ?assigns ?ensures =
+    (pos, name, throws, assumes, requires, assigns, ensures)
+
+  let mkrequires_clause expr = JCCrequires expr
+
+  let mkdecreases_clause expr = JCCdecreases expr
+
+  let mkbehavior_clause ?(pos = Loc.dummy_position) ~name ?throws ?assumes ?requires
+      ?assigns ?(ensures = mkboolean ~value:true ()) () =
+      JCCbehavior (mkbehavior ~pos ~name ?throws ?assumes ?requires ?assigns ~ensures ())
+
+  let mkbehavior_clause_with ?pos ?name ?throws ?assumes ?requires ?assigns ?ensures =
     function
-      | JCCbehavior(loc', name', throws', assumes', requires', assigns',
+      | JCCbehavior(pos', name', throws', assumes', requires', assigns',
                     ensures') ->
           JCCbehavior(
-            oo loc loc',
+            oo pos pos',
             oo name name',
             oo throws throws',
             oo assumes assumes',
@@ -407,8 +458,8 @@ module PDecl = struct
             oo ensures ensures'
           )
       | _ -> raise (Invalid_argument "mkbehavior_with")
-  let mkassigns ?(loc = Loc.dummy_position) ?(locations = []) () =
-    loc, locations
+  let mkassigns ?(pos = Loc.dummy_position) ?(locations = []) () =
+    pos, locations
 
   let mktag_invariant ~name ~var ~body = name, var, body
 
@@ -417,13 +468,29 @@ module PDecl = struct
     | _ -> raise (Invalid_argument "behavior_ensures")
 end
 
+
+(******************************************************************************)
+(*                              nexpr constructors                            *)
+(******************************************************************************)
+
+module NExpr = struct
+  let mk ?pos ~node () = new nexpr ?pos node
+
+  let mkcast ~expr ~typ = mk ~node:(JCNEcast(expr, typ))
+end
+
+
 (*******************************************************************************)
 (*                               expr constructors                             *)
 (*******************************************************************************)
 
 module Expr = struct
-  let mk ?loc ~typ ?name_label ?region ?original_type ~node () =
-    new expr ?loc ~typ ?name_label ?region ?original_type node
+  let mk ?pos ~typ ?mark ?region ?original_type ~node () =
+    new expr ?pos ~typ ?mark ?region ?original_type node
+
+  let mkconst ~const = mk ~typ:(JCTnative Tinteger) ~node:(JCEconst const)
+  let mkint ?value ?valuestr = mkconst ~const:(Const.mkint ?value ?valuestr ())
+  let mkbinary ~expr1 ~op ~expr2 = mk ~node:(JCEbinary(expr1, op, expr2))
 
   let mklet ~var ?init ~body =
     mk ~typ:var.jc_var_info_type ~node:(JCElet(var, init, body))
@@ -438,8 +505,12 @@ end
 (*******************************************************************************)
 
 module Term = struct
-  let mk ?loc ~typ ?name_label ?region ~node () =
-    new term ?loc ~typ ?name_label ?region node
+  let mk ?pos ~typ ?mark ?region ~node () =
+    new term ?pos ~typ ?mark ?region node
+
+  let mkconst ~const = mk ~typ:(JCTnative Tinteger) ~node:(JCTconst const)
+  let mkint ?value ?valuestr = mkconst ~const:(Const.mkint ?value ?valuestr ())
+  let mkbinary ~term1 ~op ~term2 = mk ~node:(JCTbinary(term1, op, term2))
 
   let mkvar ~var = 
     mk ~typ:var.jc_var_info_type ~node:(JCTvar var)
@@ -450,18 +521,43 @@ end
 (*******************************************************************************)
 
 module Assertion = struct
-  let mk ?loc ?name_label ~node () =
-    new assertion ?loc ?name_label node
+  let mk ?pos ?mark ~node () =
+    new assertion ?pos ?mark node
 
-  let fake ?loc ?name_label ~value () = value
+  let fake ?pos ?mark ~value () = value
+
+  let is_true a = 
+    match a#node with 
+      | JCAtrue -> true
+      | JCAbool_term t when t#node = JCTconst(JCCboolean true) -> true
+      | _ -> false
+    
+  let is_false a =
+    match a#node with 
+      | JCAfalse -> true
+      | JCAbool_term t when t#node = JCTconst(JCCboolean false) -> true
+      | _ -> false
 
   let mktrue = mk ~node:JCAtrue
   let mkfalse = mk ~node:JCAfalse
+
   let mkand ~conjuncts =
+    (* optimization *)
+    let conjuncts = List.filter (fun a -> not (is_true a)) conjuncts in
     match conjuncts with
       | [] -> mktrue
       | [a] -> fake ~value:a
       | alist -> mk ~node:(JCAand alist)
+
+  let mkor ~disjuncts = 
+    (* optimization *)
+    let disjuncts = List.filter (fun a -> not (is_false a)) disjuncts in
+    match disjuncts with
+      | [] -> mkfalse
+      | [a] -> fake ~value:a
+      | alist -> mk ~node:(JCAor alist)
+
+  let mknot ~asrt:a = mk ~node:(JCAnot a)
 end
 
 (*
