@@ -15,22 +15,28 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_envset.ml,v 1.22 2008/03/20 16:05:13 moy Exp $ *)
+(* $Id: jc_envset.ml,v 1.35 2008-11-05 14:03:15 filliatr Exp $ *)
 
+open Jc_stdlib
 open Jc_env
 
+module type OrderedType =
+sig
+  type t
+  val equal : t -> t -> bool
+  val compare : t -> t -> int
+end
+
 module type OrderedHashedType =
 sig
   type t
@@ -106,17 +112,33 @@ let is_pointer_type t =
     | JCTpointer _ -> true
     | _ -> false
 
+let is_nonnull_pointer_type t =
+  match t with
+    | JCTpointer _ -> true
+    | _ -> false
+
+let is_integral_type = function
+  | JCTnative Tinteger -> true
+  | JCTenum _ -> true 
+  | JCTnative _ | JCTlogic _ | JCTpointer _ | JCTnull | JCTany
+  | JCTtype_var _ -> false
+
 let is_embedded_field fi =
   match fi.jc_field_info_type with
     | JCTpointer(_,Some _,Some _) -> true
     | _ -> false
 
-module VarSet = 
-  Set.Make(struct type t = var_info
-		  let compare v1 v2 = 
-		    Pervasives.compare 
-		      v1.jc_var_info_tag v2.jc_var_info_tag
-	   end)
+module VarOrd = struct
+  type t = var_info
+  let compare v1 v2 = 
+    Pervasives.compare v1.jc_var_info_tag v2.jc_var_info_tag
+  let equal v1 v2 = v1.jc_var_info_tag = v2.jc_var_info_tag
+  let hash v = Hashtbl.hash v.jc_var_info_tag
+end
+
+module VarSet = Set.Make(VarOrd)
+
+module VarMap = Map.Make(VarOrd)
 
 module StructOrd =
   struct type t = struct_info
@@ -134,13 +156,13 @@ module StructSet = Set.Make(StructOrd)
 module StructMap = Map.Make(StructOrd)
 
 module VariantOrd = struct
-  type t = variant_info
+  type t = root_info
   let compare v1 v2 = 
-    Pervasives.compare v1.jc_variant_info_name v2.jc_variant_info_name
+    Pervasives.compare v1.jc_root_info_name v2.jc_root_info_name
   let equal v1 v2 =
-    v1.jc_variant_info_name = v2.jc_variant_info_name
+    v1.jc_root_info_name = v2.jc_root_info_name
   let hash v =
-    Hashtbl.hash v.jc_variant_info_name 
+    Hashtbl.hash v.jc_root_info_name 
 end
 
 module VariantSet = Set.Make(VariantOrd)
@@ -160,24 +182,65 @@ module FieldOrd =
 module FieldSet = Set.Make(FieldOrd)
 
 module FieldMap = Map.Make(FieldOrd)
-
 module FieldTable = Hashtbl.Make(FieldOrd)
 
-module FieldOrVariantOrd = 
+module MemClass = 
 struct
-  type t = field_or_variant_info
+  type t = mem_class
   let equal fv1 fv2 = match fv1,fv2 with
-    | FVfield a1,FVfield a2 -> FieldOrd.equal a1 a2
-    | FVvariant b1,FVvariant b2 -> VariantOrd.equal b1 b2
+    | JCmem_field a1,JCmem_field a2 -> FieldOrd.equal a1 a2
+    | JCmem_plain_union b1,JCmem_plain_union b2 -> VariantOrd.equal b1 b2
+    | JCmem_bitvector,JCmem_bitvector -> true
     | _ -> false
   let compare fv1 fv2 = match fv1,fv2 with
-    | FVfield a1,FVfield a2 -> FieldOrd.compare a1 a2
-    | FVvariant b1,FVvariant b2 -> VariantOrd.compare b1 b2
-    | FVfield _,_ -> 1
-    | FVvariant _,_ -> -1
+    | JCmem_field a1,JCmem_field a2 -> FieldOrd.compare a1 a2
+    | JCmem_plain_union b1,JCmem_plain_union b2 -> VariantOrd.compare b1 b2
+    | JCmem_bitvector,JCmem_bitvector -> 0
+    | JCmem_field _,_ -> 1
+    | _,JCmem_field _ -> -1
+    | JCmem_plain_union _,_ -> 1
+    | _,JCmem_plain_union _ -> -1
   let hash = function
-    | FVfield a -> FieldOrd.hash a
-    | FVvariant b -> VariantOrd.hash b
+    | JCmem_field a -> FieldOrd.hash a
+    | JCmem_plain_union b -> VariantOrd.hash b
+    | JCmem_bitvector -> 0
+end
+
+module MemClassSet = Set.Make(MemClass)
+
+module AllocClass = 
+struct
+  type t = alloc_class
+  let equal fv1 fv2 = match fv1,fv2 with
+    | JCalloc_root st1,JCalloc_root st2 -> VariantOrd.equal st1 st2
+    | JCalloc_bitvector,JCalloc_bitvector -> true
+    | _ -> false
+  let compare fv1 fv2 = match fv1,fv2 with
+    | JCalloc_root a1,JCalloc_root a2 -> VariantOrd.compare a1 a2
+    | JCalloc_bitvector,JCalloc_bitvector -> 0
+    | JCalloc_root _,_ -> 1
+    | _,JCalloc_root _ -> -1
+  let hash = function
+    | JCalloc_root a -> VariantOrd.hash a
+    | JCalloc_bitvector -> 0
+end
+
+(* TODO: take into account type parameters *)
+module PointerClass = 
+struct
+  type t = pointer_class
+  let equal fv1 fv2 = match fv1,fv2 with
+    | JCtag(st1,_),JCtag(st2,_) -> StructOrd.equal st1 st2
+    | JCroot vi1,JCroot vi2 -> VariantOrd.equal vi1 vi2
+    | _ -> false
+  let compare fv1 fv2 = match fv1,fv2 with
+    | JCtag(a1,_),JCtag(a2,_) -> StructOrd.compare a1 a2
+    | JCroot b1,JCroot b2 -> VariantOrd.compare b1 b2
+    | JCtag _,_ -> 1
+    | _,JCtag _ -> -1
+  let hash = function
+    | JCtag(a,_) -> StructOrd.hash a
+    | JCroot b -> VariantOrd.hash b
 end
 
 module ExceptionOrd =   
@@ -192,8 +255,8 @@ module ExceptionSet = Set.Make(ExceptionOrd)
 module ExceptionMap = Map.Make(ExceptionOrd)
 
 module LogicLabelOrd =
-  struct type t = logic_label
-	 let compare = (Pervasives.compare : logic_label -> logic_label -> int)
+  struct type t = label
+	 let compare = (Pervasives.compare : label -> label -> int)
   end
 
 module LogicLabelSet = Set.Make(LogicLabelOrd)
