@@ -15,19 +15,24 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_envset.mli,v 1.19 2008/03/20 16:05:13 moy Exp $ *)
+(* $Id: jc_envset.mli,v 1.31 2008-11-05 14:03:15 filliatr Exp $ *)
+
+module type OrderedType =
+sig
+  type t
+  val equal : t -> t -> bool
+  val compare : t -> t -> int
+end
 
 module type OrderedHashedType =
 sig
@@ -38,6 +43,7 @@ sig
 end
 
 open Jc_env
+open Jc_stdlib
 
 module StringSet : Set.S with type elt = string
 
@@ -46,18 +52,24 @@ module StringMap : Map.S with type key = string
 val get_unique_name : ?local_names:StringSet.t -> string -> string
 
 val is_pointer_type : Jc_env.jc_type -> bool
+val is_nonnull_pointer_type : Jc_env.jc_type -> bool
+val is_integral_type: jc_type -> bool
 
 val is_embedded_field : Jc_env.field_info -> bool
 
+module VarOrd : OrderedHashedType with type t = var_info
+
 module VarSet : Set.S with type elt = var_info
 
+module VarMap : Map.S with type key = var_info
+
 module StructSet : Set.S with type elt = struct_info
 
 module StructMap : Map.S with type key = struct_info
 
-module VariantSet : Set.S with type elt = variant_info
+module VariantSet : Set.S with type elt = root_info
 
-module VariantMap : Map.S with type key = variant_info
+module VariantMap : Map.S with type key = root_info
 
 module ExceptionSet : Set.S with type elt = exception_info
 
@@ -65,7 +77,7 @@ module ExceptionMap : Map.S with type key = exception_info
 
 module StructOrd : OrderedHashedType with type t = struct_info
 
-module VariantOrd : OrderedHashedType with type t = variant_info
+module VariantOrd : OrderedHashedType with type t = root_info
 
 module FieldOrd : OrderedHashedType with type t = field_info
 
@@ -75,9 +87,15 @@ module FieldMap : Map.S with type key = field_info
 
 module FieldTable : Hashtbl.S with type key = field_info
 
-module FieldOrVariantOrd : OrderedHashedType with type t = field_or_variant_info
+module MemClass : OrderedHashedType with type t = mem_class
 
-module LogicLabelSet : Set.S with type elt = logic_label
+module MemClassSet : Set.S with type elt = mem_class
+
+module AllocClass : OrderedHashedType with type t = alloc_class
+
+module PointerClass : OrderedHashedType with type t = pointer_class
+
+module LogicLabelSet : Set.S with type elt = label
 
 (*
 Local Variables: 
