@@ -15,29 +15,33 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
+(* $Id: jc_invariants.ml,v 1.85 2009-01-21 08:34:15 marche Exp $ *)
+
+open Jc_stdlib
 open Jc_env
 open Jc_envset
-open Jc_fenv
-open Jc_constructors
+open Jc_region
 open Jc_ast
+open Jc_fenv
+
+open Jc_name
+open Jc_constructors
 open Jc_pervasives
 open Jc_iterators
-open Output
-open Jc_region
-open Jc_name
 open Jc_interp_misc
+open Jc_struct_tools
+
+open Output
 
 (* other modifications for this extension can be found in:
      ast, typing, norm, interp: about pack / unpack, and mutable
@@ -73,7 +77,7 @@ let invariant_for_struct this st =
     make_and_list
       (List.map 
 	 (fun (li, _) -> 
-	    make_logic_pred_call ~label_in_name:false li [this] [] []) invs)
+	    make_logic_pred_call ~label_in_name:false ~region_assoc:[] ~label_assoc:[] li [this]) invs)
   in
   let reads =
     List.fold_left
@@ -86,11 +90,11 @@ let invariant_for_struct this st =
 let make_assume reads assume =
   BlackBox (Annot_type (LTrue, unit_type, reads, [], assume, []))
 
-let fully_packed tov e =
+let fully_packed pc e =
   LPred(
     fully_packed_name,
-    [ LVar (tag_table_name tov);
-      LVar (mutable_name tov);
+    [ LVar (generic_tag_table_name (pointer_class_root pc));
+      LVar (mutable_name pc);
       e ])
 (*
 let type_structure = function
@@ -98,9 +102,9 @@ let type_structure = function
   | _ -> failwith "type_structure"
 *)
 
-let type_tov = function
-  | JCTpointer(tov, _, _) -> tov
-  | _ -> raise (Invalid_argument "type_tov")
+let type_pc = function
+  | JCTpointer(pc, _, _) -> pc
+  | _ -> raise (Invalid_argument "type_pc")
 
 let range_min = function
   | JCTpointer(_, x, _) -> x
@@ -158,24 +162,24 @@ let make_bool b = LConst(Prim_bool b)
 (************************************)
 
 (* Typing imposes non pointer fields to have the flag "rep" *)
-let field this loc fi =
+let field this pos fi =
   if not fi.jc_field_info_rep then
-    Jc_typing.typing_error loc "this term is not a rep field of %s"
+    Jc_typing.typing_error pos "this term is not a rep field of %s"
       this.jc_var_info_name
 
-(*let rec check_rep ?(must_deref=false) this loc t =
+(*let rec check_rep ?(must_deref=false) this pos t =
   match t.jc_term_node with
     | JCTvar vi when vi == this && not must_deref -> ()
     | JCTderef (t, lab, fi) ->
-	field fi this loc;
-	check_rep ~must_deref:false this loc t
+	field fi this pos;
+	check_rep ~must_deref:false this pos t
     | JCTcast (t, _, _) -> assert false (* TODO *)
     | JCTshift (t, _) ->
 	(* t must not be this, but might be a field of this if it is a table *)
 	(* (? TODO) *)
-	check_rep ~must_deref:true this loc t
+	check_rep ~must_deref:true this pos t
     | _ ->
-	Jc_typing.typing_error loc "this term is not a rep field of %s"
+	Jc_typing.typing_error pos "this term is not a rep field of %s"
 	  this.jc_var_info_name*)
 
 (* A pattern may hide some dereferencing. *)
@@ -183,7 +187,7 @@ let pattern this p =
   iter_pattern
     (fun p -> match p#node with
        | JCPstruct(_, fipl) ->
-	   List.iter (field this p#loc) (List.map fst fipl)
+	   List.iter (field this p#pos) (List.map fst fipl)
        | _ -> ())
     p
 
@@ -196,12 +200,12 @@ let term this t =
     (fun t -> match t#node with
        | JCTapp app ->
 	   let id = app.jc_app_fun in
-	   if not (FieldOrVariantRegionMap.is_empty
+	   if not (MemoryMap.is_empty
 		     id.jc_logic_info_effects.jc_effect_memories) then
-	     Jc_typing.typing_error t#loc
+	     Jc_typing.typing_error t#pos
 	       "this call is not allowed in structure invariant"
        | JCTderef(_, _, fi) ->
-	   field this t#loc fi
+	   field this t#pos fi
        | JCTmatch(_, ptl) ->
 	   List.iter (pattern this) (List.map fst ptl)
        | _ -> ())
@@ -262,10 +266,10 @@ let rec assertion this p =
     | JCAquantifier(_,id, p) -> assertion this p
     | JCAapp app ->
 	let id = app.jc_app_fun in
-	if FieldOrVariantRegionMap.is_empty id.jc_logic_info_effects.jc_effect_memories
+	if MemoryMap.is_empty id.jc_logic_info_effects.jc_effect_memories
 	then List.iter (term this) app.jc_app_args
 	else
-	  Jc_typing.typing_error p#loc
+	  Jc_typing.typing_error p#pos
 	    "this call is not allowed in structure invariant"
     | JCAnot p -> assertion this p
     | JCAiff (p1, p2)
@@ -273,9 +277,9 @@ let rec assertion this p =
     | JCArelation (t1,_, t2) -> term this t1; term this t2
     | JCAand l | JCAor l -> List.iter (assertion this) l
     | JCAmutable _ ->
-	Jc_typing.typing_error p#loc
+	Jc_typing.typing_error p#pos
 	  "\\mutable is not allowed in structure invariant"
-    | JCAtagequality(t1, t2, _) ->
+    | JCAeqtype(t1, t2, _) | JCAsubtype(t1, t2, _) ->
 	tag this t1;
 	tag this t2
     | JCAmatch(t, pal) ->
@@ -326,7 +330,8 @@ let rec assertion_memories aux a = match a#node with
   | JCAbool_term t -> term_memories aux t
   | JCAif(t, a1, a2) -> assertion_memories (assertion_memories (term_memories aux t) a1) a2
   | JCAmutable(t, _, _) -> term_memories aux t
-  | JCAtagequality(t1, t2, _) -> tag_memories (tag_memories aux t2) t1
+  | JCAeqtype(t1, t2, _) | JCAsubtype(t1, t2, _) -> 
+      tag_memories (tag_memories aux t2) t1
   | JCAmatch(t, pal) ->
       term_memories (List.fold_left assertion_memories aux (List.map snd pal)) t
 
@@ -338,7 +343,7 @@ let rec all_structures st acc =
     List.fold_left
       (fun acc fi ->
 	 match fi.jc_field_info_type with
-	   | JCTpointer(JCtag st, _, _) -> all_structures st acc
+	   | JCTpointer(JCtag(st, _), _, _) -> all_structures st acc
 	   | _ -> acc)
       (StringSet.add st.jc_struct_info_name acc)
       st.jc_struct_info_fields   
@@ -355,29 +360,28 @@ let struct_inv_memories acc st =
 (* TODO: factorize using Jc_interp_misc.logic_params *)
 let invariant_params acc li =
   let acc =
-    FieldOrVariantRegionMap.fold
-      (fun (fvi,r) labels acc -> 
-	 (field_or_variant_region_memory_name(fvi,r), field_or_variant_memory_type fvi)::acc)
+    MemoryMap.fold
+      (fun (mc,r) labels acc -> 
+	 (memory_name(mc,r), memory_type mc)::acc)
       li.jc_logic_info_effects.jc_effect_memories
       acc
   in
   let acc =
-    StringRegionSet.fold
-      (fun (a,r) acc -> 
-	 let st, _ = Hashtbl.find Jc_typing.structs_table a in
-	 (alloc_region_table_name (JCtag st, r),
-	  alloc_table_type (JCtag st))::acc)
-      li.jc_logic_info_effects.jc_effect_alloc_table
+    AllocMap.fold
+      (fun (ac,r) labs acc -> 
+	 (alloc_table_name (ac, r),
+	  alloc_table_type (ac))::acc)
+      li.jc_logic_info_effects.jc_effect_alloc_tables
       acc
   in
   let acc =
-    VariantSet.fold
-      (fun v acc -> 
-	 let t = { logic_type_args = [variant_model_type v];
+    TagMap.fold
+      (fun (v,r) labels acc -> 
+	 let t = { logic_type_args = [root_model_type v];
 		   logic_type_name = "tag_table" }
 	 in
-	 (tag_table_name_vi v, t)::acc)
-      li.jc_logic_info_effects.jc_effect_tag_table
+	 (tag_table_name (v,r), t)::acc)
+      li.jc_logic_info_effects.jc_effect_tag_tables
       acc
   in
     acc
@@ -391,7 +395,7 @@ let invariants_params acc st =
 let rec parents acc st =
   match st.jc_struct_info_parent with
     | None -> acc
-    | Some p -> parents (st::acc) p
+    | Some(p, _) -> parents (st::acc) p
 let parents = parents []
 
 (* Returns every structure (name) that can be used by a function,
@@ -400,10 +404,10 @@ let function_structures params =
   let structures = List.fold_left
     (fun acc vi ->
        match vi.jc_var_info_type with
-	 | JCTpointer(JCtag st, _, _) ->
+	 | JCTpointer(JCtag(st, _), _, _) ->
 	     all_structures st acc
-	 | JCTpointer(JCvariant vi, _, _) ->
-	     List.fold_right all_structures vi.jc_variant_info_roots acc
+	 | JCTpointer(JCroot vi, _, _) ->
+	     List.fold_right all_structures vi.jc_root_info_hroots acc
 	 | _ -> acc)
     StringSet.empty
     params
@@ -422,7 +426,7 @@ let hierarchy_structures h =
 let hierarchies () =
   let h = Hashtbl.fold
     (fun _ (st, _) acc ->
-       StringMap.add (root_name st) st.jc_struct_info_root acc)
+       StringMap.add (root_name st) st.jc_struct_info_hroot acc)
     Jc_typing.structs_table
     StringMap.empty
   in
@@ -514,15 +518,16 @@ let make_assume_all_assocs pp params =
 (* mutable *)
 (***********)
 
-let mutable_memory_type tov =
-  memory_type (tag_or_variant_model_type tov) (tag_id_type tov)
+let mutable_memory_type pc =
+  raw_memory_type (pointer_class_model_type pc)
+    (tag_id_type (pointer_class_root pc))
 
-let committed_memory_type tov =
-  memory_type (tag_or_variant_model_type tov) (simple_logic_type "bool")
+let committed_memory_type pc =
+  raw_memory_type (pointer_class_model_type pc) (simple_logic_type "bool")
 
 let mutable_declaration st acc =
   if st.jc_struct_info_parent = None then
-    let st = JCtag st in
+    let st = JCtag(st, []) in
     (* mutable_T: T tag_id *)
     Param(
       false,
@@ -548,13 +553,13 @@ let assert_mutable e fi =
   if fi.jc_field_info_rep then
     begin
       let st = fi.jc_field_info_struct in
-      let mutable_name = mutable_name (JCtag st) in
-      (*let committed_name = committed_name st.jc_struct_info_root in*)
+      let mutable_name = mutable_name (JCtag(st, [])) in
+      (*let committed_name = committed_name st.jc_struct_info_hroot in*)
       let e_mutable = LApp("select", [LVar mutable_name; e]) in
       (*let e_committed = LApp("select", [LVar committed_name; e]) in*)
       let parent_tag = match st.jc_struct_info_parent with
 	| None -> LVar "bottom_tag"
-	| Some parent -> LVar (tag_name parent)
+	| Some(parent, _) -> LVar (tag_name parent)
       in
       let sub = make_subtag parent_tag e_mutable in
       (*let not_committed =
@@ -580,14 +585,14 @@ let assert_mutable e fi =
   let this = "this" in
   let this_ty =
     { logic_type_name = "pointer";
-      logic_type_args = [simple_logic_type st.jc_struct_info_root] } in
+      logic_type_args = [simple_logic_type st.jc_struct_info_hroot] } in
 
   (* program point *)
   let pp = "program_point" in
   let pp_ty = simple_logic_type "int" in
 
   (* assoc memories with program point => not this.mutable => this.invariant *)
-  let mutable_ty = mutable_memory_type st.jc_struct_info_root in
+  let mutable_ty = mutable_memory_type st.jc_struct_info_hroot in
   let mutable_is_false =
     LPred(
       "eq",
@@ -652,7 +657,7 @@ let not_mutable_implies_invariant this st (li, _) =
   let params = invariant_params [] li in
   
   (* this.mutable <: st *)
-  let mutable_name = mutable_name (JCtag st) in
+  let mutable_name = mutable_name (JCtag(st, [])) in
   let mutable_io = make_subtag
     (LApp("select", [ LVar mutable_name; LVar this ]))
     (LVar (tag_name st))
@@ -660,15 +665,16 @@ let not_mutable_implies_invariant this st (li, _) =
 
   (* invariant *)
   let invariant = 
-    make_logic_pred_call ~label_in_name:false li [LVar this] [] [] 
+    make_logic_pred_call ~label_in_name:false ~region_assoc:[] ~label_assoc:[] li [LVar this]
   in
 
   (* implies *)
   let impl = LImpl(mutable_io, invariant) in
 
   (* params *)
-  let params = (mutable_name, mutable_memory_type (JCtag st))::params in
-  let params = (tag_table_name (JCtag st), tag_table_type (JCtag st))::params in
+  let params = (mutable_name, mutable_memory_type (JCtag(st, [])))::params in
+  let params = (generic_tag_table_name (struct_root st),
+                tag_table_type (struct_root st))::params in
 
   params, impl
 
@@ -677,7 +683,7 @@ let not_mutable_implies_fields_committed this st =
   let fields = rep_fields st in
 
   (* this.mutable <: st *)
-  let mutable_name = mutable_name (JCtag st) in
+  let mutable_name = mutable_name (JCtag(st, [])) in
   let mutable_io = make_subtag
     (LApp("select", [ LVar mutable_name; LVar this ]))
     (LVar (tag_name st))
@@ -687,19 +693,20 @@ let not_mutable_implies_fields_committed this st =
   let fields_pc = List.fold_left
     (fun acc fi ->
        match fi.jc_field_info_type with
-	 | JCTpointer(fi_tov, min, max) ->
+	 | JCTpointer(fi_pc, min, max) ->
 	     let n = fi.jc_field_info_final_name in
 	     let index = "jc_index" in
-	     let committed_name = committed_name fi_tov in
-	     let alloc = alloc_table_name fi_tov in
+	     let committed_name = committed_name fi_pc in
+	     let fi_ac = alloc_class_of_pointer_class fi_pc in
+	     let alloc = generic_alloc_table_name fi_ac in
 	     let params =
-	       [ n, field_memory_type fi;
-		 committed_name, committed_memory_type fi_tov;
-		 alloc, alloc_table_type fi_tov; ]
+	       [ n, memory_type (JCmem_field fi);
+		 committed_name, committed_memory_type fi_pc;
+		 alloc, alloc_table_type fi_ac; ]
 	     in
 	     let this_fi = make_select (LVar n) (LVar this) in
 	     let f = make_shift this_fi (LVar index) in
-	     let eq = make_eq (make_select_committed fi_tov f)
+	     let eq = make_eq (make_select_committed fi_pc f)
 	       (make_bool true)
 	     in
 	     let omin, omax = omin_omax (LVar alloc) this_fi min max in
@@ -717,8 +724,9 @@ let not_mutable_implies_fields_committed this st =
     make_and_list (List.map snd fields_pc) in
 
   (* additional params *)
-  let params = (mutable_name, mutable_memory_type (JCtag st))::params in
-  let params = (tag_table_name (JCtag st), tag_table_type (JCtag st))::params in
+  let params = (mutable_name, mutable_memory_type (JCtag(st, [])))::params in
+  let params = (generic_tag_table_name (struct_root st),
+                tag_table_type (struct_root st))::params in
 
   (* implies *)
   let impl = LImpl(mutable_io, coms) in
@@ -731,8 +739,8 @@ let committed_implies_fully_packed this root =
   let committed_type = committed_memory_type root in
   let mutable_name = mutable_name root in
   let mutable_type = mutable_memory_type root in
-  let tag_table = tag_table_name root in
-  let tag_table_type = tag_table_type root in
+  let tag_table = generic_tag_table_name (pointer_class_root root) in
+  let tag_table_type = tag_table_type (pointer_class_root root) in
 
   (* this.committed = true *)
   let com = LPred(
@@ -775,7 +783,9 @@ let owner_unicity this root =
 
   (* x name and type *)
   let x_name = this^"_2" in
-  let x_type = pointer_type (JCtag root) in
+  let pc = JCtag(root, []) in
+  let ac = alloc_class_of_pointer_class pc in
+  let x_type = pointer_type ac pc in
 
   (* shift indexes *)
   let index1 = "jc_index" in
@@ -787,13 +797,14 @@ let owner_unicity this root =
        try
 	 let name = fi.jc_field_info_final_name in
 	 Printf.printf "***** FIELD %s *****\n%!" name;
-	 let fi_tov = type_tov fi.jc_field_info_type in
-	 let committed_name = committed_name fi_tov in
+	 let fi_pc = type_pc fi.jc_field_info_type in
+	 let committed_name = committed_name fi_pc in
 	 let this_dot_f = make_select (LVar name) (LVar this) in
 	 let x_dot_f = make_select (LVar name) (LVar x_name) in
 
 	 (* indexes, ranges *)
-	 let alloc = alloc_table_name fi_tov in
+	 let fi_ac = alloc_class_of_pointer_class fi_pc in
+	 let alloc = generic_alloc_table_name fi_ac in
 	 let omin1, omax1 = omin_omax (LVar alloc) this_dot_f
 	   (range_min fi.jc_field_info_type)
 	   (range_max fi.jc_field_info_type)
@@ -834,11 +845,12 @@ let owner_unicity this root =
   (* params *)
   let params = List.map
     (fun fi ->
-       [ fi.jc_field_info_final_name, field_memory_type fi;
-	 committed_name (JCtag fi.jc_field_info_root),
-	 committed_memory_type (JCtag fi.jc_field_info_root);
-	 alloc_table_name (JCtag fi.jc_field_info_root),
-	 alloc_table_type (JCtag fi.jc_field_info_root) ])
+       let ac = JCalloc_root (struct_root fi.jc_field_info_struct) in
+       [ fi.jc_field_info_final_name, memory_type (JCmem_field fi);
+	 committed_name (JCtag(fi.jc_field_info_hroot, [])),
+	 committed_memory_type (JCtag(fi.jc_field_info_hroot, []));
+	 generic_alloc_table_name ac,
+	 alloc_table_type ac ])
     reps
   in
   let params = List.flatten params in
@@ -848,7 +860,9 @@ let owner_unicity this root =
 let make_hierarchy_global_invariant acc root =
   (* this *)
   let this = "this" in
-  let this_ty = pointer_type (JCtag root) in
+  let pc = JCtag(root, []) in
+  let ac = alloc_class_of_pointer_class pc in
+  let this_ty = pointer_type ac pc in
 
   (* not mutable => invariant, and their parameters *)
   let structs = hierarchy_structures root in
@@ -873,7 +887,8 @@ let make_hierarchy_global_invariant acc root =
   in
 
   (* committed => fully packed *)
-  let params_cfp, com_fp = committed_implies_fully_packed this (JCtag root) in
+  let params_cfp, com_fp = committed_implies_fully_packed this
+    (JCtag(root, [])) in
   let params = params_cfp@params in
 
   (* unicity of the owner: x.f == y.f && x.f.committed ==> x == y *)
@@ -931,11 +946,11 @@ let assume_field_invariants fi =
   (* keep hierarchies only once *)
   let hl = List.fold_left
     (fun acc (st, _) ->
-       StringMap.add (root_name st) st.jc_struct_info_root acc)
+       StringMap.add (root_name st) st.jc_struct_info_hroot acc)
     StringMap.empty
     invs
   in
-  assume_global_invariants (stringmap_elements hl)
+  assume_global_invariants (StringMap.values hl)
 
 (* Given the parameters of a function, assume all potentially useful
 forall this: st, not this.mutable => invariant *)
@@ -988,25 +1003,25 @@ let components st =
     (fun acc fi ->
        if fi.jc_field_info_rep then
 	 match fi.jc_field_info_type with
-	   | JCTpointer(tov, _, _) -> (fi, tov)::acc
+	   | JCTpointer(pc, _, _) -> (fi, pc)::acc
 	   | _ -> acc
        else acc)
     []
     st.jc_struct_info_fields
 
 let components_by_type st =
-  let compare_tovs s t =
-    compare (tag_or_variant_type_name s) (tag_or_variant_type_name t) in
+  let compare_pcs s t =
+    compare (pointer_class_type_name s) (pointer_class_type_name t) in
   let comps = components st in
   let comps =
     List.sort
-      (fun (_, s) (_, t) -> compare_tovs s t)
+      (fun (_, s) (_, t) -> compare_pcs s t)
       comps
   in
   let rec part prev acc = function
     | [] -> [prev, acc]
     | (fi, si)::tl ->
-	if compare_tovs si prev = 0 then
+	if compare_pcs si prev = 0 then
 	  part prev (fi::acc) tl
 	else
 	  (prev, acc)::(part si [fi] tl)
@@ -1022,7 +1037,8 @@ field of each component of "this" (in "fields") of the
 hierarchy "root" has been set to "value", and only them *)
 let hierarchy_committed_postcond this root fields value =
   let com = committed_name root in
-  let alloc = alloc_table_name root in
+  let ac = alloc_class_of_pointer_class root in
+  let alloc = generic_alloc_table_name ac in
   (* fields information and range *)
   let fields = List.map
     (fun fi ->
@@ -1051,11 +1067,11 @@ let hierarchy_committed_postcond this root fields value =
   (* new values for the fields in their ranges *)
   let com_values = List.map
     (fun (fi, this_fi, omin, omax) ->
-       let fi_tov = type_tov fi.jc_field_info_type in
+       let fi_pc = type_pc fi.jc_field_info_type in
        let index = "jc_index" in
        let range = make_range (LVar index) omin omax in
        let new_value = make_eq
-	 (make_select_committed fi_tov
+	 (make_select_committed fi_pc
 	    (make_shift this_fi (LVar index)))
 	 (LConst(Prim_bool value))
        in
@@ -1070,7 +1086,7 @@ let make_components_postcond this st reads writes committed =
   let comps = components_by_type st in
   let writes =
     List.fold_left
-      (fun acc (tov, _) -> StringSet.add (committed_name tov) acc)
+      (fun acc (pc, _) -> StringSet.add (committed_name pc) acc)
       writes
       comps
   in
@@ -1086,16 +1102,16 @@ let make_components_postcond this st reads writes committed =
   in
   let reads = StringSet.union reads writes in
   let reads = List.fold_left
-    (fun acc (tov, fields) -> StringSet.add
-       (alloc_table_name tov) acc)
+    (fun acc (pc, fields) -> StringSet.add
+       (generic_alloc_table_name (alloc_class_of_pointer_class pc)) acc)
     reads comps
   in
   let postcond =
     make_and_list
       (* for each hierarchy... *)
       (List.map
-	 (fun (tov, fields) -> hierarchy_committed_postcond
-	    this tov fields committed)
+	 (fun (pc, fields) -> hierarchy_committed_postcond
+	    this pc fields committed)
 	 comps)
   in
   postcond, reads, writes  
@@ -1122,14 +1138,15 @@ let make_components_precond this st reads =
        let this_field =
 	 LApp("shift", [ base_field; LVar index_name ])
        in
-       let fi_tov = type_tov fi.jc_field_info_type in
-       let alloc = alloc_table_name fi_tov in
-       let reads = StringSet.add (tag_table_name fi_tov) reads in
+       let fi_pc = type_pc fi.jc_field_info_type in
+       let fi_ac = alloc_class_of_pointer_class fi_pc in
+       let alloc = generic_alloc_table_name fi_ac in
+       let reads = StringSet.add (generic_tag_table_name (pointer_class_root fi_pc)) reads in
        let reads = StringSet.add alloc reads in
        let reads = StringSet.add mutable_name reads in
        (* pre-condition: forall i, valid(x.f+i) => fp(x.f+i) /\ not committed(x.f+i) *)
        let body = make_and
-	 (fully_packed fi_tov this_field)
+	 (fully_packed fi_pc this_field)
 	 (LPred(
 	    "eq",
 	    [ LApp("select", [LVar committed_name; this_field]);
@@ -1150,10 +1167,12 @@ let make_components_precond this st reads =
 
 let pack_declaration st acc =
   let this = "this" in
-  let this_type = pointer_type (JCtag st) in
+  let pc = JCtag(st, []) in
+  let ac = alloc_class_of_pointer_class pc in
+  let this_type = pointer_type ac pc in
   let tag = "tag" in
-  let tag_type = tag_id_type (JCtag st) in
-  let mutable_name = mutable_name (JCtag st) in
+  let tag_type = tag_id_type (struct_root st) in
+  let mutable_name = mutable_name (JCtag(st, [])) in
   let inv, reads = invariant_for_struct (LVar this) st in
   let writes = StringSet.empty in
   let components_post, reads, writes = make_components_postcond (LVar this) st reads writes true in
@@ -1212,11 +1231,13 @@ let pack_declaration st acc =
 (* Unlike Boogie, Jessie has "unpack to S" instead of "unpack from T" *)
 let unpack_declaration st acc =
   let this = "this" in
-  let this_type = pointer_type (JCtag st) in
+  let pc = JCtag(st, []) in
+  let ac = alloc_class_of_pointer_class pc in
+  let this_type = pointer_type ac pc in
   let tag = "tag" in
-  let tag_type = tag_id_type (JCtag st) in
-  let mutable_name = mutable_name (JCtag st) in
-  let committed_name = committed_name (JCtag st) in
+  let tag_type = tag_id_type (struct_root st) in
+  let mutable_name = mutable_name (JCtag(st, [])) in
+  let committed_name = committed_name (JCtag(st, [])) in
   let reads = StringSet.singleton mutable_name in
   let writes = StringSet.singleton mutable_name in
   let reads = StringSet.add committed_name reads in
@@ -1326,7 +1347,7 @@ let tr_valid_inv st acc =
   let valid_inv_type = simple_logic_type "prop" in
   let vi_this = "???",
     { logic_type_name = "pointer" ;
-      logic_type_args = [simple_logic_type st.jc_struct_info_root] } in
+      logic_type_args = [simple_logic_type st.jc_struct_info_hroot] } in
   let logic = Logic(false, valid_inv_name st, vi_this::params,
     valid_inv_type) in
   let acc = logic::acc in
@@ -1336,7 +1357,7 @@ let tr_valid_inv st acc =
   let this_var = LVar this in
   let this_ty =
     { logic_type_name = "pointer";
-      logic_type_args = [simple_logic_type st.jc_struct_info_root] } in
+      logic_type_args = [simple_logic_type st.jc_struct_info_hroot] } in
   let fields_valid_inv = List.map (fun (_, fi) ->
     match fi.jc_field_info_type with
     | JCTpointer(st, _, _) ->
@@ -1365,102 +1386,104 @@ let tr_valid_inv st acc =
     LForall(id, ty, acc)) sem ((this, this_ty)::params) in
   Axiom(valid_inv_axiom_name st, sem)::acc*)
 
-let rec invariant_for_struct this si =
-  let (_, invs) = 
+let rec invariant_for_struct ?pos this si =
+  let _, invs = 
     Hashtbl.find Jc_typing.structs_table si.jc_struct_info_name 
   in
-  let invs = Assertion.mkand
+  let invs = Assertion.mkand ?pos
     ~conjuncts:(List.map 
-       (fun (li, _) -> 
-	  let a = { jc_app_fun = li;
-		    jc_app_args = [this];
-		    jc_app_label_assoc = [];
-		    jc_app_region_assoc = [] }
-	  in
-	  new assertion (JCAapp a)) invs) 
+		  (fun (li, _) -> 
+		     let a = { jc_app_fun = li;
+			       jc_app_args = [this];
+			       jc_app_label_assoc = [];
+			       jc_app_region_assoc = [] }
+		     in
+		       new assertion ?pos (JCAapp a)) invs) 
     ()
   in
     match si.jc_struct_info_parent with
       | None -> invs
-      | Some si -> (* add invariants from the type hierarchy *)
+      | Some(si, _) -> (* add invariants from the type hierarchy *)
 	  let this =
 	    match this#typ with
 	      | JCTpointer (_, a, b) ->
-		  new term_with ~typ:(JCTpointer (JCtag si, a, b)) this
+		  new term_with ~typ:(JCTpointer (JCtag(si, []), a, b)) this
 	      | _ -> assert false (* never happen *)
 	  in
-	  Assertion.mkand 
-	      ~conjuncts:[invs; (invariant_for_struct this si)]
-	    ()
-	    
-
-let code_function (fi, fs, sl) vil =
+	    Assertion.mkand ?pos
+	      ~conjuncts:[invs; (invariant_for_struct ?pos this si)]
+	      ()
+	      
+let code_function (fi, pos, fs, sl) vil =
   begin
     match !Jc_common_options.inv_sem with
-      | InvArguments ->
-	  (* apply arguments invariant policy *)
-	  let invariants =
-	    (* Calculate global invariants. *)
-	    let vitl = 
-	      List.map 
-		(fun vi -> Term.mkvar ~var:vi ()) vil 
-	    in
-	    let global_invariants =
-	      Hashtbl.fold
-		(fun li _ acc -> 
-		   li.jc_logic_info_parameters <- vil;
-		   let a = { jc_app_fun = li;
-			     jc_app_args = vitl;
-			     jc_app_label_assoc = [];
-			     jc_app_region_assoc = [] }
-		   in
-		   (new assertion (JCAapp a)) :: acc)
-		Jc_typing.global_invariants_table []
-	    in
-	    let global_invariants = 
-	      Assertion.mkand ~conjuncts:global_invariants ()
-	    in
-	      (* Calculate invariants for each parameter. *)
-	    let invariants =
-	      List.fold_left
-		(fun acc vi ->
-		   match vi.jc_var_info_type with
-		     | JCTpointer (JCtag st, _, _) ->
-			 Assertion.mkand 
-			   ~conjuncts:
-			   [acc; (invariant_for_struct 
-				    (Term.mkvar ~var:vi ()) st)]
-			   ()
-		     | _ -> acc)
-		(Assertion.mktrue ())
-		fi.jc_fun_info_parameters
-	    in
-	    Assertion.mkand ~conjuncts:[global_invariants; invariants] ()
+      | InvArguments ->  (* apply arguments invariant policy *)
+	  (* Calculate global invariants. *)
+	  let _vitl = 
+	    List.map 
+	      (fun vi -> Term.mkvar ~var:vi ()) vil 
 	  in
-	    (* add invariants to the function precondition *)
+	  let global_invariants =
+	    Hashtbl.fold
+	      (fun li _ acc -> 
+		 (* li.jc_logic_info_parameters <- vil; *)
+		 let a = { jc_app_fun = li;
+			   jc_app_args = (* vitl *)[];
+			   jc_app_label_assoc = [];
+			   jc_app_region_assoc = [] }
+		 in
+		 (new assertion ~mark:(Jc_pervasives.new_label_name ())
+		    ~pos (JCAapp a)) :: acc)
+	      Jc_typing.global_invariants_table []
+	  in
+	  let global_invariants = 
+	    Assertion.mkand ~pos ~conjuncts:global_invariants ()
+	  in
+	  (* Calculate invariants for each parameter. *)
+	  let pre_invariants,post_invariants =
+	    List.fold_left
+	      (fun (acc1,acc2) (valid,vi) ->
+		 match vi.jc_var_info_type with
+		   | JCTpointer (JCtag (st, []), _, _) ->
+		       let inv = 
+			 invariant_for_struct ~pos
+			   (Term.mkvar ~var:vi ()) st
+		       in
+		       ((if valid then
+			   Assertion.mkand ~pos
+			     ~conjuncts: [acc1; inv] ()
+			 else acc1),
+			Assertion.mkand ~pos
+			  ~conjuncts: [acc2; inv] ())
+		   | _ -> (acc1,acc2))
+	      (global_invariants,global_invariants)
+	      fi.jc_fun_info_parameters
+	  in
+	  (* add invariants to the function precondition *)
 	  fs.jc_fun_requires <- 
-	    Assertion.mkand ~conjuncts:[fs.jc_fun_requires; invariants] ();
-	    (* add invariants to the function postcondition *)
-	    if is_purely_exceptional_fun fs then () else
-	      let safety_exists = ref false in
-	      let post = invariants in
-		List.iter
-		  (fun (_, s, b) ->
-		     if s = "safety" then safety_exists := true;
-		     b.jc_behavior_ensures <- 
-		       Assertion.mkand ~conjuncts:[b.jc_behavior_ensures; post] ())
-		  fs.jc_fun_behavior;
-		(* add the 'safety' spec if it does not exist 
-		   (it could exist e.g. from Krakatoa) *)
-		if not !safety_exists then
-		  if Jc_options.verify_invariants_only then
-		    let invariants_b = { default_behavior with jc_behavior_ensures = post } in
-		      fs.jc_fun_behavior <- 
-			(Loc.dummy_position, "invariants", invariants_b) :: fs.jc_fun_behavior;
-		  else
-		    let safety_b = { default_behavior with jc_behavior_ensures = post } in
-		      fs.jc_fun_behavior <- 
-			(Loc.dummy_position, "safety", safety_b) :: fs.jc_fun_behavior;
+	    Assertion.mkand ~pos 
+	    ~conjuncts:[fs.jc_fun_requires; pre_invariants] ();
+	  (* add invariants to the function postcondition *)
+	  if is_purely_exceptional_fun fs then () else
+	    let safety_exists = ref false in
+	    let post = post_invariants in
+	    List.iter
+	      (fun (_, s, b) ->
+		 if s = "safety" then safety_exists := true;
+		 b.jc_behavior_ensures <- 
+		   Assertion.mkand ~pos ~conjuncts:[b.jc_behavior_ensures; post] ())
+	      fs.jc_fun_behavior;
+	    (* add the 'safety' spec if it does not exist 
+	       (it could exist e.g. from Krakatoa) *)
+	    if not !safety_exists then
+	      if Jc_options.verify_invariants_only then
+		let invariants_b = { default_behavior with jc_behavior_ensures = post } in
+		fs.jc_fun_behavior <- 
+		  (Loc.dummy_position, "invariants", invariants_b) :: fs.jc_fun_behavior;
+	      else
+		let safety_b = { default_behavior with jc_behavior_ensures = post } in
+		fs.jc_fun_behavior <- 
+		  (Loc.dummy_position, "safety", safety_b) :: fs.jc_fun_behavior;
       | _ -> ()
   end;
 
