@@ -15,26 +15,26 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_options.mli,v 1.27 2008/04/04 16:10:32 nrousset Exp $ *)
-
+(* $Id: jc_options.mli,v 1.37 2008-11-17 15:48:29 marche Exp $ *)
 
+open Jc_stdlib
+open Jc_env
 
 (*s environment variables *)
 
+val has_floats : bool ref
 val libdir : string
-val libfile : string
+val libfiles : unit -> string list
 
 (*s command-line options *)
 
@@ -49,6 +49,7 @@ val why_opt : string
 val verify_all_offsets : bool
 val verify_invariants_only : bool
 val verify : string list
+val behavior : string
 
 val interprocedural : bool
 val main : string
@@ -64,6 +65,10 @@ val int_model : Jc_env.int_model ref
 val trust_ai : bool
 val fast_ai : bool
 
+val verify_behavior: string -> bool
+
+val set_int_model: int_model -> unit
+
 (*s The log file *)
 
 val log : Format.formatter
@@ -74,12 +79,15 @@ val close_log : unit -> unit
 
 exception Jc_error of Loc.position * string
 
+val jc_error : Loc.position -> ('a, unit, string, 'b) format4 -> 'a
+
 val parsing_error : Loc.position -> ('a, unit, string, 'b) format4 -> 'a
 
-val locs_table : 
+val pos_table : 
   (string, (string * int * int * int * Output.kind option * (string * Rc.rc_value) list)) 
      Hashtbl.t
 
+val position_of_label: string -> Loc.position option
 
 (*
 Local Variables: 
