@@ -15,31 +15,29 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_output.ml,v 1.106 2008-05-23 13:51:39 marche Exp $ *)
+(* $Id: jc_output.ml,v 1.137 2009-01-21 08:34:15 marche Exp $ *)
 
 open Format
 open Jc_env
+open Jc_ast
 open Jc_fenv
 open Jc_pervasives
 open Jc_constructors
 open Jc_output_misc
-open Jc_ast
 open Pp
 
 type jc_decl =
-  | JCfun_def of jc_type * string * var_info list *
+  | JCfun_def of jc_type * string * (bool * var_info) list *
       fun_spec * expr option
   | JCenum_type_def of string * Num.num * Num.num
   | JCvariant_type_def of string * string list
@@ -50,8 +48,8 @@ type jc_decl =
       (* deprecated, all tag definitions of a file are mutually recursive *)
   | JCrec_fun_defs of jc_decl list
   | JCvar_def of jc_type * string * expr option
-  | JClemma_def of string * bool * logic_label list * assertion
-  | JClogic_fun_def of jc_type option * string * logic_label list 
+  | JClemma_def of string * bool * label list * assertion
+  | JClogic_fun_def of jc_type option * string * label list 
       * var_info list * term_or_assertion      
   | JCexception_def of string * exception_info
   | JCglobinv_def of string * assertion
@@ -109,7 +107,14 @@ let unary_op (op, _) = match op with
   | `Unot -> "!"
   | `Ubw_not -> "~"
 
-let real_conversion = Jc_poutput.real_conversion
+let real_conversion fmt rc =
+  match rc with
+    | Integer_to_real -> fprintf fmt "real"
+    | Double_to_real -> fprintf fmt "float_value"
+    | Float_to_real -> fprintf fmt "float_value"
+    | Real_to_integer -> fprintf fmt "integer"
+    | Round_double _ -> fprintf fmt "r_to_d" (* TODO: parameter rounding mode *)
+    | Round_float _ -> fprintf fmt "r_to_s"
 
 let rec pattern fmt p =
   match p#node with
@@ -132,9 +137,9 @@ let rec pattern fmt p =
 	fprintf fmt "%a" const c
 
 let rec term fmt t =
-  if t#name_label <> "" then
+  if t#mark <> "" then
     fprintf fmt "@[(%s : %a)@]" 
-      t#name_label term (new term_with ~name_label:"" t)
+      t#mark term (new term_with ~mark:"" t)
   else
   match t#node with
     | JCTvar vi -> fprintf fmt "%s" vi.jc_var_info_name
@@ -144,7 +149,8 @@ let rec term fmt t =
 	fprintf fmt "@[(%s %a)@]" (unary_op op) term t1
     | JCTif (t1,t2,t3) -> 
 	fprintf fmt "@[(%a ? %a : %a)@]" term t1 term t2 term t3
-    | JCTcast (t, _, si) ->
+    | JCTcast (t, _, si)
+    | JCTbitwise_cast (t, _, si) ->
 	fprintf fmt "(%a :> %s)" term t si.jc_struct_info_name
     | JCTrange_cast (t, ei) ->
 	fprintf fmt "(%a :> %s)" term t ei.jc_enum_info_name
@@ -154,6 +160,10 @@ let rec term fmt t =
 	fprintf fmt "(%a <: %s)" term t si.jc_struct_info_name
     | JCToffset (k,t,_)->
 	fprintf fmt "@[\\offset_m%a(%a)@]" offset_kind k term t
+    | JCTaddress(absolute,t) ->
+	fprintf fmt "@[\\%aaddress(%a)@]" address_kind absolute term t
+    | JCTbase_block (t)->
+	fprintf fmt "@[\\base_block(%a)@]" term t
     | JCTold t -> fprintf fmt "@[\\old(%a)@]" term t
     | JCTat(t,lab) -> fprintf fmt "@[\\at(%a,%a)@]" term t label lab
 (*
@@ -200,10 +210,10 @@ let rec term fmt t =
 let quantifier = Jc_poutput.quantifier
 
 let rec assertion fmt a =
-  if a#name_label <> "" then
+  if a#mark <> "" then
     fprintf fmt "@[(%s : %a)@]" 
-      (a#name_label) assertion
-      (new assertion_with ~name_label:"" a)
+      (a#mark) assertion
+      (new assertion_with ~mark:"" a)
   else
   match a#node with
     | JCAtrue -> fprintf fmt "true"
@@ -215,7 +225,7 @@ let rec assertion fmt a =
     | JCAold a -> fprintf fmt "\\old(%a)" assertion a
     | JCAat(a,lab) -> fprintf fmt "\\at(%a,%a)" assertion a label lab
     | JCAquantifier (q,vi, a)-> 
-	fprintf fmt "@[<v 3>(\\%a %a %s;@\n%a)@]"
+	fprintf fmt "@[(\\%a %a %s;@\n%a)@]"
 	  quantifier q
 	  print_type vi.jc_var_info_type
 	  vi.jc_var_info_name
@@ -258,7 +268,8 @@ let rec assertion fmt a =
 	fprintf fmt ")@]"
     | JCAfalse -> fprintf fmt "false"
     | JCAmutable _ -> assert false (* TODO *)
-    | JCAtagequality _ -> assert false (* TODO *)
+    | JCAeqtype _ -> assert false (* TODO *)
+    | JCAsubtype _ -> assert false (* TODO *)
     | JCAmatch (t, pal) ->
 	fprintf fmt "@[<v 2>match %a with@ " term t;
 	List.iter
@@ -266,22 +277,29 @@ let rec assertion fmt a =
 	     pattern p assertion a) pal;
 	fprintf fmt "end@]"
 
-let rec location_set fmt = function
-  | JCLSvar vi-> 
-      fprintf fmt "%s" vi.jc_var_info_name
-  | JCLSderef (locset, _, fi, _) ->
-      fprintf fmt "%a.%s" location_set locset fi.jc_field_info_name
-  | JCLSrange (locset, t1, t2) ->
-      fprintf fmt "(%a + [%a..%a])" location_set locset 
-	(print_option term) t1 (print_option term) t2
+let rec location_set fmt locs = 
+  match locs#node with
+    | JCLSvar vi-> 
+	fprintf fmt "%s" vi.jc_var_info_name
+    | JCLSderef (locs, _, fi, _) ->
+	fprintf fmt "%a.%s" location_set locs fi.jc_field_info_name
+    | JCLSrange (locset, t1, t2) ->
+	fprintf fmt "(%a + [%a..%a])" location_set locset 
+	  (print_option term) t1 (print_option term) t2
+    | JCLSrange_term (locset, t1, t2) ->
+	fprintf fmt "(%a + [%a..%a])" term locset 
+	  (print_option term) t1 (print_option term) t2
 
-let rec location fmt = function
-  | JCLvar vi -> 
-      fprintf fmt "%s" vi.jc_var_info_name
-  | JCLderef (locset, _, fi,_) ->
-      fprintf fmt "%a.%s" location_set locset fi.jc_field_info_name
-  | JCLat (loc, lab) ->
-      fprintf fmt "\\at(%a,%a)" location loc label lab
+let rec location fmt loc = 
+  match loc#node with
+    | JCLvar vi -> 
+	fprintf fmt "%s" vi.jc_var_info_name
+    | JCLderef (locs, _, fi,_) ->
+	fprintf fmt "%a.%s" location_set locs fi.jc_field_info_name
+    | JCLderef_term (t1, fi) ->
+	fprintf fmt "%a.%s" term t1 fi.jc_field_info_name
+    | JCLat (loc, lab) ->
+	fprintf fmt "\\at(%a,%a)" location loc label lab
 
 let behavior fmt (_loc,id,b) =
   fprintf fmt "@\n@[<v 2>behavior %s:" id;
@@ -299,7 +317,10 @@ let behavior fmt (_loc,id,b) =
   
 let print_spec fmt s =
   fprintf fmt "@\n@[<v 2>  requires @[%a@];" assertion s.jc_fun_requires;
-  List.iter (behavior fmt) s.jc_fun_behavior;
+  Option_misc.iter 
+    (fun t -> fprintf fmt "@\n@[<v 2>  decreases @[%a@];" term t) 
+    s.jc_fun_decreases;
+  List.iter (behavior fmt) (s.jc_fun_default_behavior :: s.jc_fun_behavior);
   fprintf fmt "@]"
 
 let call_bin_op _op =
@@ -309,9 +330,9 @@ let call_bin_op _op =
   raise Not_found
 
 let rec expr fmt e =
-  if e#name_label <> "" then
+  if e#mark <> "" then
     fprintf fmt "@[(%s : %a)@]" 
-      e#name_label expr (new expr_with ~name_label:"" e)
+      e#mark expr (new expr_with ~mark:"" e)
   else
     match e#node with
       | JCEconst c -> const fmt c
@@ -329,7 +350,8 @@ let rec expr fmt e =
 	  fprintf fmt "%a.%s = %a" expr e1 fi.jc_field_info_name expr e2
       | JCEinstanceof(e, si) ->
 	  fprintf fmt "(%a <: %s)" expr e si.jc_struct_info_name
-      | JCEcast(e, si) ->
+      | JCEcast(e, si)
+      | JCEbitwise_cast(e, si) ->
 	  fprintf fmt "(%a :> %s)" expr e si.jc_struct_info_name
       | JCErange_cast(e, ri) ->
 	  fprintf fmt "(%a :> %s)" expr e ri.jc_enum_info_name
@@ -339,6 +361,10 @@ let rec expr fmt e =
 	  fprintf fmt "@[(%a ? %a : %a)@]" expr e1 expr e2 expr e3
       | JCEoffset(k,e, _) ->
 	  fprintf fmt "\\offset_m%a(%a)" offset_kind k expr e
+      | JCEaddress(absolute,e) ->
+	  fprintf fmt "\\%aaddress(%a)" address_kind absolute expr e
+      | JCEbase_block(e) ->
+	  fprintf fmt "\\base_block(%a)" expr e
       | JCEalloc(e, si) ->
 	  fprintf fmt "(new %s[%a])" si.jc_struct_info_name expr e
       | JCEfree e ->
@@ -349,8 +375,16 @@ let rec expr fmt e =
       | JCElet(vi,None,e2) -> 
 	  fprintf fmt "@[%a %s; %a@]"  
 	    print_type vi.jc_var_info_type vi.jc_var_info_name expr e2
-      | JCEassert a -> 
-	  fprintf fmt "@\nassert %a;" assertion a
+      | JCEassert(behav,asrt,a) -> 
+	  fprintf fmt "@\n%a %a%a;" 
+	    asrt_kind asrt
+	    (print_list_delim 
+	       (constant_string "for ") (constant_string ": ") 
+	       comma string)
+	    behav
+	    assertion a
+      | JCEcontract(req,dec,vi_result,behs,e) ->
+	  assert false (* TODO *)
       | JCEblock l ->
           block fmt l
       | JCEreturn_void ->
@@ -378,8 +412,15 @@ let rec expr fmt e =
       | JCEshift(e1, e2) -> 
 	  fprintf fmt "@[(%a + %a)@]" expr e1 expr e2
       | JCEloop(la, e) ->
-          fprintf fmt "@\n@[invariant %a; %a@\nwhile (true)%a@]"
-            assertion la.jc_loop_invariant
+          fprintf fmt "@\n@[%a%a@\nwhile (true)%a@]"
+	  (print_list nothing 
+	     (fun fmt (behav,inv) -> fprintf fmt "@\ninvariant %a%a;"
+		(print_list_delim 
+		   (constant_string "for ") (constant_string ": ") 
+		   comma string)
+		behav
+		assertion inv))
+	    la.jc_loop_invariant
             (print_option (fun fmt t -> fprintf fmt "@\nvariant %a;" term t))
             la.jc_loop_variant
             expr e
@@ -416,15 +457,25 @@ and block fmt b =
 let param fmt vi =
   fprintf fmt "%a %s" print_type vi.jc_var_info_type vi.jc_var_info_name
 
+let fun_param fmt (valid,vi) =
+  fprintf fmt "%s%a %s" 
+    (if valid then "" else "!")
+    print_type vi.jc_var_info_type vi.jc_var_info_name
+
 let field fmt fi =
   fprintf fmt "@\n";
   if fi.jc_field_info_rep then
     fprintf fmt "rep ";
-  fprintf fmt "%a %s;" 
-    print_type fi.jc_field_info_type fi.jc_field_info_name
+  fprintf fmt "%a %s" 
+    print_type fi.jc_field_info_type fi.jc_field_info_name;
+  match fi.jc_field_info_bitsize with
+    | Some bitsize ->
+	fprintf fmt ": %d;" bitsize
+    | None -> 
+	fprintf fmt ";"
 
 let invariant fmt (id, vi, a) =
-  fprintf fmt "@\ninvariant %s(%s) = %a;"
+  fprintf fmt "@\n@[invariant %s(%s) =@ %a;@]"
     id vi.jc_var_info_name assertion a
 
 let term_or_assertion fmt = function
@@ -436,6 +487,18 @@ let term_or_assertion fmt = function
       fprintf fmt "reads \\nothing;"
   | JCReads locs -> 
       fprintf fmt "reads %a;" (print_list comma location) locs
+  | JCInductive l ->
+      fprintf fmt "{@\n@[<v 2>%a@]@\n}" 
+	(print_list newline 
+	   (fun fmt (id,labels,e) -> 
+	      fprintf fmt "case %s" id#name ;
+	      if labels <> [] then
+		fprintf fmt "%a" 
+		  (print_list_delim lbrace rbrace comma label) labels;	      
+	      fprintf fmt ": %a;@\n" 
+		assertion e)) l
+
+  
 
 let print_super fmt = function
   | None -> ()
@@ -456,9 +519,10 @@ let string_of_annotation_policy p =
   match p with
     | Jc_env.AnnotNone -> "None"
     | Jc_env.AnnotInvariants -> "Invariants"
-    | Jc_env.AnnotWeakPre -> "WeakPre"
+    | Jc_env.AnnotElimPre -> "ElimPre"
     | Jc_env.AnnotStrongPre -> "StrongPre"
-
+    | Jc_env.AnnotWeakPre -> "WeakPre"
+ 
 let string_of_abstract_domain p =
   match p with
     | Jc_env.AbsNone -> "None"
@@ -485,7 +549,7 @@ let rec print_decl fmt d =
         fprintf fmt "# IntModel = %s@\n" (string_of_int_model p)
     | JCfun_def(ty,id,params,spec,body) ->
 	fprintf fmt "@\n@[%a %s(@[%a@])%a%a@]@." print_type ty id
-	  (print_list comma param) params 
+	  (print_list comma fun_param) params 
 	  print_spec spec 
 	  (print_option_or_default "\n;" expr) body
     | JCenum_type_def(id,min,max) ->
