@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(* $Id: jc_poutput.ml,v 1.9 2008-05-23 13:51:39 marche Exp $ *)
+(* $Id: jc_poutput.ml,v 1.38 2009-01-21 08:34:15 marche Exp $ *)
 
 open Format
 open Jc_env
@@ -37,6 +35,11 @@ open Jc_ast
 open Jc_output_misc
 open Pp
 
+let is_not_true (p : Jc_ast.pexpr) =
+  match p#node with
+    | JCPEconst (JCCboolean true) -> false
+    | _ -> true
+
 let bin_op = function
   | `Blt -> "<"
   | `Bgt -> ">"
@@ -69,11 +72,6 @@ let unary_op = function
   | `Upostfix_dec | `Uprefix_dec -> "--"
   | `Ubw_not -> "~"
 
-let real_conversion fmt rc =
-  match rc with
-    | Integer_to_real -> fprintf fmt "real"
-    | Real_to_integer -> fprintf fmt "integer"
-
 let rec ppattern fmt p =
   match p#node with
     | JCPPstruct(st, lbls) ->
@@ -107,7 +105,7 @@ let rec pexpr fmt e =
     | JCPEvar vi -> 
 	fprintf fmt "%s" vi
     | JCPEbinary (e1, op, e2) ->
-	fprintf fmt "@[(%a %s %a)@]" pexpr e1 (bin_op op) pexpr e2
+	fprintf fmt "@[<hv 2>(%a %s@ %a)@]" pexpr e1 (bin_op op) pexpr e2
     | JCPEunary((`Upostfix_dec | `Upostfix_inc) as op,e1) ->
 	fprintf fmt "@[(%a %s)@]" pexpr e1 (unary_op op) 
     | JCPEunary(op,e1) ->
@@ -126,13 +124,17 @@ let rec pexpr fmt e =
     | JCPEassign_op (v, op, e) -> 
 	fprintf fmt "%a %s= %a" pexpr v (bin_op op) pexpr e
     | JCPEcast (e, si) ->
-	fprintf fmt "(%a :> %s)" pexpr e si
+	fprintf fmt "(%a :> %a)" pexpr e ptype si
     | JCPEalloc (e, si) ->
 	fprintf fmt "(new %s[%a])" si pexpr e 
     | JCPEfree (e) ->
 	fprintf fmt "(free(%a))" pexpr e 
     | JCPEoffset(k,e) ->
 	fprintf fmt "\\offset_m%a(%a)" offset_kind k pexpr e 
+    | JCPEaddress(absolute,e) ->
+	fprintf fmt "\\%aaddress(%a)" address_kind absolute pexpr e 
+    | JCPEbase_block(e) ->
+	fprintf fmt "\\base_block(%a)" pexpr e 
     | JCPEinstanceof (e, si) ->
 	fprintf fmt "(%a <: %s)" pexpr e si
     | JCPEderef (e, fi) -> 
@@ -153,13 +155,16 @@ let rec pexpr fmt e =
     | JCPErange(t1,t2) -> 
 	fprintf fmt "@[[%a..%a]@]" (print_option pexpr) t1 (print_option pexpr) t2
     | JCPEquantifier (q,ty,vil, a)-> 
-	fprintf fmt "@[<v 3>(\\%a %a %a;@\n%a)@]"
+	fprintf fmt "@[<hv 2>(\\%a %a %a;@\n%a)@]"
 	  quantifier q
 	  ptype ty
 	  (print_list comma string) vil
 	  pexpr a
     | JCPEmutable _ -> assert false (* TODO *)
-    | JCPEtagequality _ -> assert false (* TODO *)
+    | JCPEeqtype(tag1,tag2) -> 
+	fprintf fmt "\\typeeq(%a,%a)" ptag tag1 ptag tag2
+    | JCPEsubtype(tag1,tag2) -> 
+	fprintf fmt "(%a <: %a)" ptag tag1 ptag tag2
     | JCPEreturn (e) ->
 	fprintf fmt "@\n(return %a)" pexpr e
     | JCPEunpack _ -> assert false (* TODO *) 
@@ -181,14 +186,28 @@ let rec pexpr fmt e =
     | JCPEbreak lab -> 
 	fprintf fmt "@\n(break %s)" lab
     | JCPEwhile (e, invariant,variant, s)-> 
-	fprintf fmt "@\n@[invariant %a;%a@\nwhile (%a)%a@]"
-	  pexpr invariant 
+	fprintf fmt "@\n@[%a%a@\nwhile (%a)%a@]"
+	  (print_list nothing 
+	     (fun fmt (behav,inv) -> fprintf fmt "@\ninvariant %a%a;"
+		(print_list_delim 
+		   (constant_string "for ") (constant_string ": ") 
+		   comma string)
+		behav
+		pexpr inv))
+	  invariant
 	  (print_option (fun fmt t -> fprintf fmt "@\nvariant %a;" pexpr t))
 	  variant
 	  pexpr e block [s]
     | JCPEfor (inits, cond, updates, invariant,variant, body)-> 
-	fprintf fmt "@\n@[invariant %a;%a@\nfor (%a ; %a ; %a)%a@]"
-	  pexpr invariant 
+	fprintf fmt "@[%a%a@\nfor (%a ; %a ; %a)%a@]"
+	  (print_list nothing 
+	     (fun fmt (behav,inv) -> fprintf fmt "@\ninvariant %a%a;"
+		(print_list_delim 
+		   (constant_string "for ") (constant_string ": ") 
+		   comma string)
+		behav
+		pexpr inv))
+	  invariant 
 	  (print_option (fun fmt t -> fprintf fmt "@\nvariant %a;" pexpr t))
 	  variant
 	  (print_list comma pexpr) inits 
@@ -199,13 +218,33 @@ let rec pexpr fmt e =
     | JCPEdecl (ty,vi, Some e)-> 
 	fprintf fmt "@\n(var %a %s = %a)" 
 	  ptype ty vi pexpr e
-    | JCPEassert(a)-> 
-	fprintf fmt "@\n(assert %a)" pexpr a
+    | JCPEassert(behav,asrt,a)-> 
+	fprintf fmt "@\n(%a %a%a)" 
+	  asrt_kind asrt
+	  (print_list_delim 
+	     (constant_string "for ") (constant_string ": ") 
+	     comma string)
+	  behav
+	  pexpr a
+    | JCPEcontract(req,dec,behs,e) -> 
+	fprintf fmt "@\n@[<v 2>( ";	
+	Option_misc.iter 
+	  (fun e -> if is_not_true e then
+	     fprintf fmt "requires %a;@\n" pexpr e) req;
+	Option_misc.iter (fprintf fmt "decreases %a;@\n" pexpr) dec;
+	List.iter (behavior fmt) behs;
+	fprintf fmt "@\n{ %a@ })@]" pexpr e 	
     | JCPEblock l -> block fmt l
     | JCPEswitch (e, csl) ->
 	fprintf fmt "@\n@[<v 2>switch (%a) {%a@]@\n}"
 	  pexpr e (print_list nothing case) csl
 
+and ptag fmt tag =
+  match tag#node with
+    | JCPTtag id -> string fmt id#name
+    | JCPTbottom -> string fmt "\\bottom"
+    | JCPTtypeof e -> fprintf fmt "\\typeof(%a)" pexpr e
+
 and handler fmt (ei,vio,s) =
   fprintf fmt "@\n@[<v 2>catch %s %s %a@]"
     ei#name vio
@@ -230,52 +269,83 @@ and case fmt (c,sl) =
   in
   fprintf fmt "%a%a" (print_list nothing onecase) c pexpr sl
 
+and behavior fmt (_loc,id,throws,assumes,requires,assigns,ensures) =
+  fprintf fmt "@\n@[<v 2>behavior %s:" id;
+  Option_misc.iter (fprintf fmt "@\nassumes %a;" pexpr) assumes;
+  Option_misc.iter (fprintf fmt "@\nrequires %a;" pexpr) requires;
+  Option_misc.iter 
+    (fun id -> fprintf fmt "@\nthrows %s;" id#name) throws;
+  Option_misc.iter 
+    (fun (_,locs) -> fprintf fmt "@\nassigns %a;" 
+       (print_list_or_default "\\nothing" comma pexpr) locs)
+    assigns;
+  fprintf fmt "@\nensures %a;@]" pexpr ensures
+
 let pclause fmt = function
   | JCCrequires e -> 
-      fprintf fmt "@\n@[<v 2>  requires @[%a@];@]" pexpr e
-  | JCCbehavior(_loc,id,throws,assumes,requires,assigns,ensures) ->
-      fprintf fmt "@\n@[<v 2>behavior %s:" id;
-      Option_misc.iter (fprintf fmt "@\nassumes %a;" pexpr) assumes;
-      Option_misc.iter (fprintf fmt "@\nrequires %a;" pexpr) requires;
-      Option_misc.iter 
-	(fun id -> fprintf fmt "@\nthrows %s;" id#name) throws;
-      Option_misc.iter 
-	(fun (_,locs) -> fprintf fmt "@\nassigns %a;" 
-	  (print_list_or_default "\\nothing" comma pexpr) locs)
-	assigns;
-      fprintf fmt "@\nensures %a;@]" pexpr ensures
+      if is_not_true e then
+	fprintf fmt "@\n@[<v 2>  requires @[%a@];@]" pexpr e
+  | JCCdecreases e -> 
+      fprintf fmt "@\n@[<v 2>  decreases @[%a@];@]" pexpr e
+  | JCCbehavior b -> behavior fmt b
 
 let param fmt (ty,vi) =
   fprintf fmt "%a %s" ptype ty vi
 
-let field fmt (rep,ty,fi) =
+let fun_param fmt (valid,ty,vi) =
+  fprintf fmt "%s%a %s" (if valid then "" else "! ") ptype ty vi
+
+let field fmt (rep,ty,fi,bitsize) =
   fprintf fmt "@\n";
   if rep then
     fprintf fmt "rep ";
-  fprintf fmt "%a %s;" 
-    ptype ty fi
+  fprintf fmt "%a %s" 
+    ptype ty fi;
+  match bitsize with
+    | Some bitsize ->
+	fprintf fmt ": %d;" bitsize
+    | None -> 
+	fprintf fmt ";"
 
 let invariant fmt (id, vi, a) =
-  fprintf fmt "@\ninvariant %s(%s) = %a;"
+  fprintf fmt "@\n@[<hv 2>invariant %s(%s) =@ %a;@]"
     id#name vi pexpr a
 
 let reads_or_expr fmt = function
-  | JCreads [] -> 
-      fprintf fmt "reads \\nothing;"
+  | JCreads [] -> ()
   | JCreads el -> 
-      fprintf fmt "reads %a;" (print_list comma pexpr) el
+      fprintf fmt "@ reads %a;" (print_list comma pexpr) el
   | JCexpr e -> 
-      fprintf fmt "=@\n%a" pexpr e
+      fprintf fmt " =@\n%a" pexpr e
+  | JCinductive l ->
+      fprintf fmt " {@\n@[<v 2>%a@]@\n}" 
+	(print_list newline
+	   (fun fmt (id,labels,e) -> 
+	      fprintf fmt "case %s@[%a@]: %a;@\n" id#name 
+		(print_list_delim lbrace rbrace comma label) labels
+		pexpr e)) l
+
+let type_params_decl fmt = function
+  | [] -> ()
+  | l -> fprintf fmt "<%a>" (print_list comma Pp.string) l
+
+let type_params fmt = function
+  | [] -> ()
+  | l -> fprintf fmt "<%a>" (print_list comma ptype) l
+
+let super_option fmt = function
+  | None -> ()
+  | Some(s, p) -> fprintf fmt "%s%a with " s type_params p
 
 let rec pdecl fmt d =
   match d#node with
     | JCDfun(ty,id,params,clauses,body) ->
-	fprintf fmt "@\n@[%a %s(@[%a@])%a%a@]@." ptype ty id#name
-	  (print_list comma param) params 
+	fprintf fmt "@\n@[%a %s(@[%a@])%a%a@]@\n" ptype ty id#name
+	  (print_list comma fun_param) params 
 	  (print_list nothing pclause) clauses
 	  (print_option_or_default "\n;" pexpr) body
     | JCDenum_type(id,min,max) ->
-	fprintf fmt "@\n@[type %s = %s..%s@]@."
+	fprintf fmt "@\n@[type %s = %s..%s@]@\n"
 	  id (Num.string_of_num min) (Num.string_of_num max)
     | JCDvariant_type(id, tags) ->
 	fprintf fmt "@\n@[type %s = [" id;
@@ -283,53 +353,52 @@ let rec pdecl fmt d =
 	  (fun fmt () -> fprintf fmt " | ")
 	  (fun fmt tag -> fprintf fmt "%s" tag#name)
 	  fmt tags;
-	fprintf fmt "]@]@."
-    | JCDunion_type(id, tags) ->
+	fprintf fmt "]@]@\n"
+    | JCDunion_type(id,discriminated,tags) ->
 	fprintf fmt "@\n@[type %s = [" id;
 	print_list
-	  (fun fmt () -> fprintf fmt " & ")
+	  (fun fmt () -> fprintf fmt " %c " 
+	     (if discriminated then '^' else '&'))
 	  (fun fmt tag -> fprintf fmt "%s" tag#name)
 	  fmt tags;
-	fprintf fmt "]@]@."
-    | JCDtag(id, Some super, fields, invs) ->
-	fprintf fmt "@\n@[<v 2>tag %s = %s with {%a%a@]@\n}@."
-	  id super (print_list space field) fields 
-	  (print_list space invariant) invs
-    | JCDtag(id, None, fields, invs) ->
-	fprintf fmt "@\n@[<v 2>tag %s = {%a%a@]@\n}@."
-	  id (print_list space field) fields 
+	fprintf fmt "]@]@\n"
+    | JCDtag(id, params, super, fields, invs) ->
+	fprintf fmt "@\n@[<v 2>tag %s%a = %a{%a%a@]@\n}@\n"
+          id
+          type_params_decl params
+          super_option super
+          (print_list space field) fields 
 	  (print_list space invariant) invs
     | JCDvar(ty,id,init) ->
-	fprintf fmt "@\n@[%a %s%a;@]@." ptype ty id
+	fprintf fmt "@\n@[%a %s%a;@]@\n" ptype ty id
 	  (print_option (fun fmt e -> fprintf fmt " = %a" pexpr e)) init
     | JCDlemma(id,is_axiom,lab,a) ->
 	fprintf fmt "@\n@[%s %s" (if is_axiom then "axiom" else "lemma") id;
-	if lab <> [] then 
-	  fprintf fmt "%a" 
-	    (print_list_delim lbrace rbrace comma label) lab;
-	fprintf fmt " :@\n%a@]@." pexpr a
+	fprintf fmt "@[%a@]" 
+	  (print_list_delim lbrace rbrace comma label) lab;
+	fprintf fmt " :@\n%a@]@\n" pexpr a
     | JCDglobal_inv(id,a) ->
-	fprintf fmt "@\n@[invariant %s :@\n%a@]@." id pexpr a
+	fprintf fmt "@\n@[invariant %s :@\n%a@]@\n" id pexpr a
     | JCDexception(id,tyopt) ->
-	fprintf fmt "@\n@[exception %s of %a@]@." id
+	fprintf fmt "@\n@[exception %s of %a@]@\n" id
 	  (print_option ptype) tyopt
     | JCDlogic_var (ty, id, body) ->
-	fprintf fmt "@\n@[logic %a %s %a@]@." 
+	fprintf fmt "@\n@[logic %a %s %a@]@\n" 
 	  ptype ty id
 	  (print_option (fun fmt e -> fprintf fmt "=@\n%a" pexpr e)) body 
     | JCDlogic (None, id, labels, params, body) ->
-	fprintf fmt "@\n@[logic %s%a(@[%a@]) %a@]@." 
+	fprintf fmt "@\n@[logic %s@[%a@](@[%a@])%a@]@\n" 
 	  id (print_list_delim lbrace rbrace comma label) labels
 	  (print_list comma param) params
 	  reads_or_expr body 
     | JCDlogic (Some ty, id, labels, params, body) ->
-	fprintf fmt "@\n@[logic %a %s%a(@[%a@]) %a@]@." 
+	fprintf fmt "@\n@[logic %a %s@[%a@](@[%a@])%a@]@\n" 
 	  ptype ty 
 	  id (print_list_delim lbrace rbrace comma label) labels
 	  (print_list comma param) params
 	  reads_or_expr body 
     | JCDlogic_type id ->
-	fprintf fmt "@\n@[logic type %s@]@." id
+	fprintf fmt "@\n@[logic type %s@]@\n" id
     | JCDinvariant_policy p ->
         fprintf fmt "# InvariantPolicy = %s@\n" (string_of_invariant_policy p)  
     | JCDseparation_policy p ->
@@ -340,14 +409,18 @@ let rec pdecl fmt d =
         fprintf fmt "# AbstractDomain = %s@\n" (string_of_abstract_domain p)
     | JCDint_model p ->
         fprintf fmt "# IntModel = %s@\n" (string_of_int_model p)
-   
-let rec pdecls fmt d =
-  match d with
+    | JCDaxiomatic(id,l) ->
+	fprintf fmt "@\n@[axiomatic %s {@\n@[<v 2>%a@]@\n}@]@\n" id
+	  (print_list space pdecl) l
+	
+
+and pdecls fmt (l : pexpr decl list) =
+  match l with
     | [] -> ()
     | d::r -> pdecl fmt d; pdecls fmt r
 
 (*
 Local Variables: 
-compile-command: "LC_ALL=C make -j -C .. bin/jessie.byte bin/krakatoa.byte"
+compile-command: "LC_ALL=C make -j -C .. byte"
 End: 
 *)
