@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: output.ml,v 1.32 2008/05/07 09:54:00 filliatr Exp $ i*)
+(*i $Id: output.ml,v 1.43 2008-11-05 14:03:16 filliatr Exp $ i*)
 
 open Lexing
 open Format
@@ -58,7 +56,7 @@ type term =
   | LVar of string
   | LVarAtLabel of string * string     (*r x@L *)
   | Tnamed of string * term
-;;
+  | TIf of term * term * term
 
 let rec iter_term f t =
   match t with
@@ -67,7 +65,9 @@ let rec iter_term f t =
   | LVar(id) -> f id
   | LVarAtLabel(id,l) -> f id
   | Tnamed(_,t) -> iter_term f t
-;;
+  | TIf(t1,t2,t3) -> 
+      iter_term f t1; iter_term f t2; iter_term f t3
+
 
 let rec fprintf_term form t =
   match t with
@@ -88,6 +88,9 @@ let rec fprintf_term form t =
   | LVar(id) -> fprintf form "%s" id
   | LVarAtLabel(id,l) -> fprintf form "%s@@%s" id l
   | Tnamed(lab,t) -> fprintf form "(%s : %a)" lab fprintf_term t
+  | TIf(t1,t2,t3) -> 
+      fprintf form "@[<hv 1>(if %a@ then %a@ else %a)@]" 
+	fprintf_term t1 fprintf_term t2 fprintf_term t3
 
 type logic_type = 
     { logic_type_name : string;
@@ -122,6 +125,11 @@ let rec unname a =
     | LNamed(_,a) -> unname a
     | _ -> a
 
+let is_not_true a =
+  match unname a with
+    | LTrue -> false
+    | _ -> true
+
 let make_or a1 a2 =
   match (unname a1,unname a2) with
     | (LTrue,_) -> LTrue
@@ -187,7 +195,7 @@ let rec fprintf_logic_type form t =
 	fprintf form "%a %s" fprintf_logic_type x t.logic_type_name
     | l ->
 	fprintf form "(%a) %s" 
-	  (print_list comma fprintf_logic_type) l
+	  (print_list simple_comma fprintf_logic_type) l
 	  t.logic_type_name
 
 let rec fprintf_assertion form a =
@@ -224,6 +232,14 @@ let rec fprintf_assertion form a =
   | LExists(id,t,a) -> 
       fprintf form "@[<hv 1>(exists %s:%a.@ %a)@]" 
 	id fprintf_logic_type t fprintf_assertion a
+  | LPred("le",[t1;t2]) ->
+      fprintf form "@[(%a <= %a)@]" 
+	fprintf_term t1
+	fprintf_term t2
+  | LPred("ge",[t1;t2]) ->
+      fprintf form "@[(%a >= %a)@]" 
+	fprintf_term t1
+	fprintf_term t2
   | LPred("eq",[t1;t2]) ->
       fprintf form "@[(%a = %a)@]" 
 	fprintf_term t1
@@ -236,7 +252,8 @@ let rec fprintf_assertion form a =
       fprintf form "@[%s(%a" id fprintf_term t;
       List.iter (fun t -> fprintf form ",@ %a" fprintf_term t) tl;
       fprintf form ")@]"
-  | LPred (id, []) -> assert false
+  | LPred (id, []) -> 
+      fprintf form "%s" id
   | LNamed (n, a) ->
       fprintf form "@[(%s:@ %a)@]" n fprintf_assertion a
 ;;
@@ -306,7 +323,7 @@ let rec fprintf_type anon form t =
     | Annot_type(p,t,reads,writes,q,signals) ->
 	begin
 	  fprintf form "@[@[<hv 2>{ "; 
-	  if p <> LTrue 
+	  if is_not_true p  
 	  then fprintf_assertion form p;
 	  fprintf form "}@]@ %a@ " (fprintf_type anon) t;
 	  begin
@@ -396,19 +413,21 @@ let make_and_expr a1 a2 =
     | (_,_) -> And(a1,a2)
 
 
-let make_app_rec f l = 
+let make_app_rec ~logic f l = 
   let rec make_rec accu = function
     | [] -> accu
     | e::r -> make_rec (App(accu,e)) r
   in
   match l with
-    | [] -> make_rec f [Void]
+    | [] -> if logic then make_rec f [] else make_rec f [Void]
     | l -> make_rec f l
 ;;
 
-let make_app id l = make_app_rec (Var id) l
+let make_app id l = make_app_rec ~logic:false (Var id) l
 
-let make_app_e = make_app_rec
+let make_logic_app id l = make_app_rec ~logic:true (Var id) l
+
+let make_app_e = make_app_rec ~logic:false
 
 let make_while cond inv var e =
   let body = 
@@ -612,9 +631,11 @@ type why_decl =
   | Param of bool * string * why_type         (*r parameter in why *)
   | Def of string * expr               (*r global let in why *)
   | Logic of bool * string * (string * logic_type) list * logic_type    (*r logic decl in why *)
+  | Predicate of bool * string * (string * logic_type) list * assertion  
+  | Inductive of bool * string * (string * logic_type) list *  
+      (string * assertion) list (*r inductive definition *)
   | Axiom of string * assertion         (*r Axiom *)
   | Goal of string * assertion         (*r Goal *)
-  | Predicate of bool * string * (string * logic_type) list * assertion  
   | Function of bool * string * (string * logic_type) list * logic_type * term
   | Type of string * string list
   | Exception of string * logic_type option
@@ -630,6 +651,7 @@ let get_why_id d =
     | Goal(id,_) 
     | Predicate(_,id,_,_) 
     | Function(_,id,_,_,_) 
+    | Inductive(_,id,_,_)
     | Type (id,_) 
     | Exception(id,_) -> id
 
@@ -640,10 +662,13 @@ let iter_why_decl f d =
     | Logic(_,id,args,t) -> 
 	List.iter (fun (_,t) -> iter_logic_type f t) args;
 	iter_logic_type f t
-    | Goal(id,t) | Axiom(id,t) -> iter_assertion f t
+    | Inductive(_,id,args,cases) ->
+	List.iter (fun (_,t) -> iter_logic_type f t) args;
+	List.iter (fun (_,a) -> iter_assertion f a) cases
     | Predicate(_,id,args,p) -> 
 	List.iter (fun (_,t) -> iter_logic_type f t) args;
 	iter_assertion f p
+    | Goal(id,t) | Axiom(id,t) -> iter_assertion f t
     | Function(_,id,args,t,p) -> 
 	List.iter (fun (_,t) -> iter_logic_type f t) args;
 	iter_logic_type f t;
@@ -706,6 +731,14 @@ let fprintf_why_decl form d =
 	  (if b then "external " else "") id 
 	  (print_list comma (fun fmt (id,t) -> fprintf_logic_type fmt t)) args
 	  fprintf_logic_type t 
+    | Inductive(b,id,args,cases) ->
+	fprintf form "@[<hv 1>%sinductive %s: @[%a -> prop@] =@\n@[<v 0>%a@]@\n@."
+	  (if b then "external " else "") id 
+	  (print_list comma (fun fmt (id,t) -> fprintf_logic_type fmt t)) args
+	  (print_list newline 
+	     (fun fmt (id,a) ->
+		fprintf form "| %s: @[%a@]" id fprintf_assertion a))
+	  cases
     | Axiom(id,p) ->
 	fprintf form "@[<hv 1>axiom %s :@ %a@]@.@." id 
 	  fprintf_assertion p
@@ -818,19 +851,19 @@ type kind =
   | Unpack
 
 
-let locs_table : 
+let pos_table : 
     (string, (kind option * string option * string option * Loc.position)) 
     Hashtbl.t 
     = Hashtbl.create 97
 let name_counter = ref 0
-let reg_loc prefix ?id ?kind ?name ?formula loc =
+let reg_pos prefix ?id ?kind ?name ?formula pos =
   let id = match id with
     | None ->  
 	incr name_counter;
 	prefix ^ "_" ^ string_of_int !name_counter
     | Some n -> n
   in
-  Hashtbl.add locs_table id (kind,name,formula,loc);
+  Hashtbl.add pos_table id (kind,name,formula,pos);
   id
 
 let print_kind fmt k =
@@ -852,7 +885,7 @@ let abs_fname f =
     Filename.concat (Unix.getcwd ()) f 
   else f
 
-let print_locs fmt =
+let print_pos fmt =
   Hashtbl.iter 
     (fun id (kind,name,formula,(b,e)) ->
        fprintf fmt "[%s]@\n" id;
@@ -869,7 +902,7 @@ let print_locs fmt =
        fprintf fmt "line = %d@\n" l;
        fprintf fmt "begin = %d@\n" fc;
        fprintf fmt "end = %d@\n@\n" lc)
-    locs_table
+    pos_table
 
 (*
   Local Variables: 
