@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: encoding_pred.ml,v 1.11 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: encoding_pred.ml,v 1.15 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 open Cc
 open Logic
@@ -139,29 +137,43 @@ let rec push d =
   | Dpredicate_def (loc, ident, pred_def_sch) ->
       let p = pred_def_sch.Env.scheme_type in
       let rec lifted_t l p =
-	match l with [] -> p
-	| (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) in
-      push (Dlogic (loc, ident,
-		    (Env.generalize_logic_type (Predicate (snd (List.split (fst p)))))));
-      push (Daxiom (loc, def ident,
+	match l with 
+	  | [] -> p
+	  | (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) 
+      in
+      let name = Ident.string ident in
+      push (Dlogic (loc, name,
+		    (Env.generalize_logic_type 
+		       (Predicate (snd (List.split (fst p)))))));
+      push (Daxiom (loc, def name,
 		    (Env.generalize_predicate
 		       (lifted_t (fst p)
-			  (Piff ((Papp (Ident.create ident, List.map (fun (i,_) -> Tvar i) (fst p), [])),
+			  (Piff ((Papp (ident, 
+					List.map (fun (i,_) -> Tvar i) (fst p),
+					[])),
 			         (snd p)))))))
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "encoding rec: inductive def not yet supported"
 (* A function definition can be handled as a function logic definition + an axiom *)
   | Dfunction_def (loc, ident, fun_def_sch) ->
       let f = fun_def_sch.Env.scheme_type in
       let rec lifted_t l p =
-	match l with [] -> p
-	| (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) in
+	match l with 
+	  | [] -> p
+	  | (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) 
+      in
       let (ptl, rt, t) = f in
-      push (Dlogic (loc, ident,
-		    (Env.generalize_logic_type (Function (snd (List.split ptl), rt)))));
-      push (Daxiom (loc, def ident,
+      let name = Ident.string ident in
+      push (Dlogic (loc, name,
+		    (Env.generalize_logic_type 
+		       (Function (snd (List.split ptl), rt)))));
+      push (Daxiom (loc, def name,
 		    (Env.generalize_predicate
 		       (lifted_t ptl
 			  (Papp (Ident.t_eq,
-				 [(Tapp (Ident.create ident, List.map (fun (i,_) -> Tvar i) ptl, []));
+				 [(Tapp (ident, 
+					 List.map (fun (i,_) -> Tvar i) ptl,
+					 []));
 				  t], []))))))
 (* Goals and axioms are just translated straightworfardly *)
   | Daxiom (loc, ident, pred_sch) ->
