@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: encoding_rec.ml,v 1.12 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: encoding_rec.ml,v 1.16 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 open Options
 open Cc
@@ -214,28 +212,42 @@ let rec push d =
       let p = pred_def_sch.Env.scheme_type in
       let rec lifted_t l p =
 	match l with [] -> p
-	| (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) in
-      push (Dlogic (loc, ident, 
-		    (Env.generalize_logic_type (Predicate (snd (List.split (fst p)))))));
-      push (Daxiom (loc, def ident,
-		    (Env.generalize_predicate 
-		       (lifted_t (fst p) 
-			  (Piff ((Papp (Ident.create ident, List.map (fun (i,_) -> Tvar i) (fst p), [])),
-			         (snd p)))))))
+	  | (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) 
+      in
+      let name = Ident.string ident in
+      push (Dlogic (loc, name, 
+		    (Env.generalize_logic_type 
+		       (Predicate (snd (List.split (fst p)))))));
+      push (Daxiom 
+	      (loc, def name,
+	       (Env.generalize_predicate 
+		  (lifted_t (fst p) 
+		     (Piff ((Papp (ident, 
+				   List.map (fun (i,_) -> Tvar i) (fst p),
+				   [])),
+			    (snd p)))))))
+  | Dinductive_def _ ->
+      failwith "encoding rec: inductive def not yet supported"
 (* A function definition can be handled as a function logic definition + an axiom *)
   | Dfunction_def (loc, ident, fun_def_sch) -> 
       let f = fun_def_sch.Env.scheme_type in
       let rec lifted_t l p =
-	match l with [] -> p
-	| (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) in
+	match l with 
+	  | [] -> p
+	  | (a,t)::q -> lifted_t q (Forall(false, a, a, t, [], p)) 
+      in
       let (ptl, rt, t) = f in
-      push (Dlogic (loc, ident, 
+      let name = Ident.string ident in
+      push (Dlogic (loc, name, 
 		    (Env.generalize_logic_type (Function (snd (List.split ptl), rt)))));
-      push (Daxiom (loc, def ident,
-		    (Env.generalize_predicate
-		       (lifted_t ptl
-			  (Papp (Ident.t_eq,
-				 [(Tapp (Ident.create ident, List.map (fun (i,_) -> Tvar i) ptl, []));
+      push (Daxiom 
+	      (loc, def name,
+	       (Env.generalize_predicate
+		  (lifted_t ptl
+		     (Papp (Ident.t_eq,
+			    [(Tapp (ident, 
+				    List.map (fun (i,_) -> Tvar i) ptl, 
+				    []));
 				  t], []))))))
   | Daxiom (loc, ident, pred_sch) -> 
       let cpt = ref 0 in
