@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: encoding_strat.ml,v 1.16 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: encoding_strat.ml,v 1.21 2008-11-06 10:08:13 moy Exp $ i*)
 
 open Cc
 open Logic
@@ -323,17 +321,23 @@ let rec push d =
 (* A predicate definition can be handled as a predicate logic definition + an axiom *)
   | Dpredicate_def (loc, ident, pred_def_sch) ->
       let (argl, pred) = pred_def_sch.Env.scheme_type in
-      let rootexp = (Papp (Ident.create ident, List.map (fun (i,_) -> Tvar i) argl, [])) in
-      push (Dlogic (loc, ident, (Env.generalize_logic_type (Predicate (snd (List.split argl))))));
-      push (Daxiom (loc, def ident, (Env.generalize_predicate 
+      let rootexp = (Papp (ident, List.map (fun (i,_) -> Tvar i) argl, [])) in
+      let name = Ident.string ident in
+      push (Dlogic (loc, name, (Env.generalize_logic_type (Predicate (snd (List.split argl))))));
+      push (Daxiom (loc, def name, (Env.generalize_predicate 
 				       (lifted_t argl (Piff (rootexp, pred)) [[PPat rootexp]]))))
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "encoding strat: inductive def not yet supported"
 (* A function definition can be handled as a function logic definition + an axiom *)
   | Dfunction_def (loc, ident, fun_def_sch) ->
+(* ?????
       let _ = print_endline ident in
+*)
       let (argl, rt, term) = fun_def_sch.Env.scheme_type in
-      let rootexp = (Tapp (Ident.create ident, List.map (fun (i,_) -> Tvar i) argl, [])) in
-      push (Dlogic (loc, ident, (Env.generalize_logic_type (Function (snd (List.split argl), rt)))));
-      push (Daxiom (loc, def ident,
+      let rootexp = (Tapp (ident, List.map (fun (i,_) -> Tvar i) argl, [])) in
+      let name = Ident.string ident in
+      push (Dlogic (loc, name, (Env.generalize_logic_type (Function (snd (List.split argl), rt)))));
+      push (Daxiom (loc, def name,
 		    (Env.generalize_predicate
 		       (lifted_t argl (Papp (Ident.t_eq, [rootexp; term], [])) [[TPat rootexp]]))))
 (* Axiom definitions *)
@@ -347,23 +351,28 @@ let rec push d =
       Queue.add (Daxiom (loc, ident, new_axiom)) queue
 (* A goal is a sequent : a context and a predicate and both have to be translated *)
   | Dgoal (loc, expl, ident, s_sch) ->
-      let cpt = ref 0 in
-      let fv = Env.Vset.fold
+      begin try
+	let cpt = ref 0 in
+	let fv = Env.Vset.fold
 	  (fun tv acc -> cpt := !cpt + 1; (tv.tag, tvar^(string_of_int !cpt))::acc)
 	  (s_sch.Env.scheme_vars) [] in
-      let (context, new_cel) = 
-	List.fold_left 
-	  (fun (acc_c, acc_new_cel) s -> match s with
-	    Spred (id, p) -> (acc_c, 
-			      (Spred (id, translate_eq (translate_pred fv acc_c p)))::acc_new_cel)
-	  | Svar (id, t) -> ((id,t)::acc_c, (Svar (id, ut))::acc_new_cel))
-	  ([], [])
-	  (fst (s_sch.Env.scheme_type)) in
-      let new_sequent =
-	Env.empty_scheme
-	  (lifted_ctxt fv (List.rev new_cel),
-	   translate_eq (translate_pred fv context (snd (s_sch.Env.scheme_type)))) in
-      Queue.add (Dgoal (loc, expl, ident, new_sequent)) queue)
+	let (context, new_cel) = 
+	  List.fold_left 
+	    (fun (acc_c, acc_new_cel) s -> match s with
+		 Spred (id, p) -> (acc_c, 
+				   (Spred (id, translate_eq (translate_pred fv acc_c p)))::acc_new_cel)
+	       | Svar (id, t) -> ((id,t)::acc_c, (Svar (id, ut))::acc_new_cel))
+	    ([], [])
+	    (fst (s_sch.Env.scheme_type)) in
+	let new_sequent =
+	  Env.empty_scheme
+	    (lifted_ctxt fv (List.rev new_cel),
+	     translate_eq (translate_pred fv context (snd (s_sch.Env.scheme_type)))) in
+	Queue.add (Dgoal (loc, expl, ident, new_sequent)) queue
+      with Not_found -> 
+	Format.eprintf "Exception caught in : %a\n" Util.print_decl d;
+	Queue.add (Dgoal (loc, expl, ident, Env.empty_scheme([],Pfalse))) queue
+      end)
   with
     Not_found -> 
       Format.eprintf "Exception caught in : %a\n" Util.print_decl d;
