@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: env.ml,v 1.76 2008/04/01 14:46:05 hubert Exp $ i*)
+(*i $Id: env.ml,v 1.80 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 open Ident
 open Misc
@@ -137,6 +135,13 @@ let generalize_predicate_def (bl,p) =
   let l = find_predicate_vars l p in
   { scheme_vars = l; scheme_type = (bl,p) }
 
+let generalize_inductive_def (bl,l) =
+  let vars = List.fold_left find_pure_type_vars Vset.empty bl in
+  let vars =
+    List.fold_left (fun acc (_,p) -> find_predicate_vars acc p) vars l
+  in
+  { scheme_vars = vars ; scheme_type = (bl,l) }
+
 let generalize_function_def (bl,t,e) = 
   let l = 
     List.fold_left (fun acc (_,pt) -> find_pure_type_vars acc pt) Vset.empty bl 
@@ -238,6 +243,12 @@ let subst_predicate_def s (bl,p) =
 
 let specialize_predicate_def = specialize_scheme subst_predicate_def
 
+let subst_inductive_def s (bl,p) =
+  let bl = List.map (subst_pure_type s) bl in
+  bl, List.map (fun (id,p) -> (id,subst_predicate s p)) p
+
+let specialize_inductive_def = specialize_scheme subst_inductive_def
+
 let subst_function_def s (bl,t,e) =
   let bl = List.map (fun (x,pt) -> (x, subst_pure_type s pt)) bl in
   bl, subst_pure_type s t, e
