@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: env.mli,v 1.52 2008/04/10 14:43:57 filliatr Exp $ i*)
+(*i $Id: env.mli,v 1.56 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Environment for imperative programs.
  
@@ -140,6 +138,7 @@ val generalize_logic_type : logic_type -> logic_type scheme
 val generalize_type_v : type_v -> type_v scheme
 val generalize_predicate : predicate -> predicate scheme
 val generalize_predicate_def : predicate_def -> predicate_def scheme
+val generalize_inductive_def : inductive_def -> inductive_def scheme
 val generalize_function_def : function_def -> function_def scheme
 val generalize_sequent : sequent -> sequent scheme
 
@@ -148,6 +147,8 @@ val specialize_logic_type : logic_type scheme -> var_subst * logic_type
 val specialize_predicate : predicate scheme -> var_subst * predicate
 val specialize_predicate_def : 
   predicate_def scheme -> var_subst * predicate_def
+val specialize_inductive_def : 
+  inductive_def scheme -> var_subst * inductive_def
 val specialize_function_def : 
   function_def scheme -> var_subst * function_def
 val specialize_sequent : sequent scheme -> var_subst * sequent
