@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: harvey.ml,v 1.51 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: harvey.ml,v 1.54 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Harvey's output *)
 
@@ -279,12 +277,14 @@ let output_obligation fmt (loc, expl, o, s) =
 let decl_to_elem = function
   | Dgoal (loc,expl,id,s) -> Queue.add (loc,expl,id,s.Env.scheme_type) oblig
   | Daxiom (_, id, p) -> Queue.add (Axiom (id, p)) theory
-  | Dpredicate_def (_, id, p) -> Queue.add (Predicate (id, p)) theory
-  | Dfunction_def (_, id, p) -> Queue.add (FunctionDef (id, p)) theory
-  | _ -> ()
-
-
-
+  | Dpredicate_def (_, id, p) -> 
+      Queue.add (Predicate (Ident.string id, p)) theory
+  | Dfunction_def (_, id, p) -> 
+      Queue.add (FunctionDef (Ident.string id, p)) theory
+  | Dinductive_def (_, _, _) ->
+      failwith "Harvey output: inductive def not yet supported"
+  | Dlogic (_, _, _) -> ()
+  | Dtype (_, _, _) -> ()
 
 
 
