@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: holl.ml,v 1.45 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: holl.ml,v 1.49 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s HOL Light output *)
 
@@ -54,7 +52,9 @@ let reset () = Queue.clear elem_q
 let push_decl = function
   | Dlogic (_, id, t) -> Queue.add (Logic (id, t)) elem_q
   | Daxiom (_, id, p) -> Queue.add (Axiom (id, p)) elem_q
-  | Dpredicate_def (_, id, p) -> Queue.add (Predicate (id, p)) elem_q
+  | Dpredicate_def (_, id, p) -> Queue.add (Predicate (Ident.string id, p)) elem_q
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "HOL light output: inductive def not yet supported"
   | Dgoal (loc,expl,id,s) -> 
       Queue.add (Obligation (loc,expl,id,s.Env.scheme_type)) elem_q
   | Dfunction_def _ -> () (*TODO*)
