@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ident.ml,v 1.65 2008/04/15 07:50:19 regisgia Exp $ i*)
+(*i $Id: ident.ml,v 1.73 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 type t = { stamp : int; name : string; label : string option }
 
@@ -127,6 +125,11 @@ let fun_id x = create (string x ^ "_fun")
 
 (*s Pre-defined. *)
 
+let t_bool_and = create "bool_and"
+let t_bool_or = create "bool_or"
+let t_bool_xor = create "bool_xor"
+let t_bool_not = create "bool_not"
+
 let anonymous = create "_"
 let implicit = create "?"
 let default_post = create "%default_post"
@@ -141,6 +144,7 @@ let t_add_int = create "add_int"
 let t_sub_int = create "sub_int"
 let t_mul_int = create "mul_int"
 let t_div_int = create "div_int"
+let t_mod_int = create "mod_int"
 let t_neg_int = create "neg_int"
 
 let t_add_real = create "add_real"
@@ -148,12 +152,16 @@ let t_sub_real = create "sub_real"
 let t_mul_real = create "mul_real"
 let t_div_real = create "div_real"
 let t_neg_real = create "neg_real"
+
 let t_abs_real = create "abs_real"
 let t_pow_real = create "pow_real"
-
-let t_mod_int = create "mod_int"
 let t_sqrt_real = create "sqrt_real"
 
+let t_int_max = create "int_max"
+let t_int_min = create "int_min"
+let t_real_max = create "real_max"
+let t_real_min = create "real_min"
+
 let t_real_of_int = create "real_of_int"
 let t_int_of_real = create "int_of_real"
 
@@ -206,12 +214,27 @@ let t_le_int_ = create "le_int_"
 let t_gt_int_ = create "gt_int_"
 let t_ge_int_ = create "ge_int_"
 
+let t_lt_int_bool = create "lt_int_bool"
+let t_le_int_bool = create "le_int_bool"
+let t_gt_int_bool = create "gt_int_bool"
+let t_ge_int_bool = create "ge_int_bool"
+
 let t_lt_real_ = create "lt_real_"
 let t_le_real_ = create "le_real_"
 let t_gt_real_ = create "gt_real_"
 let t_ge_real_ = create "ge_real_"
 
-let t_zwf_zero = create "%zwf_zero"
+let t_lt_real_bool = create "lt_real_bool"
+let t_le_real_bool = create "le_real_bool"
+let t_gt_real_bool = create "gt_real_bool"
+let t_ge_real_bool = create "ge_real_bool"
+
+let t_eq_int_bool = create "t_eq_int_bool"
+let t_eq_real_bool = create "t_eq_real_bool"
+let t_neq_int_bool = create "t_neq_int_bool"
+let t_neq_real_bool = create "t_neq_real_bool"
+
+let t_zwf_zero = create "zwf_zero"
 let result = create "result"
 let default = create "H"
 let farray = create "farray"
@@ -321,3 +344,8 @@ let is_arith id =
 
 let is_simplify_arith id =
   is_int_arith id || id == t_lt_int || id == t_le_int || id == t_gt_int || id == t_ge_int
+
+let strict_bool_and_ = create "strict_bool_and_"
+let strict_bool_or_ = create "strict_bool_or_"
+let bool_not_ = create "bool_not_"
+
