@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ident.mli,v 1.57 2008/04/15 07:50:19 regisgia Exp $ i*)
+(*i $Id: ident.mli,v 1.64 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Identifiers. *)
 
@@ -96,6 +94,11 @@ val fun_id : t -> t
 
 (*s Some pre-defined identifiers. *)
 
+val t_bool_and : t 
+val t_bool_or : t
+val t_bool_xor : t
+val t_bool_not : t
+
 val anonymous : t
 val implicit : t
 val default_post : t
@@ -122,6 +125,11 @@ val t_sqrt_real : t
 val t_abs_real : t
 val t_pow_real : t
 
+val t_int_max : t
+val t_int_min : t
+val t_real_max : t
+val t_real_min : t
+
 val t_div_real_ : t
 val t_div_int_ : t
 val t_mod_int_ : t
@@ -157,6 +165,21 @@ val t_le_real : t
 val t_gt_real : t
 val t_ge_real : t
 
+val t_lt_int_bool : t
+val t_le_int_bool : t
+val t_gt_int_bool : t
+val t_ge_int_bool : t
+
+val t_lt_real_bool : t
+val t_le_real_bool : t
+val t_gt_real_bool : t
+val t_ge_real_bool : t
+
+val t_eq_int_bool : t
+val t_eq_real_bool : t
+val t_neq_int_bool : t
+val t_neq_real_bool : t
+
 val t_eq_int_ : t
 val t_eq_bool_ : t
 val t_eq_real_ : t
@@ -232,3 +255,7 @@ val is_arith_binop : t -> bool
 val is_arith : t -> bool
 
 val is_simplify_arith : t -> bool
+
+val strict_bool_and_ : t
+val strict_bool_or_ : t
+val bool_not_ : t
