@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: lib.ml,v 1.10 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: lib.ml,v 1.13 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 module Sset = Set.Make(String)
 
@@ -44,6 +42,12 @@ let file ~dir ~file =
   mkdir_p dir;
   Filename.concat dir (Filename.basename file)
 
+let file_subdir ~dir ~file = 
+  let d = Filename.dirname file in
+  let d = Filename.concat d dir in
+  mkdir_p d;
+  Filename.concat d (Filename.basename file)
+
 let file_copy src dest =
   let cin = open_in src
   and cout = open_out dest
