@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: logic.mli,v 1.45 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: logic.mli,v 1.48 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Logic. *)
 
@@ -98,6 +96,8 @@ type logic_type =
 
 type predicate_def = (Ident.t * pure_type) list * predicate
 
+type inductive_def = pure_type list * (Ident.t * predicate) list
+
 type function_def = (Ident.t * pure_type) list * pure_type * term
 
 
