@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: logic_decl.mli,v 1.15 2008/02/20 14:34:26 marche Exp $ i*)
+(*i $Id: logic_decl.mli,v 1.20 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Logical declarations. 
     This is what is sent to the various provers (see main.ml and the provers
@@ -63,8 +61,9 @@ type obligation = Loc.floc * vc_expl * string * sequent
 type t =
   | Dtype          of loc * string list * string
   | Dlogic         of loc * string * logic_type scheme
-  | Dpredicate_def of loc * string * predicate_def scheme
-  | Dfunction_def  of loc * string * function_def scheme
+  | Dpredicate_def of loc * Ident.t * predicate_def scheme
+  | Dinductive_def of loc * Ident.t * inductive_def scheme
+  | Dfunction_def  of loc * Ident.t * function_def scheme
   | Daxiom         of loc * string * predicate scheme
   | Dgoal          of loc * vc_expl * string * sequent scheme
 
