@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ltyping.ml,v 1.70 2008/04/15 08:12:50 regisgia Exp $ i*)
+(*i $Id: ltyping.ml,v 1.73 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Typing on the logical side *)
 
@@ -326,14 +324,16 @@ and type_if lab env a b c =
     | _ -> raise_located a.pp_loc ShouldBeBoolean
 
 and type_tvar loc lab env x = 
-  let xu = 
+  let x,xu = 
     if is_at x then begin
       let xu,l = un_at x in
-      if not (is_reference env xu) then raise_located loc (NotAReference xu);
       if not (Label.mem l lab) then raise_located loc (UnboundLabel l);
-      xu
+      if not (is_reference env xu) then 
+	xu,xu
+      else 
+	x,xu
     end else 
-      x
+      x,x
   in
   try 
     let t = find_logic xu env in Tvar x, t
