@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: main.ml,v 1.152 2008/05/28 13:51:26 marche Exp $ i*)
+(*i $Id: main.ml,v 1.166 2009-01-21 08:34:15 marche Exp $ i*)
 
 open Options
 open Ptree
@@ -62,14 +60,15 @@ let reset () =
     | Hol4 -> Hol4.reset ()
     | SmtLib ->  ()
     | Harvey | Simplify | Zenon | CVCLite  | Gappa 
-    | Why | MultiWhy | Dispatcher | WhyProject -> ()
+    | Ergo | Why | MultiWhy | Dispatcher | WhyProject -> ()
 
 let add_loc = function
   | Dtype (loc, _, s)
   | Dlogic (loc, s, _)
+  | Daxiom (loc, s, _) (* useful? *) -> Loc.add_ident s loc
   | Dpredicate_def (loc, s, _)
   | Dfunction_def (loc, s, _) 
-  | Daxiom (loc, s, _) (* useful? *) -> Loc.add_ident s loc
+  | Dinductive_def (loc, s , _) -> () (* TODO ? *)
   | Dgoal _ -> ()
 
 let store_decl_into_a_queue d  = 
@@ -79,7 +78,7 @@ let store_decl_into_a_queue d  =
 (** push the declarations  in the corresponding 
     prover and stores them (or their expansed version) into 
     declarationQueue **)
-let push_decl vloc d = 
+let push_decl _vloc d = 
   add_loc d;
   if (not pruning) && (Options.pruning_hyp_v = -1) then
     begin 
@@ -92,8 +91,21 @@ let push_decl vloc d =
 	  | Isabelle -> Isabelle.push_decl
 	  | Hol4 -> Hol4.push_decl
 	  | Gappa -> Gappa.push_decl 
-	  | Why | MultiWhy | WhyProject -> Pretty.push_decl 
-	  | Dispatcher ->Dispatcher.push_decl
+	  | MultiWhy (* // Moy -> Marche NO destroy eclipse plugin
+			-> Pretty.push_or_output_decl *)
+	  | Why | WhyProject -> Pretty.push_decl ~ergo:false
+	  | Ergo -> Pretty.push_decl ~ergo:true
+	  | Dispatcher -> 
+(*
+	      (fun d ->
+		 (* push for Dispatcher, used by Gwhy *)
+*)
+	      Dispatcher.push_decl
+(*
+		 if prover ~ignore_gui:true () = WhyProject then
+		   (* also push for project, used by GWhy/Coq column *)
+		   Pretty.push_decl ~ergo:false d)
+*)
 	  | Harvey -> Harvey.push_decl
 	  | Simplify -> Simplify.push_decl
 	  | Zenon -> Zenon.push_decl
@@ -101,9 +113,9 @@ let push_decl vloc d =
 	  | SmtLib -> Smtlib.push_decl 
       in
       if defExpanding != NoExpanding  then 
-	let decl = (PredDefExpansor.push d) in 
-	List.iter pushing decl ;
-	List.iter store_decl_into_a_queue decl ; 
+	let decl = PredDefExpansor.push d in 
+	(* List.iter *) pushing decl ;
+	(* List.iter *) store_decl_into_a_queue decl ; 
       else
 	begin 
 	  store_decl_into_a_queue d ;
@@ -112,7 +124,7 @@ let push_decl vloc d =
     end
   else
     if defExpanding != NoExpanding then
-      List.iter store_decl_into_a_queue (PredDefExpansor.push d)
+      (* List.iter *) store_decl_into_a_queue (PredDefExpansor.push d)
     else
       store_decl_into_a_queue d 
 	
@@ -139,12 +151,12 @@ let push_validation id tt v =
 
 let is_pure_type_scheme s = is_pure_type_v s.scheme_type
 
-let push_parameter id v tv = match prover () with
+let push_parameter id _v tv = match prover () with
   | Coq _ -> 
       if valid then Coq.push_parameter id tv
   | Pvs | HolLight | Isabelle | Hol4 | Mizar
   | Harvey | Simplify | Zenon | SmtLib | Gappa 
-  | CVCLite | Why | MultiWhy | Dispatcher | WhyProject -> 
+  | CVCLite | Ergo | Why | MultiWhy | Dispatcher | WhyProject -> 
       ()
 
 let output fwe = 
@@ -168,9 +180,14 @@ let output fwe =
     | Hol4 -> Hol4.output_file fwe
     | Gappa -> Gappa.output_file fwe
     | Dispatcher -> ()
-    | Why -> Pretty.output_file fwe
+(*
+	if prover (* ~ignore_gui:true*)  () = WhyProject then
+	  (* output project, used by GWhy/Coq column *)
+	  Options.gui_project := Some(Pretty.output_project fwe)
+*)
+    | Ergo | Why -> Pretty.output_file fwe
     | MultiWhy -> Pretty.output_files fwe
-    | WhyProject -> Pretty.output_project fwe
+    | WhyProject -> ignore(Pretty.output_project fwe)
   end
 
 
@@ -189,6 +206,7 @@ let encode q =
     | Hol4 -> Hol4.push_decl d
     | Gappa -> Gappa.push_decl d  
     | Why | MultiWhy | WhyProject -> Pretty.push_decl d
+    | Ergo -> Pretty.push_decl ~ergo:true d
     | Dispatcher -> Dispatcher.push_decl d      
     | Harvey -> Harvey.push_decl d
     | Simplify -> Simplify.push_decl d
@@ -287,9 +305,9 @@ let interp_program loc id p =
     let ren = initial_renaming env in
     let tt = Monad.trad_type_c ren env c in
     let ren = initial_renaming env in
-    let cc = Red.red (Mlize.trad p ren) in
-(*    Coq.push_parameter (ids ^ "_valid") tt; *)
-      Coq.push_program (ids ^ "_functional") tt cc;
+    let _cc = Red.red (Mlize.trad p ren) in
+      Coq.push_parameter (ids ^ "_valid") tt;
+(*      Coq.push_program (ids ^ "_functional") tt cc; *)
     (*** TODO
 	 push_validation ids tt v;
 	 if_verbose_2 eprintf "%d proof obligation(s)@\n@." (List.length ol);
@@ -335,7 +353,81 @@ let check_duplicate_params l =
   in
   loop l Ident.Idset.empty
 
-let interp_decl ?(prelude=false) d = 
+let check_duplicate_binders loc ty =
+  let rec check acc = function
+    | PVpure _ | PVref _ -> ()
+    | PVarrow (l, ty) ->
+	let rec loop l acc =
+	  match l with
+	    | [] -> acc
+	    | (x,_)::rem when x = Ident.anonymous ->
+		loop rem acc
+	    | (x,_)::rem ->
+		if Ident.Idset.mem x acc then
+		  raise_located loc (ClashParam x)
+		else loop rem (Ident.Idset.add x acc)
+	in
+	check (loop l acc) ty.pc_result_type
+  in
+  check Ident.Idset.empty ty
+
+
+(** [check_clausal_form loc pi a] checks whether the assertion [a] 
+    has a valid clausal form 
+      \forall x_1,.., x_k. P1 -> ... -> P_n -> P
+    where P is headed by pi and pi has only positive occurrences in P1 .. Pn
+*)
+
+let rec occurrences pi a =
+  match a with
+  | Ptrue | Pfalse -> (0,0)
+  | Papp (id, _, _) -> ((if id == pi then 1 else 0),0)
+  | Pimplies (_, p1, p2) -> 
+      let (pos1,neg1) = occurrences pi p1 in
+      let (pos2,neg2) = occurrences pi p2 in
+      (neg1+pos2,pos1+neg2)
+  | Pand (_, _, p1, p2) -> 
+      let (pos1,neg1) = occurrences pi p1 in
+      let (pos2,neg2) = occurrences pi p2 in
+      (pos1+pos2,neg1+neg2)
+  | Pnot p1 -> 
+      let (pos1,neg1) = occurrences pi p1 in (neg1,pos1)
+  | Forall (is_wp, id1, id2, typ, triggers, p) -> 
+      assert false (* TODO *)
+      (* occurrences pi p *)
+  | Pnamed (_, _) -> assert false (* TODO *)
+  | Pfpi (_, _, _) -> assert false (* TODO *)
+  | Exists (_, _, _, _)  -> assert false (* TODO *)
+  | Forallb (_, _, _)  -> assert false (* TODO *)
+  | Piff (_, _) -> assert false (* TODO *)
+  | Por (_, _) -> assert false (* TODO *)
+  | Pif (_, _, _) -> assert false (* TODO *)
+  | Pvar _ -> assert false (* TODO *)
+
+let rec check_unquantified_clausal_form loc id a =
+  match a with
+    | Pimplies (_, p1, p2) -> 
+	check_unquantified_clausal_form loc id p2;
+	let (pos1,neg1) = occurrences id p1 in
+	if neg1 > 0 then 
+	  raise_located loc 
+	    (AnyMessage "inductive predicate has a negative occurrence in this case")
+    | Papp(pi,_,_) -> 
+	if pi != id then
+	  raise_located loc 
+	    (AnyMessage "head of clause does not contain the inductive predicate")
+    | _ -> 
+	  raise_located loc 
+	    (AnyMessage "this case is not in clausal form")
+	
+
+let rec check_clausal_form loc id a =
+  match a with
+    | Forall (is_wp, id1, id2, typ, triggers, p) -> 
+	check_clausal_form loc id p
+    | _ -> check_unquantified_clausal_form loc id a
+
+let interp_decl ?(_prelude=false) d = 
   let lab = Label.empty in
   match d with 
     | Program (loc,id, p) ->
@@ -343,6 +435,7 @@ let interp_decl ?(prelude=false) d =
 	(try interp_program loc id p with Exit -> ())
     | Parameter (loc, ext, ids, v) ->
 	let env = Env.empty_progs () in
+	check_duplicate_binders loc v;
 	let v = Ltyping.type_v loc lab env v in
 	if ext && is_mutable v then raise_located loc MutableExternal;
 	let gv = Env.generalize_type_v v in
@@ -394,7 +487,28 @@ let interp_decl ?(prelude=false) d =
 	let p = Ltyping.predicate lab env' p in
 	add_global_logic id (generalize_logic_type t);
 	let p = generalize_predicate_def (pl,p) in
-	push_decl ("","",Loc.dummy_floc) (Dpredicate_def (Loc.extract loc, Ident.string id, p))
+	push_decl ("","",Loc.dummy_floc) (
+	  Dpredicate_def (Loc.extract loc, id, p))
+    | Inductive_def(loc,id,t,indcases) ->
+	let env = Env.empty_logic () in
+	if is_global_logic id then raise_located loc (Clash id);
+	let pl = 
+	  match Ltyping.logic_type t with
+	    | Function _ -> raise_located loc (AnyMessage "only predicates can be inductively defined")
+	    | Predicate l -> l
+	in
+	(* add id first because indcases contain id, without generalization *)
+	add_global_logic id (empty_scheme (Predicate pl));
+	let l =
+	  List.map (fun (loc,i,p) ->
+		      let p = Ltyping.predicate lab env p in
+		      check_clausal_form loc id p;
+		      (i,p))
+	    indcases
+	in
+	let d = generalize_inductive_def (pl,l) in
+	push_decl ("","",Loc.dummy_floc) 
+	  (Dinductive_def (Loc.extract loc, id, d))
     | Function_def (loc, id, pl, ty, e) ->
 	let env = Env.empty_logic () in
 	if is_global_logic id then raise_located loc (Clash id);
@@ -408,7 +522,8 @@ let interp_decl ?(prelude=false) d =
 	  Ltyping.expected_type loc (PureType ty);
 	add_global_logic id (generalize_logic_type t);
 	let f = generalize_function_def (pl,ty,e) in
-	push_decl ("","",Loc.dummy_floc) (Dfunction_def (Loc.extract loc, Ident.string id, f))
+	push_decl ("","",Loc.dummy_floc) 
+	  (Dfunction_def (Loc.extract loc, id, f))
     | Axiom (loc, id, p) ->
 	let env = Env.empty_logic () in
 	let p = Ltyping.predicate lab env p in
@@ -419,7 +534,11 @@ let interp_decl ?(prelude=false) d =
 	let env = Env.empty_logic () in
 	let p = Ltyping.predicate lab env p in
 	let s = ([], p) in
-	let l = Loc.extract loc in
+	let l = 
+	  try
+	    Util.loc_of_label (Ident.string id)
+	  with Not_found -> Loc.extract loc
+	in
 	let xpl =
 	  { lemma_or_fun_name = Ident.string id;
 	    behavior = "";
@@ -462,15 +581,15 @@ let interp_decl ?(prelude=false) d =
 	    
 (*s Prelude *)
 
-let load_file ?(prelude=false) f =
+let load_file ?(_prelude=false) f =
   let c = open_in f in
   let p = Lexer.parse_file (Lexing.from_channel c) in
-  List.iter (interp_decl ~prelude) p;
+  List.iter (interp_decl ~_prelude) p;
   close_in c
 
 let load_prelude () =
   try
-    List.iter (load_file ~prelude:true) lib_files_to_load;
+    List.iter (load_file ~_prelude:true) lib_files_to_load;
     begin match prover () with
       | Simplify when no_simplify_prelude -> Simplify.reset ()
       | _ -> ()
@@ -492,7 +611,7 @@ let deal_channel parsef cin =
 
 let single_file () = match prover () with
   | Simplify | Harvey | Zenon | CVCLite | Gappa | Dispatcher 
-  | SmtLib | Why | MultiWhy | WhyProject -> true
+  | SmtLib | Ergo | Why | MultiWhy | WhyProject -> true
   | Coq _ | Pvs | Mizar | Hol4 | HolLight | Isabelle -> false
 
 let deal_file f =
@@ -506,23 +625,24 @@ let deal_file f =
 let main () =
   let t0 = Unix.times () in
   load_prelude ();
-  if files = [] then begin
-    deal_channel (why_parser "standard input") stdin;
-    output (Options.out_file "out")
-  end else begin
-    List.iter deal_file files;
-    if (pruning) or (Options.pruning_hyp_v != -1) then
-      begin
-	let q =  declarationQueue in 
-	encode q ;
-	if single_file () then 
-	  let lf = Filename.chop_extension (last files) in
-	  output (Options.out_file lf)  
-      end
-    else if single_file () then 
-      let lf = Filename.chop_extension (last files) in
-      output (Options.out_file lf)
-  end;
+  if files = [] then 
+    begin
+      deal_channel (why_parser "standard input") stdin;
+      output (Options.out_file "out")
+    end 
+  else 
+    begin
+      List.iter deal_file files;
+      if (pruning) or (Options.pruning_hyp_v != -1) then
+	begin
+	  let q =  declarationQueue in 
+	  encode q 
+	end;
+      if single_file () then 
+	let lf = Filename.chop_extension (last files) in
+	let outf = Options.out_file lf in      
+	output outf
+    end;
   if show_time then
     let t1 = Unix.times () in
     printf "Why execution time : %3.2f@." 
