@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: mizar.ml,v 1.45 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: mizar.ml,v 1.50 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (*s Mizar output *)
 
@@ -57,7 +55,10 @@ let push_decl = function
       Queue.add (Obligation (loc, expl, id, s.Env.scheme_type)) elem_q
   | Dlogic (_, id, t) -> Queue.add (Logic (id, t)) elem_q
   | Daxiom (_, id, p) -> Queue.add (Axiom (id, p)) elem_q
-  | Dpredicate_def (_, id, p) -> Queue.add (Predicate (id, p)) elem_q
+  | Dpredicate_def (_, id, p) -> 
+      Queue.add (Predicate (Ident.string id, p)) elem_q
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "Mizar output: inductive def not yet supported"
   | Dfunction_def _ -> () (*TODO*)
   | Dtype _ -> () (*TODO*)
 
@@ -325,6 +326,7 @@ struct
       | Logic (id, t) -> print_logic fmt id t
       | Axiom (id, p) -> print_axiom fmt id p
       | Predicate _ -> assert false (*TODO*)
+      | Inductive _ -> assert false (*TODO*)
       | Function _ -> assert false (*TODO*)
       | AbstractType _ -> assert false (*TODO*)
     end;
@@ -337,6 +339,7 @@ struct
     | Logic (id, t) -> reprint_logic fmt id t
     | Axiom (id, p) -> reprint_axiom fmt id p
     | Predicate _ -> assert false (*TODO*)
+    | Inductive _ -> assert false (*TODO*)
     | Function _ -> assert false (*TODO*)
     | AbstractType _ -> assert false (*TODO*)
 
