@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: monomorph.ml,v 1.33 2008/02/05 12:10:49 marche Exp $ i*)
+(*i $Id: monomorph.ml,v 1.37 2008-11-05 14:03:17 filliatr Exp $ i*)
 
 (* monomorphic output *)
 
@@ -422,27 +420,27 @@ let rec declare_logic loc id i =
 
 and push_predicate_def_instance loc id i ((bl,p) as d) =
   IterIT.predicate_def (declare_logic loc) (declare_type loc) d;
-  push (Dpredicate_def (loc, name id i, empty_scheme d))
+  push (Dpredicate_def (loc,Ident.create (name id i), empty_scheme d))
 
 and push_function_def_instance loc id i ((bl,t,e) as d) =
   IterIT.function_def (declare_logic loc) (declare_type loc) d;
-  push (Dfunction_def (loc, name id i, empty_scheme d))
+  push (Dfunction_def (loc, Ident.create (name id i), empty_scheme d))
       
 let push_predicate_def loc id p0 =
   let (bl,_) = p0.scheme_type in
   assert (bl <> []);
   if Vset.is_empty p0.scheme_vars then
-    push_predicate_def_instance loc id [] p0.scheme_type
+    push_predicate_def_instance loc (Ident.string id) [] p0.scheme_type
   else 
-    Hashtbl.add logic_symbols (Ident.create id) (PredicateDef p0)
+    Hashtbl.add logic_symbols id (PredicateDef p0)
 
 let push_function_def loc id p0 =
   let (bl,_,_) = p0.scheme_type in
   assert (bl <> []);
   if Vset.is_empty p0.scheme_vars then
-    push_function_def_instance loc id [] p0.scheme_type
+    push_function_def_instance loc (Ident.string id) [] p0.scheme_type
   else 
-    Hashtbl.add logic_symbols (Ident.create id) (FunctionDef p0)
+    Hashtbl.add logic_symbols id (FunctionDef p0)
 
   (* axioms *)
 
@@ -551,6 +549,8 @@ let push_decl = function
   | Dtype _ -> ()
   | Dlogic (loc, x, t) -> push_logic loc x t
   | Dpredicate_def (loc, x, d) -> push_predicate_def loc x d
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "monomorph: inductive def not supported"
   | Dfunction_def (loc, x, d) -> push_function_def loc x d
   | Daxiom (loc, x, p) -> push_axiom loc x p
   | Dgoal (loc, expl, x, s) -> push_obligation loc expl x s
