@@ -15,19 +15,18 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 val map : ('a -> 'b) -> 'a option -> 'b option
+val map_default : ('a -> 'b) -> 'b -> 'a option -> 'b
 val iter : ('a -> unit) -> 'a option -> unit
 val fold : ('a -> 'b -> 'b) -> 'a option -> 'b -> 'b
 val fold_left : ('b -> 'a -> 'b) -> 'b -> 'a option -> 'b 
