@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: options.ml,v 1.116 2008-05-25 12:11:28 stoulsn Exp $ i*)
+(*i $Id: options.ml,v 1.124 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 open Format
 
@@ -56,6 +54,7 @@ let white_ = ref false
 let black_ = ref true
 let wbb_ = ref false
 let split_user_conj_ = ref false
+let split_bool_op_ = ref false
 let lvlmax_ = ref max_int
 let all_vc_ = ref false
 let prelude_ = ref true
@@ -77,8 +76,12 @@ let pruning_hyp_arithmetic_tactic_ = ref false
 let pruning_hyp_var_tactic_ = ref 0
 let pruning_hyp_polarized_preds_ = ref false
 let prune_context_ = ref false
+let prune_coarse_pred_comp_ = ref false
 (* FIN de Heuristiques en test *)
 let modulo_ = ref false 
+
+let phantom_types = Hashtbl.create 17
+
 let gappa_rnd_ = ref "float < ieee_64, ne >"
 let lib_files_to_load_ = ref []
 let files_to_load_ = ref []
@@ -115,8 +118,8 @@ let coq_version = match Version.coqversion with "v8" -> V8 | _ -> V7
 
 type prover = 
   | Coq of coq_version | Pvs | HolLight | Mizar | Harvey | Simplify | CVCLite
-  | SmtLib | Isabelle | Hol4 | Gappa | Zenon | Why | MultiWhy | Dispatcher
-  | WhyProject
+  | SmtLib | Isabelle | Hol4 | Gappa | Zenon 
+  | Ergo | Why | MultiWhy | Dispatcher | WhyProject
 
 let prover_ = ref (Coq coq_version)
 
@@ -146,6 +149,15 @@ let mizar_environ_from f =
     close_in c;
     Buffer.contents buf
 
+(*s GUI settings *)
+
+(* Are we the GUI? (set in intf/stat.ml) *)
+let gui = ref false
+
+(* GUI project (set in src/main.ml) *)
+let gui_project = ref None
+
+
 (*s Parsing the command-line *)
 
 let copying () =
@@ -198,11 +210,13 @@ Typing/Annotations/VCG options:
   --black            black boxes: WP calculus does not enter pure expressions
   --wbb              while loops as black boxes (careful: incomplete WP)
   --split-user-conj  splits also user conjunctions in goals 
+  --split-bool-op    splits VCs for boolean operations &&, || and not
   --split n          splits conditions into several pieces up to n levels
   --partial          partial correctness
   --total            total correctness
   --explain          outputs explanations for VCs in file.xpl
   --locs f           reads source locations from file f
+  --phantom <name>   declare <name> as a phantom type
 
 VC transformation options:
   --all-vc           outputs all verification conditions (no auto discharge)
@@ -221,9 +235,10 @@ Heuristics UNDER TEST of pruning (needs --prun-hyp to be used) :
   --prune-suffixed-comp       suffixes comparison predicates
   --prune-link-eqs            link each suffixed equalitie to the unsuffixed
   --prune-arith-tactic        statically link arithmetic operators (=, < & <=)
-  --prune-vars-filter T       T in {All, One-var, One-branch, Split-hyps, CNF}        
+  --prune-vars-filter T       T in {All, One-var, One-branch, Split-hyps, CNF}   
   --prune-polarized-preds     Consider polarity on predicates 
   --prune-context             Filter axioms with Pred depth
+  --prune-coarse-pred-comp    abstract weights in predicate predecessors computation  
 
 Prelude files:
   --no-prelude   do not read the prelude files (prelude.why and arrays.why)
@@ -240,19 +255,20 @@ Encoding for types in untyped logic:
   --encoding sstrat  encodes types using some types  + strat
 
 Prover selection:
+  --alt-ergo  selects Alt-Ergo prover
   --coq       selects COQ prover (default)
-  --pvs       selects PVS prover
-  --hol-light selects HOL Light prover
-  --mizar     selects Mizar prover
-  --harvey    selects haRVey prover
-  --isabelle  selects Isabelle prover
-  --hol4      selects HOL4 prover
-  --simplify  selects Simplify prover
   --cvcl      selects CVC Lite prover
+  --gappa     selects the Gappa prover
+  --harvey    selects haRVey prover
+  --hol-light selects HOL Light prover
+  --hol4      selects HOL4 prover
+  --isabelle  selects Isabelle prover
+  --mizar     selects Mizar prover
+  --pvs       selects PVS prover
+  --simplify  selects Simplify prover
   --smtlib    selects the SMT-LIB format
   --zenon     selects the Zenon prover
-  --gappa     selects the Gappa prover
-
+ or generic Why outputs formats:
   --why       selects the Why pretty-printer
   --multi-why selects the Why pretty-printer, with one file per goal
   --project   selects the Why project format, with one file per goal
@@ -315,6 +331,7 @@ let files =
   let rec parse = function
     | [] -> List.rev !filesq
     | ("-h" | "-help" | "--help") :: _ -> usage (); exit 0
+    | ("-alt-ergo" | "--alt-ergo") :: args -> prover_ := Ergo; parse args
     | ("-pvs" | "--pvs") :: args -> prover_ := Pvs; parse args
     | ("-coq-v7" | "--coq-v7") :: args -> prover_ := Coq V7; parse args
     | ("-coq-v8" | "--coq-v8") :: args -> prover_ := Coq V8; parse args
@@ -420,6 +437,8 @@ let files =
         wbb_ := true; parse args
     | ("-split-user-conj" | "--split-user-conj") :: args ->
 	split_user_conj_ := true; parse args
+    | ("-split-bool-op" | "--split-bool-op") :: args ->
+	split_bool_op_ := true; parse args
     | ("-split" | "--split") :: n :: args ->
 	begin try lvlmax_ := int_of_string n with _ -> usage (); exit 1 end; 
 	parse args
@@ -489,6 +508,10 @@ let files =
 
     | ("--prune-context" | "-prune-context") :: args ->
 	prune_context_ := true ; parse args
+    | ("--prune-coarse-pred-comp" | "-prune-coarse-pred-comp") :: args ->
+	prune_coarse_pred_comp_ := true ; parse args
+	    
+
 
 (* FIN de Heuristiques en test *)
 
@@ -515,15 +538,14 @@ let files =
     | ("-locs" | "--locs") :: s :: args ->
 	locs_files := s :: !locs_files;
 	parse args
+    | ("-phantom" | "--phantom") :: s :: args ->
+	Hashtbl.add phantom_types s ();
+	parse args
     | f :: args -> 
 	filesq := f :: !filesq; parse args
   in
   parse (List.tl (Array.to_list Sys.argv))
 
-(* Are we the GUI? (set in intf/stat.ml) *)
-
-let gui = ref false
-
 (*s Finally, we dereference all the refs *)
 
 let verbose = !verbose_
@@ -531,7 +553,8 @@ let debug = !debug_
 let parse_only = !parse_only_
 let type_only = !type_only_
 let wp_only = !wp_only_
-let prover () = if !gui then Dispatcher else !prover_
+let prover (* ?(ignore_gui=false) *) () = 
+  if (* not ignore_gui &&*) !gui then Dispatcher else !prover_
 let valid = !valid_
 let coq_tactic = !coq_tactic_
 let coq_preamble = match !coq_preamble_ with
@@ -557,6 +580,7 @@ let white = !white_
 let black = !black_
 let wbb = !wbb_
 let split_user_conj = !split_user_conj_
+let split_bool_op = !split_bool_op_
 let lvlmax = !lvlmax_
 let all_vc = !all_vc_
 let termination = !termination_
@@ -577,6 +601,7 @@ let pruning_hyp_arithmetic_tactic = !pruning_hyp_arithmetic_tactic_
 let pruning_hyp_var_tactic = !pruning_hyp_var_tactic_
 let pruning_hyp_polarized_preds = !pruning_hyp_polarized_preds_
 let prune_context = !prune_context_
+let prune_coarse_pred_comp = !prune_coarse_pred_comp_
 (* FIN de Heuristiques en test *)
 let modulo = !modulo_
 let defExpanding = !defExpanding_
@@ -649,11 +674,11 @@ end
 
 let prelude = !prelude_
 
-let lib_file f =
-  try
-    Filename.concat (Sys.getenv "WHYLIB") f
-  with Not_found ->
-    Filename.concat Version.libdir f
+let lib_dir =
+  try Sys.getenv "WHYLIB"
+  with Not_found -> Version.libdir
+
+let lib_file f = Filename.concat lib_dir (Filename.concat "why" f)
 
 let prelude_file = lib_file "prelude.why"
 let arrays_file = lib_file "arrays.why"
