@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: options.mli,v 1.88 2008-05-25 12:11:28 stoulsn Exp $ i*)
+(*i $Id: options.mli,v 1.96 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (*s General options *)
 
@@ -67,6 +65,7 @@ val black : bool
 val white : bool
 val wbb : bool
 val split_user_conj : bool
+val split_bool_op : bool
 val lvlmax : int
 val all_vc : bool
 val eval_goals : bool
@@ -74,6 +73,7 @@ val pruning : bool
 val pruning_hyp_v : int
 val pruning_hyp_p : int
 (* Heuristiques en test *)
+val prune_coarse_pred_comp : bool
 val pruning_hyp_CompInGraph : bool
 val pruning_hyp_CompInFiltering : bool
 val pruning_hyp_LinkVarsQuantif : bool
@@ -88,6 +88,8 @@ val prune_context : bool
 (* FIN de Heuristiques en test *)
 val modulo : bool
 
+val phantom_types : (string,unit) Hashtbl.t
+
 type expanding = All | Goal | NoExpanding
 val defExpanding : expanding
 val get_type_expanding : unit -> expanding 
@@ -107,10 +109,10 @@ type coq_version = V7 | V8
 
 type prover = 
   | Coq of coq_version | Pvs | HolLight | Mizar | Harvey | Simplify | CVCLite
-  | SmtLib | Isabelle | Hol4 | Gappa | Zenon | Why | MultiWhy | Dispatcher
-  | WhyProject
+  | SmtLib | Isabelle | Hol4 | Gappa | Zenon 
+  | Ergo | Why | MultiWhy | Dispatcher | WhyProject
 
-val prover : unit -> prover
+val prover : (* ?ignore_gui:bool  -> *) unit -> prover
 
 val valid : bool
 val coq_tactic : string option
@@ -152,7 +154,7 @@ val files : string list
 (*s GUI? *)
 
 val gui : bool ref
-
+val gui_project : Project.t option ref
 val lib_files_to_load : string list
 
 (*
