@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: pp.mli,v 1.15 2008/02/26 10:16:36 moy Exp $ i*)
+(*i $Id: pp.mli,v 1.19 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 open Format
 
@@ -52,6 +50,7 @@ val space : formatter -> unit -> unit
 val alt : formatter -> unit -> unit
 val newline : formatter -> unit -> unit
 val comma : formatter -> unit -> unit
+val simple_comma : formatter -> unit -> unit
 val semi : formatter -> unit -> unit
 val underscore : formatter -> unit -> unit
 val arrow : formatter -> unit -> unit
@@ -59,6 +58,7 @@ val lbrace : formatter -> unit -> unit
 val rbrace : formatter -> unit -> unit
 val nothing : formatter -> unit -> unit
 val string : formatter -> string -> unit
+val constant_string : string -> formatter -> unit -> unit
 val hov : int -> formatter -> ('a -> unit) -> 'a -> unit
 
 val open_formatter : ?margin:int -> out_channel -> formatter
