@@ -15,17 +15,27 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-val  push : Logic_decl.t -> Logic_decl.t list
+
+val inductive_inverse_body : 
+    Ident.t ->  (* predicate name *)
+    'a list ->  (* types of parameters *)
+    (Ident.t * Logic.predicate) list -> (* inductive cases *) 
+      (Ident.t * 'a) list * Logic.predicate
+      (* binders, predicate body *)
+
+val inductive_def : 
+  Logic_decl.loc -> Ident.t -> Logic.inductive_def Env.scheme 
+  -> Logic_decl.t list
+
+val push : Logic_decl.t -> Logic_decl.t
 
