@@ -15,24 +15,23 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: pretty.ml,v 1.26 2008/04/11 13:31:54 filliatr Exp $ i*)
+(*i $Id: pretty.ml,v 1.39 2009-01-21 08:34:15 marche Exp $ i*)
 
 open Format
 open Pp
 open Ident
 open Options
+open Misc
 open Logic
 open Logic_decl
 
@@ -40,7 +39,16 @@ let queue = Queue.create ()
 
 let reset () = Queue.clear queue
 
-let push_decl d = Queue.add d queue
+let push_decl ?(ergo=false) d = 
+  if ergo
+  then
+    match d with
+      | Dinductive_def (loc, id, d) ->
+	  List.iter
+	    (fun d -> Queue.add d queue)
+            (PredDefExpansor.inductive_def loc id d)
+      | _ -> Queue.add d queue
+  else Queue.add d queue
 
 let iter f = Queue.iter f queue
 
@@ -93,13 +101,13 @@ let rec term fmt = function
       fprintf fmt "%s.%se%s" i f e
   | Tvar id | Tderef id | Tapp (id, [], _) -> 
       ident fmt id
-  | Tapp (id, [t1; t2], _) when id == t_add_int ->
+  | Tapp (id, [t1; t2], _) when id == t_add_int || id == t_add_real ->
       fprintf fmt "(%a + %a)" term t1 term t2
-  | Tapp (id, [t1; t2], _) when id == t_sub_int ->
+  | Tapp (id, [t1; t2], _) when id == t_sub_int || id == t_sub_real ->
       fprintf fmt "(%a - %a)" term t1 term t2
-  | Tapp (id, [t1; t2], _) when id == t_mul_int ->
+  | Tapp (id, [t1; t2], _) when id == t_mul_int || id == t_mul_real ->
       fprintf fmt "(%a * %a)" term t1 term t2
-  | Tapp (id, [t1; t2], _) when id == t_div_int ->
+  | Tapp (id, [t1; t2], _) when id == t_div_int || id == t_div_real ->
       fprintf fmt "(%a / %a)" term t1 term t2
   | Tapp (id, [t1; t2], _) when id == t_mod_int ->
       fprintf fmt "(%a %% %a)" term t1 term t2
@@ -118,6 +126,13 @@ let int_relation_string id =
   else if id == t_ge_int then ">="
   else assert false
 
+let real_relation_string id =
+  if id == t_lt_real then "<" 
+  else if id == t_le_real then "<="
+  else if id == t_gt_real then ">"
+  else if id == t_ge_real then ">="
+  else assert false
+
 let rec predicate fmt = function
   | Pvar id | Papp (id, [], _) -> 
       ident fmt id
@@ -127,6 +142,8 @@ let rec predicate fmt = function
       fprintf fmt "(%a <> %a)" term t1 term t2
   | Papp (id, [t1; t2], _) when is_int_comparison id ->
       fprintf fmt "(%a %s %a)" term t1 (int_relation_string id) term t2
+  | Papp (id, [t1; t2], _) when is_real_comparison id ->
+      fprintf fmt "(%a %s %a)" term t1 (real_relation_string id) term t2
   | Papp (id, [a;b], _) when id == t_zwf_zero ->
       fprintf fmt "@[((0 <= %a) and@ (%a < %a))@]" term b term a term b
   | Papp (id, [t], _) when id == well_founded ->
@@ -153,12 +170,18 @@ let rec predicate fmt = function
       fprintf fmt "(@[%a or@ %a@])" predicate a predicate b
   | Pnot a ->
       fprintf fmt "(not %a)" predicate a
-  | Forall (_,_,b,v,tl,p) ->
+  | Forall (_,id,n,v,tl,p) ->
+      let id = next_away id (predicate_vars p) in
+      let s = subst_onev n id in
+      let p = subst_in_predicate s p in
+      let tl = List.map (List.map (subst_in_pattern s)) tl in
       fprintf fmt "@[<hov 2>(forall %a:%a%a.@ %a)@]"
-	ident b pure_type v print_triggers tl predicate p
-  | Exists (_,b,v,p) ->
+	ident id pure_type v print_triggers tl predicate p
+  | Exists (id,n,v,p) ->
+      let id = next_away id (predicate_vars p) in
+      let p = subst_in_predicate (subst_onev n id) p in
       fprintf fmt "@[<hov 2>(exists %a:%a.@ %a)@]" 
-	ident b pure_type v predicate p
+	ident id pure_type v predicate p
   | Pfpi (t, (i1,f1,e1), (i2,f2,e2)) ->
       fprintf fmt "@[<hov 2>fpi(%a,@ %s.%se%s,@ %s.%se%s)@]" 
 	term t i1 f1 e1 i2 f2 e2
@@ -212,11 +235,19 @@ let decl fmt d =
       fprintf fmt "@[logic %s : %a@]" id logic_type lt
   | Dpredicate_def (_, id, def) ->
       let bl,p = specialize def in
-      fprintf fmt "@[<hov 2>predicate %s(%a) =@ %a@]" id 
+      fprintf fmt "@[<hov 2>predicate %a(%a) =@ %a@]" ident id 
 	(print_list comma logic_binder) bl predicate p
+  | Dinductive_def (_, id, indcases) ->
+      let bl,l = specialize indcases in
+      fprintf fmt 
+	"@[<hov 0>predicate %a: @[%a -> prop@] {@\n  @[<v 0>%a@]@\n}@\n@]" 
+	ident id 
+	(print_list comma pure_type) bl 
+	(print_list newline 
+	   (fun fmt (id,p) -> fprintf fmt "@[%a: %a;@]" ident id predicate p)) l
   | Dfunction_def (_, id, def) ->
       let bl,pt,t = specialize def in
-      fprintf fmt "@[<hov 2>function %s(%a) : %a =@ %a@]" id
+      fprintf fmt "@[<hov 2>function %a(%a) : %a =@ %a@]" ident id
 	(print_list comma logic_binder) bl pure_type pt term t
   | Daxiom (_, id, p) ->
       let p = specialize p in
@@ -246,6 +277,7 @@ let output_file f =
   if explain_vc then print_in_file print_traces (f ^ "_why.xpl")
 
 let output_files f =
+  eprintf "Starting Multi-Why output with basename %s@." f;
   let po = ref 0 in
   print_in_file
     (fun ctxfmt ->
@@ -261,7 +293,21 @@ let output_files f =
 	    | d -> 
 		decl ctxfmt d)
 	 queue)
-    (f ^ "_ctx.why")
+    (f ^ "_ctx.why");
+  eprintf "Multi-Why output done@."
+  
+let push_or_output_decl = 
+  let po = ref 0 in 
+  function d ->
+    match d with
+      | Dgoal (loc,expl,id,_) as d -> 
+	  incr po;
+	  let fpo = Options.out_file (id ^ ".why") in
+	  print_in_file (fun fmt -> decl fmt d) fpo;
+	  if explain_vc then
+	    let ftr = Options.out_file (id ^ ".xpl") in
+	    print_in_file (fun fmt -> print_trace fmt id ((*loc,*)expl)) ftr
+      | d -> push_decl d
 
 module SMap = Map.Make(String)
 
@@ -300,22 +346,6 @@ let output_project f =
 		decl ctxfmt d)
 	 queue)
     (f ^ "_ctx.why");
-(*  Env.iter_global
-    (fun (x,ty) -> 
-       let fn = "function "^Ident.string x in
-       let size = String.length fn in
-       try 
-	 if (String.sub fn (size-5) 4 = "_imp")
-	 then
-	   begin
-	     eprintf "function %s@." fn;
-	     try
-	       let _ = SMap.find fn !functions in ()
-	     with Not_found ->
-	       functions := SMap.add fn SMap.empty !functions 
-	   end
-	 else ()
-       with Invalid_argument _ -> ());*)
   Hashtbl.iter 
     (fun key (fn,beh,loc) -> 
        try
@@ -348,4 +378,5 @@ let output_project f =
 	      vcs)
 	 behs)
     !functions;
-  Project.save p f
+  Project.save p f;
+  p
