@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: ptree.mli,v 1.39 2008/05/28 13:51:26 marche Exp $ i*)
+(*i $Id: ptree.mli,v 1.44 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (*s Parse trees. *)
 
@@ -118,6 +116,9 @@ and t_desc =
   | Sderef of variable
   | Sloop of assertion option * variant option * t
   | Sif of t * t * t
+  | Slazy_and of t * t
+  | Slazy_or of t * t
+  | Snot of t
   | Sapp of t * t
   | Sletref of variable * t * t
   | Sletin of variable * t * t
@@ -151,6 +152,7 @@ type decl =
   | Exception of loc * Ident.t * ppure_type option
   | Logic of loc * external_ * Ident.t list * plogic_type
   | Predicate_def of loc * Ident.t * (loc * Ident.t * ppure_type) list * lexpr
+  | Inductive_def of loc * Ident.t * plogic_type * (loc * Ident.t * lexpr) list
   | Function_def 
       of loc * Ident.t * (loc * Ident.t * ppure_type) list * ppure_type * lexpr
   | Axiom of loc * Ident.t * lexpr
