@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: pvs.ml,v 1.88 2008/02/20 14:34:26 marche Exp $ i*)
+(*i $Id: pvs.ml,v 1.99 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 open Logic
 open Logic_decl
@@ -41,22 +39,86 @@ open Format
 open Vcg
 open Pp
 
-let relation id =
-  if id == t_lt then "<" 
-  else if id == t_le then "<="
-  else if id == t_gt then ">"
-  else if id == t_ge then ">="
-  else if id == t_eq then "="
-  else if id == t_neq then "/="
-  else assert false
+
+let infix_table = ref Idmap.empty
+
+let () =
+  List.iter (fun (id,s) -> infix_table := Idmap.add id s !infix_table)
+    [ t_lt, "<" ;
+      t_le, "<=";
+      t_gt, ">";
+      t_ge, ">=";
+      t_eq, "=";
+      t_neq, "/=";
+      t_bool_and, "AND";
+      t_bool_or, "OR";
+      t_bool_xor, "/=" ;
+      t_gt_int_bool, ">";
+      t_gt_real_bool, ">";
+      t_ge_int_bool, ">=";
+      t_ge_real_bool, ">=";
+      t_le_int_bool, "<=";
+      t_le_real_bool, "<=";
+      t_lt_int_bool, "<";
+      t_lt_real_bool, "<";
+      t_eq_int_bool, "=";
+      t_eq_real_bool, "=";
+      t_neq_int_bool, "/=";
+      t_neq_real_bool, "/=";
+      t_lt_int, "<" ;
+      t_le_int, "<=";
+      t_gt_int, ">";
+      t_ge_int, ">=";
+      t_eq_int, "=";
+      t_neq_int, "/=";
+      t_lt_real, "<" ;
+      t_le_real, "<=";
+      t_gt_real, ">";
+      t_ge_real, ">=";
+      t_eq_real, "=";
+      t_neq_real, "/=";
+    ]
+
+let is_infix id = Idmap.mem id !infix_table
+	
+let infix id = 
+  try
+    Idmap.find id !infix_table
+  with Not_found -> assert false
+
+
+
+let prefix_table = ref Idmap.empty
+
+let () =
+  List.iter (fun (id,s) -> prefix_table := Idmap.add id s !prefix_table)
+    [ t_abs_real, "abs";
+      t_real_max, "max";
+      t_real_min, "min";
+      t_sqrt_real, "sqrt";
+    ]
+
+let is_prefix id = Idmap.mem id !prefix_table
+	
+let prefix id = 
+  try
+    Idmap.find id !prefix_table
+  with Not_found -> assert false
+
 
 let print_real fmt = function
   | "","0",_ | "0","",_ | "0","0",_ -> 
-      fprintf fmt "0"
+      fprintf fmt "0.0"
+  | "",f,"" -> 
+      fprintf fmt "0.%s" f
+  | i,"","" -> 
+      fprintf fmt "%s.0" i
+  | i,f,"" ->
+      fprintf fmt "%s.%s" i f      
   | i,f,e ->
-      let e = (if e = "" then 0 else int_of_string e) - String.length f in
+      let e = (int_of_string e) - String.length f in
       if e = 0 then
-	fprintf fmt "(%s%s :: real)" i f
+	fprintf fmt "%s%s" i f
       else if e > 0 then
 	fprintf fmt "(%s%s * 1%s)" i f (String.make e '0')
       else
@@ -64,6 +126,12 @@ let print_real fmt = function
 
 let ident = Ident.print
 
+let rec filter_phantom_type = function
+  | PTexternal (_, id) ->
+      not (Hashtbl.mem Options.phantom_types (Ident.string id))
+  | PTvar { type_val = Some t} -> filter_phantom_type t   
+  | _ -> true
+
 let rec print_pure_type fmt = function
   | PTint -> fprintf fmt "int"
   | PTbool -> fprintf fmt "bool"
@@ -73,16 +141,18 @@ let rec print_pure_type fmt = function
       fprintf fmt "warray[%a]" print_pure_type pt
   | PTvar { type_val = Some t} -> fprintf fmt "%a" print_pure_type t      
   | PTvar v -> fprintf fmt "A%d" v.tag
-  | PTexternal (i, id) -> fprintf fmt "%a%a" ident id print_instance i
+  | PTexternal (i, id) -> 
+      fprintf fmt "%a%a" ident id print_instance i
 
-and print_instance fmt = function
-  | [] -> ()
-  | i -> fprintf fmt "[%a]" (print_list comma print_pure_type) i
+and print_instance fmt l = 
+  match List.filter filter_phantom_type l with
+    | [] -> ()
+    | i -> fprintf fmt "[%a]" (print_list comma print_pure_type) i
 
 let print_term fmt t = 
   let rec print0 fmt = function
-    | Tapp (id, [a;b], _) when is_relation id ->
-	fprintf fmt "@[<hov 2>%a %s@ %a@]" print1 a (relation id) print1 b
+    | Tapp (id, [a;b], _) when is_infix id ->
+	fprintf fmt "@[<hov 2>(%a %s@ %a)@]" print1 a (infix id) print1 b
     | t -> 
 	print1 fmt t
   and print1 fmt = function
@@ -107,15 +177,15 @@ let print_term fmt t =
 	print3 fmt t
   and print3 fmt = function
     | Tconst (ConstInt n) -> 
-	fprintf fmt "(%s :: int)" n
+	fprintf fmt "%s" n
     | Tconst (ConstBool b) -> 
 	fprintf fmt "%b" b
     | Tconst ConstUnit -> 
 	fprintf fmt "unit" 
     | Tconst (ConstFloat f) -> 
 	print_real fmt f
-    | Tapp (id, [Tconst (ConstInt n)], _) when id == t_real_of_int ->
-	fprintf fmt "(%s :: real)" n
+    | Tapp (id, [t], _) when id == t_real_of_int ->
+	fprintf fmt "%a" print3 t
     | Tderef _ ->
 	assert false
     | Tvar id when id == implicit ->
@@ -124,11 +194,23 @@ let print_term fmt t =
 	fprintf fmt "zwf_zero"
     | Tvar id ->
 	Ident.print fmt id
+    | Tapp (id, [a; Tapp (id', [b], _)], _) 
+      when id == t_pow_real && id' == t_real_of_int ->
+	fprintf fmt "(@[%a@ ^ %a@])" print3 a print3 b
+    | Tapp (id, [a;b], _) when id == t_pow_real ->
+	fprintf fmt "(@[%a@ ^ %a@])" print3 a print3 b
+    | Tapp (id, tl, i) when is_prefix id -> 
+	fprintf fmt "%s%a(@[%a@])" 
+	  (prefix id) print_instance (List.rev i) (print_list comma print0) tl
     | Tapp (id, [t], _) when id == t_neg_int || id == t_neg_real ->
 	fprintf fmt "-%a" print3 t
+    | Tapp (id, [t], _) when id == t_bool_not ->
+	fprintf fmt "(NOT(%a))" print3 t
     | Tapp (id, [a; b; c], _) when id == if_then_else -> 
-	fprintf fmt "(@[if %a@ then %a@ else %a@])" print0 a print0 b print0 c
-    | Tapp (id, l, _) as t when is_relation id || is_arith_binop id ->
+	fprintf fmt "(@[IF %a@ THEN %a@ ELSE %a@ ENDIF@])" print0 a print0 b print0 c
+    | Tapp (id, l, _) as t when is_infix id ->
+	fprintf fmt "@[%a@]" print0 t
+    | Tapp (id, l, _) as t when is_arith_binop id ->
 	fprintf fmt "@[(%a)@]" print0 t
     | Tapp (id, [], i) -> 
 	fprintf fmt "%a%a" ident id print_instance (List.rev i)
@@ -142,20 +224,6 @@ let print_term fmt t =
 let print_logic_binder fmt (id,pt) =
   fprintf fmt "%s:%a" (Ident.string id) print_pure_type pt
 
-let infix_relation id =
-  if id == t_lt_int then "<" 
-  else if id == t_le_int then "<="
-  else if id == t_gt_int then ">"
-  else if id == t_ge_int then ">="
-  else if id == t_eq_int then "="
-  else if id == t_neq_int then "/="
-  else if id == t_lt_real then "<" 
-  else if id == t_le_real then "<="
-  else if id == t_gt_real then ">"
-  else if id == t_ge_real then ">="
-  else if id == t_eq_real then "="
-  else if id == t_neq_real then "/="
-  else assert false
 
 let print_predicate fmt p =
   let rec print0 fmt = function
@@ -194,7 +262,7 @@ let print_predicate fmt p =
 	fprintf fmt "zwf_zero(%a, %a)" print_term a print_term b
     | Papp (id, [a;b], _) when is_int_comparison id || is_real_comparison id ->
 	fprintf fmt "@[%a %s@ %a@]" 
-	  print_term a (infix_relation id) print_term b
+	  print_term a (infix id) print_term b
     | Papp (id, [a;b], _) when is_eq id ->
 	fprintf fmt "@[%a =@ %a@]" print_term a print_term b
     | Papp (id, [a;b], _) when is_neq id ->
@@ -259,6 +327,7 @@ let begin_sub_theory fmt n th =
     fprintf fmt ": TYPE]"
   end;
   fprintf fmt ": THEORY@\nBEGIN@\n@\n";
+  fprintf fmt "  %s@\n" Options.pvs_preamble;
   import_last_theory fmt
     
 let end_theory fmt th =
@@ -277,7 +346,7 @@ let print_logic_type fmt = function
 	(print_list comma print_pure_type) pl print_pure_type t
 
 let declare_type fmt id = 
-  fprintf fmt "  @[%s: NONEMPTY_TYPE;@]@\n@\n" id
+  fprintf fmt "  @[%s: TYPE+;@]@\n@\n" id
 
 let print_logic fmt id t = 
   fprintf fmt "  %%%% Why logic %s@\n" id;
@@ -288,12 +357,18 @@ let print_axiom fmt id p =
   fprintf fmt "  @[<hov 2>%s: AXIOM@ @[%a@]@]@\n@\n" id print_predicate p
     
 let print_predicate_def fmt id (bl,p) =
-  fprintf fmt "  @[<hov 2>%s(@[%a@]) : bool =@ @[%a@]@]@\n@\n"
-    id (print_list comma print_logic_binder) bl print_predicate p
+  fprintf fmt "  @[<hov 2>%a(@[%a@]) : bool =@ @[%a@]@]@\n@\n"
+    ident id (print_list comma print_logic_binder) bl print_predicate p
+    
+let print_inductive_def fmt id inddef =
+  let (vars,(bl,cases)) = Env.specialize_inductive_def inddef in
+  let newvars,body = PredDefExpansor.inductive_inverse_body id bl cases in
+  fprintf fmt "  @[<hov 2>INDUCTIVE %a(@[%a@]) : bool =@ @[%a@]@]@\n@\n"
+    ident id (print_list comma print_logic_binder) newvars print_predicate body
     
 let print_function_def fmt id (bl,t,e) =
-  fprintf fmt "  @[<hov 2>%s(@[%a@]) : %a =@ @[%a@]@]@\n@\n"
-    id (print_list comma print_logic_binder) bl 
+  fprintf fmt "  @[<hov 2>%a(@[%a@]) : %a =@ @[%a@]@]@\n@\n"
+    ident id (print_list comma print_logic_binder) bl 
     print_pure_type t print_term e
     
 let print_obligation fmt (loc,expl,id,s) =
@@ -324,7 +399,7 @@ let print_goal fmt (loc, expl, id, s) =
     for i = !tvar_so_far + 1 to n do
       fprintf fmt "A%d" i; if i < n then fprintf fmt ", "
     done;
-    fprintf fmt ": NONEMPTY_TYPE;@\n@\n";
+    fprintf fmt ": TYPE+;@\n@\n";
     tvar_so_far := n
   end;
   let l,s = Env.specialize_sequent s in
@@ -344,6 +419,7 @@ let print_axiom_scheme fmt id s =
 type def =
   | DefFunction of function_def scheme
   | DefPredicate of predicate_def scheme
+  | DefInductive of inductive_def scheme
 
 let print_def_scheme fmt id = function
   | DefFunction s -> 
@@ -354,6 +430,10 @@ let print_def_scheme fmt id = function
       let l,d = Env.specialize_predicate_def s in
       print_scheme l;
       print_predicate_def fmt id d
+  | DefInductive s ->
+      let l,d = Env.specialize_inductive_def s in
+      print_scheme l;      
+      print_inductive_def fmt id s
 
 let queue = Queue.create ()
 
@@ -368,6 +448,7 @@ let output_elem fmt = function
   | Dtype _ -> assert false
   | Dlogic (loc, id, t) -> print_logic fmt id t.scheme_type
   | Dpredicate_def (loc, id, d) -> print_predicate_def fmt id d.scheme_type
+  | Dinductive_def(loc, ident, inddef) -> print_inductive_def fmt ident inddef
   | Dfunction_def (loc, id, d) -> print_function_def fmt id d.scheme_type
   | Daxiom (loc, id, p) -> print_axiom fmt id p.scheme_type
   | Dgoal (loc, expl, id, s) -> print_goal fmt (loc, expl, id, s)
@@ -438,9 +519,11 @@ let sort_theory () =
     | Dlogic (_, id, s) -> 
 	poly s; ArMap.add_scheme s (id,s) th.decls
     | Dpredicate_def (_, id, s) -> 
-	poly s; Queue.add (id, DefPredicate s) th.defs
+	poly s; Queue.add (Ident.string id, DefPredicate s) th.defs
+    | Dinductive_def(loc, ident, inddef) ->
+	poly inddef; Queue.add (Ident.string ident, DefInductive inddef) th.defs
     | Dfunction_def (_, id, s) -> 
-	poly s; Queue.add (id, DefFunction s) th.defs
+	poly s; Queue.add (Ident.string id, DefFunction s) th.defs
     | Daxiom (_, id, s) -> 
 	poly s; ArMap.add_scheme s (id,s) th.axioms
     | Dgoal (loc, expl, id, s) -> 
@@ -483,7 +566,9 @@ let output_theory fmt th_name =
       (fun (id,def) -> 
 	let n = 
 	  let vars = match def with 
-	    | DefFunction s -> s.scheme_vars  | DefPredicate s -> s.scheme_vars
+	    | DefFunction s -> s.scheme_vars
+	    | DefInductive s -> s.scheme_vars
+	    | DefPredicate s -> s.scheme_vars
 	  in
 	  Vset.cardinal vars
 	in
@@ -492,7 +577,7 @@ let output_theory fmt th_name =
 	import_types ();
 	import_decls ();
 	importing !defs_so_far;
-	print_def_scheme fmt id def;
+	print_def_scheme fmt (Ident.create id) def;
 	end_theory fmt name;
         defs_so_far := name :: !defs_so_far)
       th.defs;
