@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: regen.ml,v 1.27 2008/02/20 14:34:26 marche Exp $ i*)
+(*i $Id: regen.ml,v 1.30 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (* files partly edited and partly regenerated *)
 
@@ -46,6 +44,7 @@ type element_kind =
   | Lg
   | Ax
   | Pr
+  | Ind
   | Fun
   | Ty
 
@@ -58,6 +57,7 @@ type element =
   | Logic of string * logic_type Env.scheme
   | Axiom of string * predicate Env.scheme
   | Predicate of string * predicate_def Env.scheme
+  | Inductive of string * inductive_def Env.scheme
   | Function of string * function_def Env.scheme
   | AbstractType of string * string list
 
@@ -91,6 +91,7 @@ module Make(X : S) = struct
     | Lg, s -> fprintf fmt "logic %s" s
     | Ax, s -> fprintf fmt "axiom %s" s
     | Pr, s -> fprintf fmt "predicate %s" s
+    | Ind, s -> fprintf fmt "inductive %s" s
     | Fun, s -> fprintf fmt "function %s" s
     | Ty, s -> fprintf fmt "type %s" s
 
