@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: simplify.ml,v 1.80 2008/02/20 14:34:26 marche Exp $ i*)
+(*i $Id: simplify.ml,v 1.87 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (*s Simplify's output *)
 
@@ -49,14 +47,21 @@ type elem =
 
 let queue = Queue.create ()
 
-let reset () = Queue.clear queue; Encoding.reset ()
+let reset () = 
+  Queue.clear queue; 
+  Encoding.reset ()
 
-let decl_to_elem = function
+let rec decl_to_elem = function
   | Dgoal (loc, expl, id, s) -> Queue.add (Oblig (loc, expl,id, s)) queue
   | Daxiom (_, id, p) -> Queue.add (Axiom (id, p)) queue
-  | Dpredicate_def (_, id, p) -> Queue.add (Predicate (id, p)) queue
-  | Dfunction_def (_, id, p) -> Queue.add (FunctionDef (id, p)) queue
-  | _ -> ()
+  | Dpredicate_def (_, id, p) -> 
+      Queue.add (Predicate (Ident.string id, p)) queue
+  | Dfunction_def (_, id, p) -> 
+      Queue.add (FunctionDef (Ident.string id, p)) queue
+  | Dinductive_def (loc, id, d) ->
+      List.iter decl_to_elem (PredDefExpansor.inductive_def loc id d)
+  | Dlogic _ -> ()
+  | Dtype _ -> ()
 
 let push_decl = Encoding.push
 
@@ -64,36 +69,58 @@ let defpred = Hashtbl.create 97
 
 (*s Pretty print *)
 
-let prefix id =
-  if id == t_lt then "<"
-  else if id == t_le then "<="
-  else if id == t_gt then ">"
-  else if id == t_ge then ">="
-  (* int cmp *)
-  else if id == t_lt_int then "<"
-  else if id == t_le_int then "<="
-  else if id == t_gt_int then ">"
-  else if id == t_ge_int then ">="
-  (* int ops *)
-  else if id == t_add_int then "+"
-  else if id == t_sub_int then "-"
-  else if id == t_mul_int then "*"
-  else if id == t_div_int then "int_div"
-  else if id == t_mod_int then "int_mod"
-  (* real ops *)
-  else if id == t_add_real then "add_real"
-  else if id == t_sub_real then "sub_real"
-  else if id == t_mul_real then "mul_real"
-  else if id == t_div_real then "div_real"
-  else if id == t_neg_real then "neg_real"
-  else if id == t_sqrt_real then "sqrt_real"
-  else if id == t_real_of_int then "real_of_int"
-  else if id == t_int_of_real then "int_of_real"
-  else if id == t_lt_real then "lt_real"
-  else if id == t_le_real then "le_real"
-  else if id == t_gt_real then "gt_real"
-  else if id == t_ge_real then "ge_real"
-  else (print_endline (Ident.string id); assert false)
+let prefix_table = ref Idmap.empty
+
+let () =
+  List.iter (fun (id,s) -> prefix_table := Idmap.add id s !prefix_table)
+    [ 
+      (* booleans ops *)
+      t_bool_and, "bool_and";
+      t_bool_or, "bool_or";
+      t_bool_xor, "bool_xor";
+      t_bool_not, "bool_not";
+      (* poly comps *)
+      t_lt, "<";
+      t_le, "<=";
+      t_gt, ">";
+      t_ge, ">=";
+      (* int cmp *)
+      t_lt_int, "<";
+      t_le_int, "<=";
+      t_gt_int, ">";
+      t_ge_int, ">=";
+      (* int ops *)
+      t_add_int, "+";
+      t_sub_int, "-";
+      t_mul_int, "*";
+      t_div_int, "int_div";
+      t_mod_int, "int_mod";
+      (* real ops *)
+      t_add_real, "real_add";
+      t_sub_real, "real_sub";
+      t_mul_real, "real_mul";
+      t_div_real, "real_div";
+      t_neg_real, "real_neg";
+      t_sqrt_real, "real_sqrt";
+      t_real_of_int, "real_of_int";
+      t_int_of_real, "int_of_real";
+      t_lt_real, "real_lt";
+      t_le_real, "real_le";
+      t_gt_real, "real_gt";
+      t_ge_real, "real_ge";
+      t_abs_real, "real_abs";
+      t_real_max, "real_max";
+      t_real_min, "real_min";
+      t_sqrt_real, "real_sqrt";
+    ]
+
+let is_prefix id = Idmap.mem id !prefix_table
+	
+let prefix id = 
+  try
+    Idmap.find id !prefix_table
+  with Not_found -> assert false
+
 
 let is_simplify_ident s =
   let is_simplify_char = function
@@ -152,14 +179,14 @@ let rec print_term fmt = function
 **)
   | Tapp (id, [t], _) when id == t_neg_int ->
       fprintf fmt "@[(- 0 %a)@]" print_term t
-  | Tapp (id, tl, _) when is_relation id || is_arith id ->
+  | Tapp (id, tl, _) when is_prefix id ->
       fprintf fmt "@[(%s %a)@]" (prefix id) print_terms tl
   | Tapp (id, [], _) ->
       ident fmt id
   | Tapp (id, tl, _) ->
       fprintf fmt "@[(%a@ %a)@]" 
 	ident id (print_list space print_term) tl
-  | Tnamed (n, t) ->
+  | Tnamed (_n, t) ->
       (*fprintf fmt "@[;;%s@\n%a@]" n pp p*) 
       print_term fmt t
 
@@ -211,7 +238,7 @@ let rec print_predicate pos fmt p =
       ident fmt id
   | Papp (id, tl, _) when id == t_distinct ->
       fprintf fmt "@[(DISTINCT@ %a)@]" print_terms tl
-  | Papp (id, [t], _) when id == well_founded ->
+  | Papp (id, [_t], _) when id == well_founded ->
       fprintf fmt "TRUE ; was well_founded@\n"
   | Papp (id, [a; b], _) when is_eq id ->
       fprintf fmt "@[(EQ %a@ %a)@]" print_term a print_term b
@@ -264,13 +291,13 @@ let rec print_predicate pos fmt p =
       let bv,p = Util.decomp_forall p in
       let var fmt (x,_) = ident fmt x in
       fprintf fmt "@[(FORALL (%a)@ %a)@]" (print_list space var) bv pp p
-  | Exists (id,n,t,p) -> 
+  | Exists (id,n,_t,p) -> 
       let id' = next_away id (predicate_vars p) in
       let p' = subst_in_predicate (subst_onev n id') p in
       fprintf fmt "@[(EXISTS (%a)@ %a)@]" ident id' pp p'
   | Pfpi _ ->
       failwith "fpi not supported with Simplify"
-  | Pnamed (n, p) ->
+  | Pnamed (_n, p) ->
       (*fprintf fmt "@[;;%s@\n%a@]" n pp p*) 
       pp fmt p
   (** BUG Simplify
@@ -299,7 +326,7 @@ let print_sequent fmt (hyps,concl) =
   let rec print_seq fmt = function
     | [] ->
 	print_predicate false fmt concl
-    | Svar (id, v) :: hyps -> 
+    | Svar (id, _v) :: hyps -> 
 	fprintf fmt "@[(FORALL (%a)@ %a)@]" ident id print_seq hyps
     | Spred (_,p) :: hyps -> 
 	fprintf fmt "@[(IMPLIES %a@ %a)@]" 
@@ -362,7 +389,7 @@ let idents_plus_prefix fmt s  =
       fprintf fmt "|%s|" s
 
 let print_function fmt id p =
-  let (bl,pt,e) = p.Env.scheme_type in
+  let (bl,_pt,e) = p.Env.scheme_type in
   match bl with 
     [] -> 
       fprintf fmt "@[(BG_PUSH@\n ;; Why function %s@\n" id;
@@ -376,7 +403,7 @@ let print_function fmt id p =
 	print_term e
 
 let print_elem fmt = function
-  | Oblig (loc, expl, id, s) -> print_obligation fmt loc id s
+  | Oblig (loc, _expl, id, s) -> print_obligation fmt loc id s
   | Axiom (id, p) -> print_axiom fmt id p
   | Predicate (id, p) -> print_predicate fmt id p
   | FunctionDef (id, f) -> print_function fmt id f
@@ -385,7 +412,7 @@ let output_file fwe =
   let sep = ";; DO NOT EDIT BELOW THIS LINE" in
   let file = out_file (fwe ^ "_why.sx") in
   do_not_edit_below ~file
-    ~before:(fun fmt -> ())
+    ~before:(fun _fmt -> ())
     ~sep
     ~after:(fun fmt -> 
    Encoding.iter decl_to_elem;
