@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: theoryreducer.ml,v 1.10 2008/02/05 12:10:50 marche Exp $ i*)
+(*i $Id: theoryreducer.ml,v 1.14 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (*s Harvey's output *)
 
@@ -484,12 +482,28 @@ let declareType id ax =
     
     
 let launcher decl = match decl with   
-  | Dtype (_, _, id) as ax -> (*Printf.printf  "Dtype %s \n"  id ;*) declareType id ax 
-  | Dlogic (_, id, t) as ax -> (* Printf.printf  "Dlogic %s \n"  id ;*) typingPredicate  id ax
-  | Dpredicate_def (_, id, d) as ax -> (*Printf.printf  "Dpredicate_def %s \n"  id ; *)managesPredicate id ax d.scheme_type
-  | Dfunction_def (_, id, d)  as ax -> (*Printf.printf  "Dfunction_def %s \n"  id ; *)managesFunction  id ax d.scheme_type
-  | Daxiom (_, id, p) as ax         -> (*Printf.printf  "Daxiom %s \n"  id ; *)managesAxiom  id ax p.scheme_type
-  | Dgoal (_, expl, id, s)  as ax -> (*Printf.printf  "Dgoal %s \n"  id ; *)managesGoal id ax s.Env.scheme_type 
+  | Dtype (_, _, id) as ax -> 
+      (*Printf.printf  "Dtype %s \n"  id ;*) 
+      declareType id ax 
+  | Dlogic (_, id, t) as ax -> 
+      (* Printf.printf  "Dlogic %s \n"  id ;*) 
+      typingPredicate  id ax
+  | Dpredicate_def (_, id, d) as ax -> 
+      (*Printf.printf  "Dpredicate_def %s \n"  id ; *)
+      let id = Ident.string id in
+      managesPredicate id ax d.scheme_type
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "Theory reducer: inductive def not yet supported"
+  | Dfunction_def (_, id, d)  as ax -> 
+      (*Printf.printf  "Dfunction_def %s \n"  id ; *)
+      let id = Ident.string id in
+      managesFunction  id ax d.scheme_type
+  | Daxiom (_, id, p) as ax -> 
+      (*Printf.printf  "Daxiom %s \n"  id ; *)
+      managesAxiom  id ax p.scheme_type
+  | Dgoal (_, expl, id, s)  as ax -> 
+      (*Printf.printf  "Dgoal %s \n"  id ; *)
+      managesGoal id ax s.Env.scheme_type 
 
 
 
@@ -497,8 +511,14 @@ let display q =
   let displayMatch e  = match e with   
   | Dtype (_, _, id) -> Printf.printf  "%s \n"  id  
   | Dlogic (_, id, t) -> Printf.printf  "%s \n" id   
-  | Dpredicate_def (_, id, d) -> Printf.printf  "%s \n" id
-  | Dfunction_def (_, id, d) -> Printf.printf  "%s \n" id
+  | Dpredicate_def (_, id, d) -> 
+      let id = Ident.string id in
+      Printf.printf  "%s \n" id
+  | Dinductive_def(loc, ident, inddef) ->
+      failwith "Theory reducer: inductive def not yet supported"
+  | Dfunction_def (_, id, d) -> 
+      let id = Ident.string id in
+      Printf.printf  "%s \n" id
   | Daxiom (_, id, p)          -> Printf.printf  "%s \n"  id
   | Dgoal (_, expl, id, s)   -> Printf.printf  "%s \n"  id
   in
