@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: typing.ml,v 1.136 2008/04/10 14:43:57 filliatr Exp $ i*)
+(*i $Id: typing.ml,v 1.141 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (*s Typing. *)
 
@@ -97,7 +95,8 @@ let decomp_fun_type loc = function
       x, v, type_c_of_v (Arrow (bl, k))
   | Arrow ([], _) ->
       assert false
-  | _ -> 
+  | ty -> 
+      Format.eprintf "decomp_fun_type=%a@." print_type_v ty;
       raise_located loc AppNonFunction
 
 let expected_type loc t et =
@@ -180,6 +179,11 @@ let gmake_node loc env userlabel l ?(post=None) p rt e =
 	     t_result_name = result; 
 	     t_result_type = rt; t_effect = e; t_post = post } }
 
+
+let bool_constant b loc env =
+  gmake_node loc env "" (label_name ()) (Expression (Tconst (ConstBool b)))
+    type_v_bool Effect.bottom
+
 let make_arrow_type lab bl k =
   let k = 
     let q = optpost_app (change_label lab "") k.c_post in
@@ -206,6 +210,15 @@ let typing_info_of_type_c loc env l k =
     t_effect = k.c_effect;
     t_post = optpost_app (post_named loc) k.c_post }
 
+let assume loc env ef p =
+  let k = { c_result_name = result; c_result_type = type_v_unit;
+	    c_effect = ef; c_pre = []; c_post = Some (p,[]) } in
+  gmake_node loc env "" (label_name ()) (Any k) type_v_unit ef
+
+let seq loc env e1 e2 =
+  gmake_node loc env "" (label_name ()) (Seq (e1, e2)) (result_type e2)
+    (Effect.union (effect e1) (effect e2))
+
 (*s Typing variants. 
     Return the effect i.e. variables appearing in the variant. *)
 
@@ -330,9 +343,6 @@ let check_array_type loc env id =
     | Not_found -> raise_located loc (UnboundArray id)
     | Invalid_argument _ -> raise_located loc (NotAnArray id)
       
-let check_no_effect loc ef =
-  if not (Effect.get_writes ef = []) then raise_located loc HasSideEffects
-
 let is_pure_type = function
   | PureType _ -> true
   | _ -> false
@@ -347,6 +357,22 @@ let decompose_app e =
     | _ -> e, args
   in
   loop [] e
+
+let pdesc loc p = { pdesc = p; ploc = loc }
+let sapp loc e1 e2 = { pdesc = Sapp (e1, e2); ploc = loc }
+let svar loc v = { pdesc = Svar v; ploc = loc }
+let sassume loc p = 
+  let a = { pa_name = Anonymous; pa_value = p; pa_loc = loc } in
+  let k = { pc_result_name = result; pc_result_type = PVpure PPTunit;
+	    pc_effect = { pe_reads = []; pe_writes = []; pe_raises = [] };
+	    pc_pre = []; pc_post = Some (a, []) } in
+  { pdesc = Sany k; ploc = loc }
+let sletin loc v e1 e2 = { pdesc = Sletin (v, e1, e2); ploc = loc }
+let sseq loc e1 e2 = { pdesc = Sseq (e1, e2); ploc = loc }
+
+let ppinfix loc l1 i l2 = { pp_loc = loc; pp_desc = PPinfix (l1, i, l2) }
+let ppvar loc v = { pp_loc = loc; pp_desc = PPvar v }
+let ppconst loc c = { pp_loc = loc; pp_desc = PPconst c }
     
 (*s Saturation of postconditions: a postcondition must be set for
     any possibly raised exception *)
@@ -406,6 +432,24 @@ let without_exn_check f x =
   end else
     f x
 
+(*s Pure expressions. Used in [Slazy_and] and [Slazy_or] to decide
+    whether to use [strict_bool_and_] and [strict_bool_or_] or not. *)
+
+let has_no_effect e = 
+  let ef = effect e in get_writes ef = [] && get_exns ef = []
+
+let rec is_pure_expr e = 
+  has_no_effect e &&
+  match e.desc with
+  | Var _ | Expression _ -> true
+  | If (e1, e2, e3) -> is_pure_expr e1 && is_pure_expr e2 && is_pure_expr e3
+  | LetIn (_, e1, e2) -> is_pure_expr e1 && is_pure_expr e2
+  | Label (_, e1) | Assertion (_, [], e1) -> is_pure_expr e1
+  | AppRef (e1, _, _) | AppTerm (e1, _, _) -> is_pure_expr e1
+  | Any _ | Rec _ | Lam _ | Try _ 
+  | Raise _ | Post _ | Assertion _ | LetRef _ 
+  | Loop _ | Seq _ | Absurd -> false
+
 (*s Typing programs. We infer here the type with effects. 
     [lab] is the set of labels, [env] the environment 
     and [expr] the program. [pre] indicates whether preconditions are true 
@@ -655,6 +699,59 @@ let rec typef ?(userlabel="") lab env expr =
       let v = type_v_sup loc t1 t2 in
       make_node toplabel (If (t_b, t_e1, t_e2)) v ef
 
+  | Slazy_and (e1, e2) ->
+      let t_e1 = typef lab env e1 in
+      expected_type e1.ploc (result_type t_e1) type_v_bool;
+      let t_e2 = typef lab env e2 in
+      expected_type e2.ploc (result_type t_e2) type_v_bool;
+      if not split_bool_op && is_pure_expr t_e2 then
+	(* we build strict_bool_and_(e1, e2)
+	   TODO:  avoid typing e1 and e2 twice *)
+	let d = sapp loc (sapp loc (svar loc strict_bool_and_) e1) e2 in
+	(* we build let v = e1 in strict_bool_and_ v1 (assume v1=true; e2)
+	let v = fresh_var () in
+	let p = 
+	  ppinfix loc (ppvar loc v) PPeq (ppconst loc (ConstBool true)) 
+	in
+	let d = 
+	  sletin loc v e1 
+	    (sapp loc (sapp loc (svar loc strict_bool_and_) (svar loc v)) 
+		(sseq loc (sassume loc p) e2))
+	in
+	*)
+	typef lab env d
+      else
+	let ef = union (effect t_e1) (effect t_e2) in
+	let bool_false = bool_constant false loc env in
+	make_node toplabel (If (t_e1, t_e2, bool_false)) type_v_bool ef
+
+  | Slazy_or (e1, e2) ->
+      let t_e1 = typef lab env e1 in
+      expected_type e1.ploc (result_type t_e1) type_v_bool;
+      let t_e2 = typef lab env e2 in
+      expected_type e2.ploc (result_type t_e2) type_v_bool;
+      if not split_bool_op && is_pure_expr t_e2 then
+	(* we build strict_bool_or_(e1, e2)
+	   TODO:  avoid typing e1 and e2 twice *)
+	let d = sapp loc (sapp loc (svar loc strict_bool_or_) e1) e2 in
+	typef lab env d
+      else
+	let ef = union (effect t_e1) (effect t_e2) in
+	let bool_true = bool_constant true loc env in
+	make_node toplabel (If (t_e1, bool_true, t_e2)) type_v_bool ef
+
+  | Snot e1 -> 
+      let t_e1 = typef lab env e1 in
+      expected_type e1.ploc (result_type t_e1) type_v_bool;
+      if not split_bool_op then
+	let d = sapp loc (svar loc bool_not_) e1 in
+	typef lab env d
+      else
+	let bool_false = bool_constant false loc env in
+	let bool_true = bool_constant true loc env in
+	let d = If (t_e1, bool_false, bool_true) in
+	make_node toplabel d type_v_bool (effect t_e1)
+
   | Srec (f,bl,v,var,p,e) ->
       let loc_e = e.ploc in
       let bl',env' = binders loc lab env bl in
