@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 (*s Verification Conditions Generator. *)
@@ -863,15 +861,7 @@ let rec explain_for_pred internal user = function
       (if debug then
 	 eprintf "User label for explanation: `%a'@." (Pp.print_option pp_print_string)user;
        user)
-(*
-      (match user with
-	| None -> None
-	| Some lab -> 
-	   try Some(Util.loc_of_label lab) with Not_found -> 
-	     if debug then
-	       eprintf "Warning: no loc found for label %s@." lab;
-	     None)
-*),
+	,
       (match internal with
 	| Some n ->
 	    begin
@@ -887,8 +877,7 @@ let rec explain_for_pred internal user = function
 		    assert false
 	    end
 	| None -> 
-	    fprintf str_formatter 
-	      "unexplained assertion `%a'" print_predicate p;
+	    fprintf str_formatter "unexplained assertion";
 	    VCEexternal(flush_str_formatter())) 
 	      
 (* Proof obligations from the WP *)
