@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: zenon.ml,v 1.28 2008/02/18 11:06:36 moy Exp $ i*)
+(*i $Id: zenon.ml,v 1.33 2008-11-05 14:03:18 filliatr Exp $ i*)
 
 (*s Zenon output *)
 
@@ -287,7 +285,7 @@ let print_logic fmt id t =
 
 let print_predicate_def fmt id (bl,p) =
   fprintf fmt "@[;; Why predicate %a@]@\n" idents id;
-  fprintf fmt "@[<hov 2>\"%a\" " idents id;
+  fprintf fmt "@[<hov 2>$hyp \"%a\" " idents id;
   List.iter (fun (x,_) -> fprintf fmt "(A. ((%a)@ " ident x) bl;
   fprintf fmt "(<=> (%a %a)@ %a)" 
     idents id
@@ -298,7 +296,7 @@ let print_predicate_def fmt id (bl,p) =
 
 let print_function_def fmt id (bl,t,e) =
   fprintf fmt "@[;; Why function %a@]@\n" idents id;
-  fprintf fmt "@[<hov 2>\"%a\" " idents id;
+  fprintf fmt "@[<hov 2>$hyp \"%a\" " idents id;
   List.iter (fun (x,_) -> fprintf fmt "(A. ((%a)@ " ident x) bl;
   fprintf fmt "(= (%a %a)@ %a)" 
     idents id
@@ -309,7 +307,7 @@ let print_function_def fmt id (bl,t,e) =
 
 let print_axiom fmt id p =
   fprintf fmt "@[;; Why axiom %s@]@\n" id;
-  fprintf fmt "@[<hov 2>\"%s\" %a@]@\n@\n" id print_predicate p
+  fprintf fmt "@[<hov 2>$hyp \"%s\" %a@]@\n@\n" id print_predicate p
 
 let print_obligation fmt loc expl o s = 
   fprintf fmt "@[;; %s, %a@]@\n" o Loc.gen_report_line loc;
@@ -324,8 +322,14 @@ let reset () = Encoding.reset (*Monomorph.reset*) ()
 let output_elem fmt = function
   | Dtype (loc, _, id) -> declare_type fmt id
   | Dlogic (loc, id, t) -> print_logic fmt id t.scheme_type
-  | Dpredicate_def (loc, id, d) -> print_predicate_def fmt id d.scheme_type
-  | Dfunction_def (loc, id, d) -> print_function_def fmt id d.scheme_type
+  | Dpredicate_def (loc, id, d) -> 
+      let id = Ident.string id in
+      print_predicate_def fmt id d.scheme_type
+  | Dinductive_def _ ->
+      failwith "Zenon output: inductive def not yet supported"
+  | Dfunction_def (loc, id, d) -> 
+      let id = Ident.string id in
+      print_function_def fmt id d.scheme_type
   | Daxiom (loc, id, p) -> print_axiom fmt id p.scheme_type
   | Dgoal (loc, expl, id, s) -> 
       print_obligation fmt loc expl id s.Env.scheme_type
@@ -337,11 +341,11 @@ let prelude fmt =
     fprintf fmt "
 ;; Zenon prelude
 
-\"why__prelude_1\" ; x+0=x
+$hyp \"why__prelude_1\" ; x+0=x
    (A. ((x \"INT\") (= (why__add_int x why__int_const_0) x)))
-\"why__prelude_2\" ; 0+x=x
+$hyp \"why__prelude_2\" ; 0+x=x
    (A. ((x \"INT\") (= (why__add_int why__int_const_0 x) x)))
-\"why__prelude_3\" ; x+y=y+x
+$hyp \"why__prelude_3\" ; x+y=y+x
    (A. ((x \"INT\") (A. ((y \"INT\") 
      (= (why__add_int x y) (why__add_int y x))))))
 
