@@ -15,19 +15,17 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
-(*i $Id: dp.ml,v 1.40 2008/02/05 12:10:50 marche Exp $ i*)
+(*i $Id: dp.ml,v 1.50 2008-11-12 16:31:50 moy Exp $ i*)
 
 (* script to call automatic provers *)
 
@@ -39,10 +37,13 @@ let eclauses = ref 2000 (* E prover max nb of clauses *)
 let debug = ref false
 let batch = ref false
 let timings = ref true (* print timings *)
+let basename = ref false
+let listing = ref false
+let select_hypotheses = ref false
 let files = Queue.create ()
 
 type smt_solver = Yices | CVC3 | Z3
-let smt_solver = ref Yices
+let smt_solver = ref Z3
 let set_smt_solver = function
   | "yices" -> smt_solver := Yices
   | "cvc3" -> smt_solver := CVC3
@@ -57,11 +58,34 @@ let spec =
     "-batch", Arg.Set batch, "run in batch mode";
     "-no-timings", Arg.Clear timings, "do not display timings";
     "-smt-solver", Arg.String set_smt_solver, "<solver>";
+    "-basename", Arg.Set basename, "prints only file basenames";
+    "-listing", Arg.Set listing, "argument file only lists real argument files";
+    "-select", Arg.Set select_hypotheses, 
+    "applies some selection of hypotheses (only Alt-Ergo)";
   ]
 
-let usage = "usage: dp [options] files.{why,rv,znn,cvc,cvc.all,sx,sx.all,smt,smt.all}"
-let () = Arg.parse spec (fun s -> Queue.push s files) usage 
+let () = 
+  let d = Filename.dirname Sys.argv.(0) in
+  if not (Filename.is_relative d) then
+    Calldp.cpulimit := Filename.concat d "why-cpulimit"
 
+let usage = "usage: why-dp [options] files.{why,rv,znn,cvc,cvc.all,sx,sx.all,smt,smt.all}"
+let () = 
+  Arg.parse spec 
+    (fun s -> 
+       if !listing then
+	 let c = open_in s in
+	 let rec push_file () = 
+	   let line = input_line c in
+	   let flist = Str.split (Str.regexp "[ \t]+") line in
+	   List.iter (fun f -> Queue.push f files) flist;
+	   push_file ()
+	 in
+	 (try push_file () with End_of_file -> ());
+	 close_in c
+       else
+	 Queue.push s files) usage 
+    
 let () = 
   Cvcl_split.debug := !debug; 
   Simplify_split.debug := !debug;
@@ -128,7 +152,8 @@ let wrapper r =
   flush stdout
 
 let call_ergo f = 
-  wrapper (Calldp.ergo ~debug:!debug ~timeout:!timeout ~filename:f ())
+  wrapper (Calldp.ergo ~debug:!debug ~timeout:!timeout 
+	     ~select_hypotheses:!select_hypotheses ~filename:f ())
 let call_cvcl f = 
   wrapper (Calldp.cvcl ~debug:!debug ~timeout:!timeout ~filename:f ())
 let call_simplify f = 
@@ -152,7 +177,8 @@ let call_smt_solver = match !smt_solver with
   | Z3 -> call_z3
 
 let split f =
-  if not !batch then printf "%-30s: " f;
+  if not !batch then printf "%-30s: " 
+    (if !basename then Filename.basename f else f);
   let oldv = !nvalid in
   let oldi = !ninvalid in
   let oldt = !ntimeout in
@@ -199,7 +225,15 @@ let print_time fmt f =
       let m = m mod 60 in
       fprintf fmt "%d h %02d m %02d sec" h m s  
 
+
 let main () = 
+  begin
+    try
+      DpConfig.load_rc_file ()
+    with Not_found ->
+      print_endline "Why config file not found, please run why-config first.";
+      exit 1
+  end;
   if Queue.is_empty files then begin Arg.usage spec usage; exit 1 end;
   let wctime0 = Unix.gettimeofday() in
   if not !batch then printf "(. = valid * = invalid ? = unknown # = timeout ! = failure)@."; 
@@ -241,8 +275,6 @@ unknown VCs:
     !tmaxinvalid
       (!tunknown /. float !nunknown)
     !tmaxunknown;
-
-
   try Sys.remove "out" with _ -> ()
-
+    
 let () = Printexc.catch main ()
