@@ -15,16 +15,14 @@
 (*    Xavier URBAIN                                                       *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
-(*  modify it under the terms of the GNU General Public                   *)
-(*  License version 2, as published by the Free Software Foundation.      *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
 (*                                                                        *)
 (*  This software is distributed in the hope that it will be useful,      *)
 (*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
-(*  See the GNU General Public License version 2 for more details         *)
-(*  (enclosed in the file GPL).                                           *)
-(*                                                                        *)
 (**************************************************************************)
 
 (* Why code obfuscator *)
@@ -341,6 +339,12 @@ let rec program m fmt p =
       assert false
   | Sif (e1, e2, e3) ->
       fprintf fmt "(if %a then@ %a else@ %a)" progm e1 progm e2 progm e3
+  | Slazy_and (e1, e2) ->
+      fprintf fmt "(%a &&@ %a)" progm e1 progm e2
+  | Slazy_or (e1, e2) ->
+      fprintf fmt "(%a ||@ %a)" progm e1 progm e2
+  | Snot e1 ->
+      fprintf fmt "(not %a)" progm e1
   | Sapp ({pdesc = Sapp ({ pdesc = Svar id }, e1)}, e2) when is_binop id ->
       fprintf fmt "(%a %s %a)" progm e1 (binop id) progm e2
   | Sapp ({ pdesc = Svar id }, e1) when id == t_neg ->
@@ -445,6 +449,14 @@ let decl fmt = function
       let m = List.fold_left rename M.empty (List.map (fun (_,x,_) -> x) bl) in
       fprintf fmt "@[<hov 2>predicate %a(%a) =@ %a@]" gident id 
 	(print_list comma (logic_binder m)) bl (lexpr m) p
+  | Inductive_def(_,id, bl, l) ->
+      rename_global id;
+      fprintf fmt "@[<hov 2>predicate %a: @[%a@] {@\n  @[<v 0>%a@]@\n}@\n@]" 
+	gident id 
+	logic_type bl 
+	(print_list newline 
+	   (fun fmt (_,id,p) -> 
+	      fprintf fmt "%a: %a;" gident id (lexpr M.empty) p)) l
   | Function_def (_, id, bl, pt, e) ->
       rename_global id;
       let m = List.fold_left rename M.empty (List.map (fun (_,x,_) -> x) bl) in
