@@ -1,3 +1,30 @@
+(**************************************************************************)
+(*                                                                        *)
+(*  The Why platform for program certification                            *)
+(*  Copyright (C) 2002-2008                                               *)
+(*    Romain BARDOU                                                       *)
+(*    Jean-Franois COUCHOT                                               *)
+(*    Mehdi DOGGUY                                                        *)
+(*    Jean-Christophe FILLITRE                                           *)
+(*    Thierry HUBERT                                                      *)
+(*    Claude MARCH                                                       *)
+(*    Yannick MOY                                                         *)
+(*    Christine PAULIN                                                    *)
+(*    Yann RGIS-GIANAS                                                   *)
+(*    Nicolas ROUSSET                                                     *)
+(*    Xavier URBAIN                                                       *)
+(*                                                                        *)
+(*  This software is free software; you can redistribute it and/or        *)
+(*  modify it under the terms of the GNU Library General Public           *)
+(*  License version 2, with the special exception on linking              *)
+(*  described in file LICENSE.                                            *)
+(*                                                                        *)
+(*  This software is distributed in the hope that it will be useful,      *)
+(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
+(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
+(*                                                                        *)
+(**************************************************************************)
+
 (** the options to launch the toplevel with if the test file is not
      annotated with test options *)
 let default_options = [ "" ]
@@ -40,7 +67,7 @@ let behavior = ref Run
 let verbosity = ref 0
 let use_byte = ref false
 let do_diffs = ref "diff -u"
-let n = ref 4    (* the level of parallelism *)
+let n = ref 2    (* the level of parallelism *)
 let suites = ref []
 (** options given to toplevel for all tests *)
 let additional_options = ref ""
@@ -81,10 +108,10 @@ let () = Arg.parse
     "-add-options", Arg.Set_string additional_options,
     "add additional options to be passed to the toplevels \
      that will be launched";
-    "", Arg.Unit (fun () -> ()) ,"\nA test suite can be the name of a directory in ./tests or the path to a file.\n\nExamples:\nptests\nptests -diff \"echo diff\" -examine     # see again the list of tests that failed\nptests misc                           # for a single test suite\nptests tests/misc/alias.c             # for a single test\nptests -examine tests/misc/alias.c    # to see the differences again\nptests -v -j 1                        # to check the time taken by each test\n"
+    "", Arg.Unit (fun () -> ()) ,"\nA test suite can be the name of a directory in ./tests or the path to a file.\n\nExamples:\nregtest\nregtest -diff \"echo diff\" -examine     # see again the list of tests that failed\nregtest misc                           # for a single test suite\nregtest tests/misc/alias.c             # for a single test\nregtest -examine tests/misc/alias.c    # to see the differences again\nregtest -v -j 1                        # to check the time taken by each test\n"
   ]
   make_test_suite
-  "usage: ptests [options] [names of test suites]"
+  "usage: ./regtest.opt [options] [names of test suites]"
 
 (* redefine config file if special configuration expected *)
 let dir_config_file = 
@@ -112,7 +139,7 @@ let make_result_file = gen_make_file result_dirname
 let make_oracle_file = gen_make_file oracle_dirname
 
 let toplevel_path =
-  make_toplevel_path (gen_make_file "bin" base_path "krakatoa.opt")
+  make_toplevel_path (gen_make_file "tests" base_path "regtest.sh")
 
 type execnow =
     {
@@ -149,7 +176,7 @@ let launch command_string =
   let result = Unix.system command_string in
   match result with
   | Unix.WEXITED 127 ->
-      lock_printf "%% Couldn't execute command. Retrying once.@.";
+      lock_printf "%% Couldn't execute command `%s'@.Retrying once.@." command_string;
       Thread.delay 0.1;
       ( match Unix.system command_string with
 	Unix.WEXITED r when r <> 127 -> r
@@ -370,13 +397,13 @@ let update_toplevel_command command =
   let log_prefix = log_prefix command in
   let oracle_prefix = oracle_prefix command in
   let command_string =
-    "mv " ^
+    "cp " ^
       log_prefix ^ ".res.log " ^
       oracle_prefix ^ ".res.oracle"
   in
   ignore (launch command_string);
  let command_string =
-    "mv " ^
+    "cp " ^
       log_prefix ^ ".err.log " ^
       oracle_prefix ^ ".err.oracle"
   in
@@ -388,7 +415,7 @@ let update_command = function
 
 let update_log_files dir file =
   let command_string =
-    "mv " ^ make_result_file dir file ^ " " ^ make_oracle_file dir file
+    "cp " ^ make_result_file dir file ^ " " ^ make_oracle_file dir file
   in
   ignore (launch command_string)
 
@@ -772,6 +799,6 @@ let () =
 
 (*
 Local Variables: 
-compile-command: "make -j -C .. bin/ptests.byte"
+compile-command: "make -j -C .. bin/regtest.byte"
 End: 
 *)
